# Начальная версия основана на аналогичном модуле из пакета PolyORB.

dnl ##########################################################################
dnl Usage: TS_AC_PROG_ADA
dnl Try to compile a simple Ada program to test the compiler installation

AC_DEFUN([TS_AC_PROG_ADA],
[AC_REQUIRE([_TS_AM_PROG_ADA])
AC_REQUIRE([TS_AC_PROG_GNATCHOP])
AC_MSG_CHECKING([if the Ada compiler works])
_TS_AM_TRY_ADA([$ADA -c],[check.adb],
[procedure Check is
begin
   null;
end Check;
], [], [AC_MSG_RESULT(yes)],
[AC_MSG_RESULT(no)
AC_MSG_ERROR([Ada compiler is not working])])
AC_SUBST(ADA)])

dnl ##########################################################################
dnl Usage: TS_AC_PROG_GNATCHOP
dnl Look for GNATCHOP program

AC_DEFUN([TS_AC_PROG_GNATCHOP],
[AC_CHECK_PROG(GNATCHOP, gnatchop, gnatchop)])

dnl ##########################################################################
dnl Внутренние макроопределения.
dnl ##########################################################################

dnl ##########################################################################
dnl Usage: _TS_AM_PROG_ADA
dnl Look for an Ada compiler (ADA environment variable, then gcc, then $CC)

AC_DEFUN([_TS_AM_PROG_ADA],
[AC_BEFORE([$0], [_TS_AM_TRY_ADA])
AC_REQUIRE([AC_PROG_CC])
AC_CHECK_PROGS(ADA, gnatgcc adagcc gcc)
if test -z "$ADA"; then
  AC_MSG_RESULT([  Tentatively using $CC as an Ada compiler])
  ADA="$CC"
fi])

dnl ##########################################################################
dnl Usage: _TS_AM_TRY_ADA(gnatmake, filename, content, pragmas, success,
dnl                       failure)
dnl Compile, bind and link an Ada program and report its success or failure

AC_DEFUN([_TS_AM_TRY_ADA],
[mkdir conftest
cat > conftest/src.ada <<EOF
[$3]
EOF
cat > conftest/gnat.adc <<EOF
[$4]
EOF
ac_try="cd conftest && $GNATCHOP -q src.ada && $1 $2 > /dev/null 2>../conftest.out"
if AC_TRY_EVAL(ac_try); then
  ifelse([$5], , :, [rm -rf conftest*
  $5])
else
  ifelse([$6], , :, [ rm -rf conftest*
  $6])
fi
rm -f conftest*])

dnl ##########################################################################
dnl Usage: QA_AC_PROG_GNAT
dnl Look for GNAT program

AC_DEFUN([QA_AC_PROG_GNAT],
[AC_CHECK_PROG(GNAT, gnat, gnat)])

dnl ##########################################################################
dnl Usage: QA_AC_PROG_GNATMAKE
dnl Look for GNATMAKE program

AC_DEFUN([QA_AC_PROG_GNATMAKE],
[AC_CHECK_PROG(GNATMAKE, gnatmake, gnatmake)])

dnl ##########################################################################
dnl Usage: QA_AM_HAS_GNAT_PROJECT(project)
dnl dnl Check whether a given project file is available, and set
dnl dnl HAVE_GNAT_PROJECT_<project> to "yes" or "no" accordingly.
dnl
AC_DEFUN([QA_AM_HAS_GNAT_PROJECT],
[AC_REQUIRE([QA_AC_PROG_GNAT])
AC_MSG_CHECKING([whether GNAT project $1.gpr is available])
mkdir conftest
cat > conftest/check.gpr <<EOF
with "[$1]";
project Check is for Source_Files use (); end Check;
EOF
if AC_TRY_COMMAND([cd conftest && $GNAT ls -Pcheck system.ads > /dev/null 2>../conftest.out])
then
HAVE_GNAT_PROJECT_$1=yes
else
HAVE_GNAT_PROJECT_$1=no
fi
AC_MSG_RESULT($HAVE_GNAT_PROJECT_$1)
AC_SUBST(HAVE_GNAT_PROJECT_$1)
rm -fr conftest])
