//--------------------------------------------------------------------------//
//                                                                          //
//                An Ada2005 binding to the Qt C++ framework                //
//                                                                          //
//--------------------------------------------------------------------------//
//                                                                          //
// Copyright © 2007-2011 Vadim Godunko <vgodunko@gmail.com>                 //
//                                                                          //
// QtAda is free software;  you can  redistribute it and/or modify it under //
// terms of the  GNU General Public License as published  by the Free Soft- //
// ware  Foundation;  either version 2,  or (at your option) any later ver- //
// sion. QtAda is distributed in the hope that it will be useful, but WITH- //
// OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY //
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License //
// for  more details.  You should have  received  a copy of the GNU General //
// Public License distributed with QtAda;  see file COPYING.  If not, write //
// to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, //
// Boston, MA 02110-1301, USA.                                              //
//                                                                          //
// As a special exception,  if  other files  instantiate  templates  or use //
// macros or inline functions from this file,  or you compile this file and //
// link it with other files to produce an executable, this file does not by //
// itself cause  the resulting executable to be covered by  the GNU General //
// Public License.  This exception  does not however  invalidate any  other //
// reasons  why  the executable file  might be covered  by  the GNU General //
// Public License.                                                          //
//                                                                          //
//--------------------------------------------------------------------------//
//  $Revision: 3863 $ $Date: 2012-04-08 09:49:28 +0400 (Sun, 08 Apr 2012) $
//--------------------------------------------------------------------------//
#include <new>
#include <QByteArray>
#include <QChar>
#include <QString>
#include <QStringList>
#include <QVector>
#include "qstringwrapper.h"

extern "C"
{

    //  XXX For integration with old code only.
    QString* __Qt4Ada_copy_QString (const QString* cpp_object)
    {
        return new QString (*cpp_object);
    }

    QString* __QtAda_inititialize_QString(void* storage)
    {
        return new (storage) QString();
    }

    QString* __QtAda_inititialize_QString_QChar(void* storage, QChar* c)
    {
        return new (storage) QString(*c);
    }

    QString* __QtAda_adjust_QString(void* storage, const QString* from)
    {
        return new (storage) QString(*from);
    }

    void __QtAda_finalize_QString(QString *item)
    {
        item->~QString();
    }

    void __QtAda_QString_append_QStringCR(QString* self, const QString* s)
    {
        self->append(*s);
    }

    void __QtAda_QString_append_QChar(QString* self, QChar* c)
    {
        self->append(*c);
    }

    void __QtAda_QString_at_int_Q(QChar* returns, const QString* self, int i)
    {
        *returns = self->at(i);
    }

    int __QtAda_QString_capacity_Q(const QString* self)
    {
        return self->capacity();
    }

    void __QtAda_QString_chop_int(QString* self, int n)
    {
        self->chop(n);
    }

    void __QtAda_QString_clear(QString* self)
    {
        self->clear();
    }

    int __QtAda_QString_compare_QStringCR_Q(const QString* self, const QString* s)
    {
        return self->compare(*s);
    }

    int __QtAda_QString_compare_QStringCR_CaseSensitivity_Q(const QString* self, const QString* s, unsigned cs)
    {
        return self->compare(*s, (Qt::CaseSensitivity)cs);
    }

    unsigned __QtAda_QString_contains_QStringCR_CaseSensitivity_Q(const QString* self, const QString* s, unsigned cs)
    {
        return (unsigned)(self->contains(*s, (Qt::CaseSensitivity)cs) == true);
    }

    unsigned __QtAda_QString_contains_QChar_CaseSensitivity_Q(const QString* self, QChar* c, unsigned cs)
    {
        return (unsigned)(self->contains(*c, (Qt::CaseSensitivity)cs) == true);
    }

    unsigned __QtAda_QString_contains_QRegExpCR_Q(const QString* self, const QRegExp* s)
    {
        return (unsigned)(self->contains(*s) == true);
    }

    int __QtAda_QString_count_QStringCR_CaseSensitivity_Q(const QString* self, const QString* s, unsigned cs)
    {
        return self->count(*s, (Qt::CaseSensitivity)cs);
    }

    int __QtAda_QString_count_QChar_CaseSensitivity_Q(const QString* self, QChar* c, unsigned cs)
    {
        return self->count(*c, (Qt::CaseSensitivity)cs);
    }

    int __QtAda_QString_count_QRegExpCR_Q(const QString* self, const QRegExp* s)
    {
        return self->count(*s);
    }

    int __QtAda_QString_count_Q(const QString* self)
    {
        return self->count();
    }

    unsigned __QtAda_QString_endsWith_QStringCR_CaseSensitivity_Q(const QString* self, const QString* s, unsigned cs)
    {
        return (unsigned)self->endsWith(*s, (Qt::CaseSensitivity)cs);
    }

    unsigned __QtAda_QString_endsWith_QCharCR_CaseSensitivity_Q(const QString* self, const QChar* c, unsigned cs)
    {
        return (unsigned)self->endsWith(*c, (Qt::CaseSensitivity)cs);
    }

    void __QtAda_QString_fill_QChar_int(QString* self, QChar* c, int size)
    {
        self->fill(*c, size);
    }

    int __QtAda_QString_indexOf_QStringCR_int_CaseSensitivity_Q(const QString* self, const QString* s, int from, unsigned cs)
    {
        return self->indexOf(*s, from, (Qt::CaseSensitivity)cs);
    }

    int __QtAda_QString_indexOf_QChar_int_CaseSensitivity_Q(const QString* self, QChar* c, int from, unsigned cs)
    {
        return self->indexOf(*c, from, (Qt::CaseSensitivity)cs);
    }

    int __QtAda_QString_indexOf_QRegExpCR_int_Q(const QString* self, const QRegExp* rx, int from)
    {
        return self->indexOf(*rx, from);
    }

    void __QtAda_QString_insert_int_QStringCR(QString* self, int i, const QString* s)
    {
        self->insert(i, *s);
    }

    void __QtAda_QString_insert_int_QChar(QString* self, int i, QChar* c)
    {
        self->insert(i, *c);
    }

    unsigned __QtAda_QString_isEmpty_Q(const QString* self)
    {
        return (unsigned)self->isEmpty();
    }

    unsigned __QtAda_QString_isNull_Q(const QString* self)
    {
        return (unsigned)self->isNull();
    }

    int __QtAda_QString_lastIndexOf_QStringCR_int_CaseSensitivity_Q(const QString* self, const QString* s, int from, unsigned cs)
    {
        return self->lastIndexOf(*s, from, (Qt::CaseSensitivity)cs);
    }

    int __QtAda_QString_lastIndexOf_QChar_int_CaseSensitivity_Q(const QString* self, QChar* c, int from, unsigned cs)
    {
        return self->lastIndexOf(*c, from, (Qt::CaseSensitivity)cs);
    }

    int __QtAda_QString_lastIndexOf_QRegExpCR_int_Q(const QString* self, const QRegExp* rx, int from)
    {
        return self->lastIndexOf(*rx, from);
    }

    void __QtAda_QString_left_int_Q(QString* returns, const QString* self, int n)
    {
        *returns = self->left(n);
    }

    void __QtAda_QString_leftJustified_int_QChar_bool_Q(QString* returns, const QString* self, int width, QChar* fill, unsigned trunc)
    {
        *returns = self->leftJustified(width, *fill, (bool)trunc);
    }

    int __QtAda_QString_length_Q(const QString* self)
    {
        return self->length();
    }

    int __QtAda_QString_localeAwareCompare_QStringCR_Q(const QString* self, const QString* s)
    {
        return self->localeAwareCompare(*s);
    }

    void __QtAda_QString_mid_int_int_Q(QString* returns, const QString* self, int position, int n)
    {
        *returns = self->mid(position, n);
    }

    void __QtAda_QString_normalized_NormalizationForm_Q(QString* returns, const QString* self, unsigned mode)
    {
        *returns = self->normalized((QString::NormalizationForm)mode);
    }

    void __QtAda_QString_normalized_NormalizationForm_UnicodeVersion_Q(QString* returns, const QString* self, unsigned mode, unsigned version)
    {
        *returns = self->normalized((QString::NormalizationForm)mode, (QChar::UnicodeVersion)version);
    }

    void __QtAda_QString_prepend_QStringCR(QString* self, const QString* s)
    {
        self->prepend(*s);
    }

    void __QtAda_QString_prepend_QChar(QString* self, QChar* c)
    {
        self->prepend(*c);
    }

    void __QtAda_QString_remove_int_int(QString* self, int i, int len)
    {
        self->remove(i, len);
    }

    void __QtAda_QString_remove_QStringCR_CaseSensitivity(QString* self, const QString* s, unsigned cs)
    {
        self->remove(*s, (Qt::CaseSensitivity)cs);
    }

    void __QtAda_QString_remove_QChar_CaseSensitivity(QString* self, QChar* c, unsigned cs)
    {
        self->remove(*c, (Qt::CaseSensitivity)cs);
    }

    void __QtAda_QString_remove_QRegExpCR(QString* self, const QRegExp* s)
    {
        self->remove(*s);
    }

    void __QtAda_QString_replace_int_int_QStringCR(QString* self, int i, int len, const QString* after)
    {
        self->replace(i, len, *after);
    }

    void __QtAda_QString_replace_int_int_QChar(QString* self, int i, int len, QChar* after)
    {
        self->replace(i, len, *after);
    }

    void __QtAda_QString_replace_QStringCR_QStringCR_CaseSensitivity(QString* self, const QString* before, const QString* after, unsigned cs)
    {
        self->replace(*before, *after, (Qt::CaseSensitivity)cs);
    }

    void __QtAda_QString_replace_QChar_QStringCR_CaseSensitivity(QString* self, QChar* ch, const QString* after, unsigned cs)
    {
        self->replace(*ch, *after, (Qt::CaseSensitivity)cs);
    }

    void __QtAda_QString_replace_QChar_QChar_CaseSensitivity(QString* self, QChar* before, QChar* after, unsigned cs)
    {
        self->replace(*before, *after, (Qt::CaseSensitivity)cs);
    }

    void __QtAda_QString_replace_QRegExpCR_QStringCR(QString* self, const QRegExp* rx, const QString* after)
    {
        self->replace(*rx, *after);
    }

    void __QtAda_QString_reserve_int(QString* self, int size)
    {
        self->reserve(size);
    }

    void __QtAda_QString_resize_int(QString* self, int size)
    {
        self->resize(size);
    }

    void __QtAda_QString_right_int_Q(QString* returns, const QString* self, int n)
    {
        *returns = self->right(n);
    }

    void __QtAda_QString_rightJustified_int_QChar_bool_Q(QString* returns, const QString* self, int width, QChar* fill, unsigned trunc)
    {
        *returns = self->rightJustified(width, *fill, (bool)trunc);
    }

    void __QtAda_QString_simplified_Q(QString* returns, const QString* self)
    {
        *returns = self->simplified();
    }

    int __QtAda_QString_size_Q(const QString* self)
    {
        return self->size();
    }

    void __QtAda_QString_split_QStringCR_SplitBehavior_CaseSensitivity_Q(QStringList* returns, const QString* self, const QString* sep, unsigned behavior, unsigned cs)
    {
        *returns = self->split(*sep, (QString::SplitBehavior)behavior, (Qt::CaseSensitivity)cs);
    }

    void __QtAda_QString_split_QCharCR_SplitBehavior_CaseSensitivity_Q(QStringList* returns, const QString* self, const QChar* sep, unsigned behavior, unsigned cs)
    {
        *returns = self->split(*sep, (QString::SplitBehavior)behavior, (Qt::CaseSensitivity)cs);
    }

    void __QtAda_QString_split_QRegExpCR_SplitBehavior_Q(QStringList* returns, const QString* self, const QRegExp* sep, unsigned behavior)
    {
        *returns = self->split(*sep, (QString::SplitBehavior)behavior);
    }

    void __QtAda_QString_squeeze(QString* self)
    {
        self->squeeze();
    }

    unsigned __QtAda_QString_startsWith_QStringCR_CaseSensitivity_Q(const QString* self, const QString* s, unsigned cs)
    {
        return (unsigned)self->startsWith(*s, (Qt::CaseSensitivity)cs);
    }

    unsigned __QtAda_QString_startsWith_QCharCR_CaseSensitivity_Q(const QString* self, const QChar* s, unsigned cs)
    {
        return (unsigned)self->startsWith(*s, (Qt::CaseSensitivity)cs);
    }

    void __QtAda_QString_toCaseFolded_Q(QString* returns, const QString* self)
    {
        *returns = self->toCaseFolded();
    }

    void __QtAda_QString_toLatin1_Q(QByteArray* returns, const QString* self)
    {
        *returns = self->toLatin1();
    }

    void __QtAda_QString_toLocal8Bit_Q(QByteArray* returns, const QString* self)
    {
        *returns = self->toLocal8Bit();
    }

    void __QtAda_QString_toLower_Q(QString* returns, const QString* self)
    {
        *returns = self->toLower();
    }

    void __QtAda_QString_toUpper_Q(QString* returns, const QString* self)
    {
        *returns = self->toUpper();
    }

    void __QtAda_QString_toUtf8_Q(QByteArray* returns, const QString* self)
    {
        *returns = self->toUtf8();
    }

    void __QtAda_QString_trimmed_Q(QString* returns, const QString* self)
    {
        *returns = self->trimmed();
    }

    void __QtAda_QString_truncate_int(QString* self, int size)
    {
        self->truncate(size);
    }

    unsigned __QtAda_QString_eq_QStringCR_Q(const QString* self, const QString* s)
    {
        return (unsigned)self->operator==(*s);
    }

    unsigned __QtAda_QString_lt_QStringCR_Q(const QString* self, const QString* s)
    {
        return (unsigned)self->operator<(*s);
    }

    unsigned __QtAda_QString_gt_QStringCR_Q(const QString* self, const QString* s)
    {
        return (unsigned)self->operator>(*s);
    }

    unsigned __QtAda_QString_le_QStringCR_Q(const QString* self, const QString* s)
    {
        return (unsigned)self->operator<=(*s);
    }

    unsigned __QtAda_QString_ge_QStringCR_Q(const QString* self, const QString* s)
    {
        return (unsigned)self->operator>=(*s);
    }

    void __QtAda_QString_fromUtf8_S(QString* returns, const char* str)
    {
	*returns = QString::fromUtf8(str);
    }

    void __QtAda_QString_fromUtf16_S(QString* returns, const ushort* str)
    {
	*returns = QString::fromUtf16(str);
    }

    void __QtAda_QString_fromUcs4_S(QString* returns, const uint* str)
    {
	*returns = QString::fromUcs4(str);
    }

    void __QtAda_QString_fromLocal8Bit_S(QString* returns, const char* str)
    {
	*returns = QString::fromLocal8Bit(str);
    }

    void __QtAda_QString_toUtf8_Q_1(char* returns, QString* self)
    {
	strcpy(returns, self->toUtf8().constData());
    }

    void __QtAda_QString_utf16_Q(ushort* returns, QString* self)
    {
	const ushort* data = self->utf16();
	int last = 0;

	for(int j=0; data[j] != 0; j++)
	{
            returns[j] = data[j];
	    last = j;
	}

	returns[last+1] = 0;
    }

    void __QtAda_QString_toUcs4_Q(uint* returns, QString* self)
    {
	QVector<uint> data = self->toUcs4();

	for(int j=0; j < data.size(); j++)
	{
            returns[j] = data.at(j);
	}

	returns[data.size()] = 0;
    }

    void __QtAda_QString_toLocal8Bit_Q_1(char* returns, QString* self)
    {
	strcpy(returns, self->toLocal8Bit().constData());
    }

    void __QtAda_plus_QStringCR_QStringCR(QString* returns, QString* left, QString* right)
    {
	*returns = *left + *right;
    }

};

