------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------

package Qt4.Generic_Arguments is

   type Q_Generic_Argument (<>) is limited private;

   No_Argument : constant Q_Generic_Argument;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Name
   --!    <Purpose> Эквивалент функции QGenericArgument::name.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Name (Self : Q_Generic_Argument) return String;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Data
   --!    <Purpose> Эквивалент функции QGenericArgument::data.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Data (Self : Q_Generic_Argument) return System.Address;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> To_Argument
   --!    <Purpose> Эквивалент конструктора для шаблонного класса QArgument.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   generic
      type T (<>) is limited private;
      Type_Name : String;

   function To_Argument (Data : not null access T) return Q_Generic_Argument;

   package Internals is

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> Name
      --!    <Purpose> Эквивалент функции QGenericArgument::name для
      --! внутреннего использования.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function Name (Self : Q_Generic_Argument)
        return access constant Interfaces.C.char;

   end Internals;

private

   type Q_Generic_Argument (Last : Interfaces.C.size_t) is record
      Name : Interfaces.C.char_array (1 .. Last);
      Data : System.Address := System.Null_Address;
   end record;

   No_Argument : constant Q_Generic_Argument
     := Q_Generic_Argument'(Last => 0, others => <>);

end Qt4.Generic_Arguments;
