------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------
with System.Address_To_Access_Conversions;

package body Qt4.Generic_Return_Arguments is

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Data
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   function Data (Self : Q_Generic_Return_Argument) return System.Address is
   begin
      return Self.Data;
   end Data;

   ---------------
   -- Internals --
   ---------------

   package body Internals is

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> Name
      --!    <ImplementationNotes>
      ------------------------------------------------------------------------
      function Name (Self : Q_Generic_Return_Argument)
        return access constant Interfaces.C.char
      is
         use type Interfaces.C.size_t;

      begin
         if Self.Last = 0 then
            return null;

         else
            return Self.Name (Self.Name'First)'Unchecked_Access;
         end if;
      end Name;

   end Internals;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Name
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   function Name (Self : Q_Generic_Return_Argument) return String is
   begin
      return Interfaces.C.To_Ada (Self.Name);
   end Name;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> To_Argument
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   function To_Argument (Data : not null access T)
     return Q_Generic_Return_Argument
   is
      use type Interfaces.C.size_t;

      package Convertors is new System.Address_To_Access_Conversions (T);

   begin
      return
        Q_Generic_Return_Argument'
         (Last => Type_Name'Length + 1,
          Name => Interfaces.C.To_C (Type_Name),
          Data => Convertors.To_Address (Convertors.Object_Pointer (Data)));
   end To_Argument;

end Qt4.Generic_Return_Arguments;
