------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------
with Qt4_Entities;
with Qt4.Model_Indices;

package Qt4.Model_Index_Lists is

   package Thin is

      subtype QModelIndexList is Qt4_Entities.Cpp_Class;

      function copy_QModelIndexList
       (Cpp_Object : not null access QModelIndexList)
          return not null access QModelIndexList;
      pragma Import
       (C, copy_QModelIndexList, "__Qt4Ada_copy_QModelIndexList");

   end Thin;

   type Q_Model_Index_List is tagged private;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> =
   --!    <Purpose> Операция сравнения.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding function "="
    (Left  : Q_Model_Index_List;
     Right : Q_Model_Index_List) return Boolean;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Append
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   procedure Append (Self  : Q_Model_Index_List;
                     Value : Model_Indices.Q_Model_Index);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Size
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Item_At (Self : Q_Model_Index_List;
                     I    : Q_Integer)
      return Model_Indices.Q_Model_Index;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Size
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Size (Self  : Q_Model_Index_List) return Q_Integer;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Value
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Value (Self : Q_Model_Index_List;
                   I    : Q_Integer) return Model_Indices.Q_Model_Index;

   package Internals is

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> QModelIndexList_View
      --!    <Purpose> Возвращает ссылку на ассоциированный объект C++.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function QModelIndexList_View (Self : Q_Model_Index_List)
      return not null access Thin.QModelIndexList;

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> To_Q_Model_Index_List
      --!    <Purpose> Возвращает эквивалентный объект Ada.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function To_Q_Model_Index_List
      (Item : not null access constant Thin.QModelIndexList)
         return Q_Model_Index_List;

   end Internals;

private

   type Q_Model_Index_List is new Qt4_Entities.Entity with null record;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Copy_Cpp_Object
   --!    <Purpose> Осуществляет копирование ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Copy_Cpp_Object (Self : not null access Q_Model_Index_List)
   return not null access Qt4_Entities.Cpp_Class;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Delete_Cpp_Object
   --!    <Purpose> Осуществляет уничтожение ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   procedure Delete_Cpp_Object (Self : not null access Q_Model_Index_List);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Default_Cpp_Object
   --!    <Purpose> Возвращает новый пустой экземпляр класса QModelIndexList.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Default_Cpp_Object (Self : not null access Q_Model_Index_List)
   return access Qt4_Entities.Cpp_Class;

end Qt4.Model_Index_Lists;
