------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2010 Vadim Godunko <vgodunko@gmail.com>                      --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------
private with Ada.Finalization;
with Qt4.String_Lists;
with Qt4.Strings;
private with Qt_Ada.Internals.Value_Class_Sizes;
with System;

package Qt4.Process_Environments is

   type Q_Process_Environment is tagged private;

   procedure Clear
    (Self : in out Q_Process_Environment'Class);

   function Contains
    (Self : Q_Process_Environment'Class;
     Name : Qt4.Strings.Q_String'Class)
       return Boolean;

   procedure Insert
    (Self : in out Q_Process_Environment'Class;
     Name : Qt4.Strings.Q_String'Class;
     Value : Qt4.Strings.Q_String'Class);

   function Is_Empty
    (Self : Q_Process_Environment'Class)
       return Boolean;

   procedure Remove
    (Self : in out Q_Process_Environment'Class;
     Name : Qt4.Strings.Q_String'Class);

   function To_String_List
    (Self : Q_Process_Environment'Class)
       return Qt4.String_Lists.Q_String_List;

   function Value
    (Self : Q_Process_Environment'Class;
     Name : Qt4.Strings.Q_String'Class;
     Default_Value : Qt4.Strings.Q_String'Class)
       return Qt4.Strings.Q_String;

   overriding function "="
    (Self : Q_Process_Environment;
     S : Q_Process_Environment)
       return Boolean;

   function System_Environment
     return Q_Process_Environment;

   package Internals is

      type QProcessEnvironment is private;

      type QProcessEnvironment_Access is access all QProcessEnvironment;
      pragma Convention (C, QProcessEnvironment_Access);

      function QProcessEnvironment_View
       (Item : Q_Process_Environment'Class)
          return QProcessEnvironment_Access;
      pragma Inline (QProcessEnvironment_View);

      function To_Pointer
       (Address : System.Address)
          return QProcessEnvironment_Access;
      pragma Inline (To_Pointer);

      function To_Address
       (Pointer : QProcessEnvironment_Access)
          return System.Address;
      pragma Inline (To_Address);

      function Create_Wrapper
       (Item : Internals.QProcessEnvironment_Access)
          return Q_Process_Environment;

   private

      type QProcessEnvironment is record
         Placeholder : Interfaces.C.char_array (1 .. Qt_Ada.Internals.Value_Class_Sizes.QProcessEnvironment_C_Size_Of);
      end record;
      pragma Convention (C, QProcessEnvironment);

   end Internals;

private

   type Q_Process_Environment is new Ada.Finalization.Controlled with record
      Storage : aliased Internals.QProcessEnvironment;
      Object  : Internals.QProcessEnvironment_Access := null;
      Wrapper : Boolean := False;
   end record;

   overriding
   procedure Adjust (Self : in out Q_Process_Environment);

   overriding
   procedure Finalize (Self : in out Q_Process_Environment);

   overriding
   procedure Initialize (Self : in out Q_Process_Environment);

end Qt4.Process_Environments;
