------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008, 2009 Vadim Godunko <vgodunko@gmail.com>                --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3134 $ $Date: 2009-02-23 10:44:53 +0300 (Mon, 23 Feb 2009) $
------------------------------------------------------------------------------
with Ada.Command_Line;
with Interfaces.C;
with System;

with Qt4_Debug;
with Qt4.Core_Applications;

package body Qt_Ada.Core_Application is

   procedure new_QtAdaCoreApplication;
   pragma Import
           (C, new_QtAdaCoreApplication, "__QtAda_new_QtAdaCoreApplication");

   procedure delete_QCoreApplication;
   pragma Import
           (C, delete_QCoreApplication, "__QtAda_delete_QCoreApplication");

   function cpp_notify
    (Self     : System.Address;
     Receiver : System.Address;
     Event    : System.Address)
       return Interfaces.C.int;
   pragma Import (C, cpp_notify, "__QtAdaCoreApplication_cpp_notify");

   function ada_notify
    (Self     : System.Address;
     Receiver : System.Address;
     Event    : System.Address)
       return Interfaces.C.int;
   pragma Export (C, ada_notify, "__QtAdaCoreApplication_ada_notify");

   ----------------
   -- ada_notify --
   ----------------

   function ada_notify
    (Self     : System.Address;
     Receiver : System.Address;
     Event    : System.Address)
       return Interfaces.C.int
   is
   begin
      return cpp_notify (Self, Receiver, Event);

   exception
      when E : others =>
         Qt4_Debug.Ada_Exception_In_Event_Notification (E);

         return 1;
   end ada_notify;

   -------------
   -- Execute --
   -------------

   procedure Execute is
   begin
      Ada.Command_Line.Set_Exit_Status
       (Ada.Command_Line.Exit_Status (Qt4.Core_Applications.Exec));
   end Execute;

   --------------
   -- Finalize --
   --------------

   procedure Finalize is
   begin
      delete_QCoreApplication;
   end Finalize;

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize is
   begin
      new_QtAdaCoreApplication;
   end Initialize;

end Qt_Ada.Core_Application;
