------------------------------------------------------------------------------
--                                                                          --
--                 An Ada2005 binding to the Qt C++ library                 --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2007-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
with Ada.Unchecked_Deallocation;
with System.Address_To_Access_Conversions;

package body Qt_Ada.Generic_Variants is

   package Conversions is
     new System.Address_To_Access_Conversions (Value_Type);

   -----------------
   -- Constructor --
   -----------------

   function Constructor (Item : Value_Type_Access)
     return not null Value_Type_Access
   is
   begin
      if Item = null then
         return new Value_Type;

      else
         return new Value_Type'(Item.all);
      end if;
   end Constructor;

   ------------
   -- Create --
   ------------

   function Create (Item : Value_Type) return Qt4.Variants.Q_Variant is
      Aux : aliased Value_Type := Item;

   begin
      return
        Qt4.Variants.Internals.Create
         (Type_Id, Conversions.To_Address (Aux'Unchecked_Access));
   end Create;

   ----------------
   -- Destructor --
   ----------------

   procedure Destructor (Item : Value_Type_Access) is

      procedure Free is
        new Ada.Unchecked_Deallocation (Value_Type, Value_Type_Access);

      Aux : Value_Type_Access := Item;

   begin
      Free (Aux);
   end Destructor;

   --------------
   -- To_Value --
   --------------

   function To_Value (Variant : Qt4.Variants.Q_Variant'Class)
     return Value_Type
   is
      use type Qt4.Meta_Types.Meta_Type_Type;

      Result : Value_Type;

   begin
      if Variant.Get_User_Type = Type_Id then
         Result :=
           Conversions.To_Pointer
            (Qt4.Variants.Internals.Const_Data (Variant)).all;
      end if;

      return Result;
   end To_Value;

end Qt_Ada.Generic_Variants;
