------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3761 $ $Date: 2010-10-23 23:11:09 +0400 (Sat, 23 Oct 2010) $
------------------------------------------------------------------------------
with Ada.Unchecked_Deallocation;

package body Qt_Ada.Internals.Objects is

   use type QObject_Access;

   procedure associate
    (Cpp_Object : not null Qt_Ada.Internals.Objects.QObject_Access;
     Ada_Object : not null Qt_Ada.Internals.Objects.Q_Object_Base_Access);
   pragma Import (C, associate, "__QtAda_associate_Q_Object_Impl");
   pragma Warnings (Off, associate);

   procedure Free_Q_Object_Impl
    (Self : not null Qt_Ada.Internals.Objects.Q_Object_Base_Access);
   pragma Export (C, Free_Q_Object_Impl, "__QtAda_free_Q_Object_Impl");
   pragma Warnings (Off, Free_Q_Object_Impl);

   ------------------------
   -- Free_Q_Object_Impl --
   ------------------------

   procedure Free_Q_Object_Impl
    (Self : not null Qt_Ada.Internals.Objects.Q_Object_Base_Access) is
--  GNAT GPL 2010: internal cleanup subprogram is called before actual memory
--  deallocation, but call is dispatched incorrectly. To prevent crash GNAT
--  specific deallocation subprogram is called instead.
--      procedure Free is
--        new Ada.Unchecked_Deallocation
--             (Q_Object_Impl_Base'Class, Q_Object_Impl_Base_Access);

      procedure GNAT_Free (X : Q_Object_Impl_Base_Access);
      pragma Import (C, GNAT_Free, "__gnat_free");
      pragma Warnings (Off, GNAT_Free);

      Aux : Q_Object_Impl_Base_Access := Q_Object_Impl_Base_Access (Self);

   begin
      if Aux.Object /= null then
         Aux.Finalize;
         Aux.Object := null;
         GNAT_Free (Aux);
--         Free (Aux);
      end if;
   end Free_Q_Object_Impl;

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize
    (Self   : not null access Q_Object_Impl_Base'Class;
     Object : not null QObject_Access)
   is
   begin
      Self.Object := Object;
      associate (Object, Q_Object_Base_Access (Self));
   end Initialize;

   ------------------
   -- QObject_View --
   ------------------

   overriding
   function QObject_View (Self : not null access constant Q_Object_Impl_Base)
     return not null access QObject
   is
   begin
      return Self.Object;
   end QObject_View;

end Qt_Ada.Internals.Objects;
