------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3786 $ $Date: 2010-10-27 11:09:19 +0400 (Wed, 27 Oct 2010) $
------------------------------------------------------------------------------
with Qt_Ada.Internals.Events.Registry;

package body Qt4.Accessible_Events is

   function child_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
       return Interfaces.C.int;
   pragma Import (C, child_wrapper, "__Qt4Ada_QAccessiblEvent_child_wrapper");

   procedure setValue_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent;
     Text : not null access Qt4.Strings.Internals.QString);
   pragma Import
    (C, setValue_wrapper, "__Qt4Ada_QAccessiblEvent_setValue_wrapper");

   function value_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
       return not null access Qt4.Strings.Internals.QString;
   pragma Import (C, value_wrapper, "__Qt4Ada_QAccessiblEvent_value_wrapper");

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access;

   -----------
   -- Child --
   -----------

   function Child
    (Self : not null access constant Q_Accessible_Event'Class)
       return Q_Integer is
   begin
      return Q_Integer (child_wrapper (Self.QEvent_View));
   end Child;

   ------------
   -- Create --
   ------------

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access is
   begin
      return Result : constant not null Qt4.Events.Q_Event_Access
        := new Q_Accessible_Event
      do
         Qt_Ada.Internals.Events.Initialize (Result, Object);
      end return;
   end Create;

   ---------------
   -- Set_Value --
   ---------------

   procedure Set_Value
    (Self : not null access Q_Accessible_Event'Class;
     Text : Qt4.Strings.Q_String) is
   begin
      setValue_wrapper
       (Self.QEvent_View, Qt4.Strings.Internals.QString_View (Text));
   end Set_Value;

   -----------
   -- Value --
   -----------

   function Value
    (Self : not null access constant Q_Accessible_Event'Class)
       return Qt4.Strings.Q_String is
   begin
      return
        Qt4.Strings.Internals.Create_Wrapper
         (Qt4.Strings.Internals.QString_Access
           (value_wrapper (Self.QEvent_View)));
   end Value;

begin
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Accessibility_Help, Create'Access);
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Accessibility_Description, Create'Access);
end Qt4.Accessible_Events;
