------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3786 $ $Date: 2010-10-27 11:09:19 +0400 (Wed, 27 Oct 2010) $
------------------------------------------------------------------------------
with Qt4.Events;
with Qt_Ada.Internals.Events.Registry;

package body Qt4.Drag_Move_Events is

   procedure accept_with_rect_wrapper
    (Self      : not null access Qt_Ada.Internals.Events.QEvent;
     Rectangle : not null access Qt4.Rects.Internals.QRect);
   pragma Import (C,
                  accept_with_rect_wrapper,
                  "__Qt4Ada_QDragMoveEvent_accept_with_rect_wrapper");

   procedure accept_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent);
   pragma Import (C, accept_wrapper, "__Qt4Ada_QDragMoveEvent_accept_wrapper");

   procedure answerRect_wrapper
    (Self      : not null access Qt_Ada.Internals.Events.QEvent;
     Rectangle : not null access Qt4.Rects.Internals.QRect);
   pragma Import (C,
                  answerRect_wrapper,
                  "__Qt4Ada_QDragMoveEvent_answerRect_wrapper");

   procedure ignore_with_rect_wrapper
    (Self      : not null access Qt_Ada.Internals.Events.QEvent;
     Rectangle : not null access Qt4.Rects.Internals.QRect);
   pragma Import (C,
                  ignore_with_rect_wrapper,
                  "__Qt4Ada_QDragMoveEvent_ignore_with_rect_wrapper");

   procedure ignore_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent);
   pragma Import (C, ignore_wrapper, "__Qt4Ada_QDragMoveEvent_ignore_wrapper");

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access;

   ----------------------------
   -- Accept_Drag_Move_Event --
   ----------------------------

   procedure Accept_Drag_Move_Event
    (Self      : not null access Q_Drag_Move_Event'Class;
     Rectangle : Qt4.Rects.Q_Rect)
   is
   begin
      accept_with_rect_wrapper (Self.QEvent_View,
                                Qt4.Rects.Internals.QRect_View (Rectangle));
   end Accept_Drag_Move_Event;

   ----------------------------
   -- Accept_Drag_Move_Event --
   ----------------------------

   procedure Accept_Drag_Move_Event
    (Self : not null access Q_Drag_Move_Event'Class) is
   begin
      accept_wrapper (Self.QEvent_View);
   end Accept_Drag_Move_Event;

   -----------------
   -- Answer_Rect --
   -----------------

   function Answer_Rect
    (Self : not null access constant Q_Drag_Move_Event'Class)
       return Qt4.Rects.Q_Rect is
   begin
      return Result : Qt4.Rects.Q_Rect do
         answerRect_wrapper (Self.QEvent_View,
                             Qt4.Rects.Internals.QRect_View (Result));
      end return;
   end Answer_Rect;

   ------------
   -- Create --
   ------------

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access
   is
   begin
      return Result : constant not null Qt4.Events.Q_Event_Access
        := new Q_Drag_Move_Event
      do
         Qt_Ada.Internals.Events.Initialize (Result, Object);
      end return;
   end Create;

   ------------
   -- Ignore --
   ------------

   procedure Ignore
    (Self      : not null access Q_Drag_Move_Event'Class;
     Rectangle : Qt4.Rects.Q_Rect) is
   begin
      ignore_with_rect_wrapper
       (Self.QEvent_View, Qt4.Rects.Internals.QRect_View (Rectangle));
   end Ignore;

   ------------
   -- Ignore --
   ------------

   procedure Ignore (Self : not null access Q_Drag_Move_Event'Class) is
   begin
      ignore_wrapper (Self.QEvent_View);
   end Ignore;

begin
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Drag_Move, Create'Access);
end Qt4.Drag_Move_Events;
