------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3786 $ $Date: 2010-10-27 11:09:19 +0400 (Wed, 27 Oct 2010) $
------------------------------------------------------------------------------
with Qt_Ada.Internals.Object_Registry;
with Qt_Ada.Internals.Objects;
with Qt_Ada.Internals.Events.Registry;

package body Qt4.Drop_Events is

   procedure acceptProposedAction_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent);
   pragma Import (C,
                  acceptProposedAction_wrapper,
                  "__Qt4Ada_QDropEvent_acceptProposedAction_wrapper");

   function dropAction_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
     return Qt4.Drop_Action;
   pragma Import (C,
                  dropAction_wrapper,
                  "__Qt4Ada_QDropEvent_dropAction_wrapper");

   function keyboardModifiers_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
       return Qt4.Keyboard_Modifiers;
   pragma Import (C,
                  keyboardModifiers_wrapper,
                  "__Qt4Ada_QDropEvent_keyboardModifiers_wrapper");

   function mouseButtons_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
       return Qt4.Mouse_Buttons;
   pragma Import (C,
                  mouseButtons_wrapper,
                  "__Qt4Ada_QDropEvent_mouseButtons_wrapper");

   function pos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
       return not null access Qt4.Points.Internals.QPoint;
   pragma Import (C, pos_wrapper, "__Qt4Ada_QDropEvent_pos_wrapper");

   function possibleActions_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
       return Qt4.Drop_Actions;
   pragma Import (C,
                  possibleActions_wrapper,
                  "__Qt4Ada_QDropEvent_possibleActions_wrapper");

   function proposedAction_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
       return Qt4.Drop_Action;
   pragma Import (C,
                  proposedAction_wrapper,
                  "__Qt4Ada_QDropEvent_proposedAction_wrapper");

   procedure setDropAction_wrapper
    (Self   : not null access Qt_Ada.Internals.Events.QEvent;
     Action : Qt4.Drop_Action);
   pragma Import
    (C, setDropAction_wrapper, "__Qt4Ada_QDropEvent_setDropAction_wrapper");

   function source_wrapper
    (Self   : not null access Qt_Ada.Internals.Events.QEvent)
      return not null access Qt_Ada.Internals.Objects.QObject;
   pragma Import (C, source_wrapper, "__Qt4Ada_QDropEvent_source_wrapper");

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access;

   ----------------------------
   -- Accept_Proposed_Action --
   ----------------------------

   procedure Accept_Proposed_Action
    (Self : not null access Q_Drop_Event'Class) is
   begin
      acceptProposedAction_wrapper (Self.QEvent_View);
   end Accept_Proposed_Action;

   ------------
   -- Create --
   ------------

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access is
   begin
      return Result : constant not null Qt4.Events.Q_Event_Access
        := new Q_Drop_Event
      do
         Qt_Ada.Internals.Events.Initialize (Result, Object);
      end return;
   end Create;

   -----------------
   -- Drop_Action --
   -----------------

   function Drop_Action
    (Self : not null access constant Q_Drop_Event'Class)
       return Qt4.Drop_Action is
   begin
      return dropAction_wrapper (Self.QEvent_View);
   end Drop_Action;

   ------------------------
   -- Keyboard_Modifiers --
   ------------------------

   function Keyboard_Modifiers
    (Self : not null access constant Q_Drop_Event'Class)
       return Qt4.Keyboard_Modifiers is
   begin
      return keyboardModifiers_wrapper (Self.QEvent_View);
   end Keyboard_Modifiers;

   -------------------
   -- Mouse_Buttons --
   -------------------

   function Mouse_Buttons
    (Self : not null access constant Q_Drop_Event'Class)
       return Qt4.Mouse_Buttons is
   begin
      return mouseButtons_wrapper (Self.QEvent_View);
   end Mouse_Buttons;

   ---------
   -- Pos --
   ---------

   function Pos
    (Self : not null access constant Q_Drop_Event'Class)
       return Qt4.Points.Q_Point is
   begin
      return
        Qt4.Points.Internals.Create_Wrapper
         (Qt4.Points.Internals.QPoint_Access (pos_wrapper (Self.QEvent_View)));
   end Pos;

   ----------------------
   -- Possible_Actions --
   ----------------------

   function Possible_Actions
    (Self : not null access constant Q_Drop_Event'Class)
       return Qt4.Drop_Actions is
   begin
      return possibleActions_wrapper (Self.QEvent_View);
   end Possible_Actions;

   ---------------------
   -- Proposed_Action --
   ---------------------

   function Proposed_Action
    (Self : not null access constant Q_Drop_Event'Class)
       return Qt4.Drop_Action is
   begin
      return proposedAction_wrapper (Self.QEvent_View);
   end Proposed_Action;

   ---------------------
   -- Set_Drop_Action --
   ---------------------

   procedure Set_Drop_Action
    (Self   : not null access Q_Drop_Event'Class;
     Action : Qt4.Drop_Action) is
   begin
      setDropAction_wrapper (Self.QEvent_View, Action);
   end Set_Drop_Action;

   ------------
   -- Source --
   ------------

   function Source
    (Self : not null access constant Q_Drop_Event'Class)
       return not null access Qt4.Widgets.Q_Widget'Class is
   begin
      return Qt4.Widgets.Q_Widget'Class
               (Qt_Ada.Internals.Object_Registry.To_Q_Object
                 (source_wrapper (Self.QEvent_View)).all)'Access;
   end Source;

begin
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Drop, Create'Access);
end Qt4.Drop_Events;
