------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------
with Qt4_Entities;
with Qt4.Colors;

package Qt4.Gradients is

   package Thin is

      subtype QGradient is Qt4_Entities.Cpp_Class;

      function copy_QGradient (Cpp_Object : not null access QGradient)
        return not null access QGradient;
      pragma Import (C, copy_QGradient, "__Qt4Ada_copy_QGradient");

   end Thin;

   type Q_Gradient is new Qt4_Entities.Entity with null record;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Set_Color_At
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   procedure Set_Color_At (Self  : Q_Gradient;
                           Pos   : Q_Real;
                           Color : Qt4.Colors.Q_Color);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Copy_Cpp_Object
   --!    <Purpose> Осуществляет копирование ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Copy_Cpp_Object (Self : not null access Q_Gradient)
     return not null access Qt4_Entities.Cpp_Class;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Delete_Cpp_Object
   --!    <Purpose> Осуществляет уничтожение ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   procedure Delete_Cpp_Object (Self : not null access Q_Gradient);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Default_Cpp_Object
   --!    <Purpose> Возвращает новый пустой экземпляр класса QGradient.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Default_Cpp_Object (Self : not null access Q_Gradient)
     return access Qt4_Entities.Cpp_Class;

   package Internals is

      type Gradient_Type is new Interfaces.C.int range 0 .. 3;

      Linear_Gradient  : constant Gradient_Type := 0;
      Radial_Gradient  : constant Gradient_Type := 1;
      Conical_Gradient : constant Gradient_Type := 2;
      No_Gradient      : constant Gradient_Type := 3;

      type Ada_Object_Constructor is
        access function (The_Type : Gradient_Type)
          return Q_Gradient'Class;

      type QGradient_Access is access all Qt4_Entities.Cpp_Class;

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> QGradient_View
      --!    <Purpose> Возвращает ссылку на ассоциированный объект C++.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function QGradient_View (Self : Q_Gradient'Class)
        return not null QGradient_Access;

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> Create_Wrapper
      --!    <Purpose>
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function Create_Wrapper (Cpp_Object : not null access Thin.QGradient)
        return Q_Gradient'Class;

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> Register
      --!    <Purpose> Производит регистрацию подпрограммы создания Ada объекта
      --! для инкапсуляции подклассов QGradient. Этот механиз используется
      --! только для классов объявленных в C++. Для классов, описанных на Ada
      --! используется реестр экземпляров событий, заполняемый при
      --! инициализации обертки и освобождаемый деструктором Ada объекта.
      --!    <Exceptions>
      -------------------------------------------------------------------------
      procedure Register (The_Type    : Gradient_Type;
                          Constructor : Ada_Object_Constructor);

   end Internals;

end Qt4.Gradients;
