------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3786 $ $Date: 2010-10-27 11:09:19 +0400 (Wed, 27 Oct 2010) $
------------------------------------------------------------------------------
with Qt4.Events;
with Qt_Ada.Internals.Events.Registry;

package body Qt4.Graphics_Scene_Mouse_Events is

   function button_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
      return Qt4.Mouse_Button;
   pragma Import
    (C,
     button_wrapper,
     "__Qt4Ada_QGraphicsSceneMouseEvent_button_wrapper");

   procedure buttonDownPos_wrapper
    (Self   : not null access Qt_Ada.Internals.Events.QEvent;
     Button : Qt4.Mouse_Button;
     Pos    : not null access Qt4.Point_Fs.Internals.QPointF);
   pragma Import (C,
                  buttonDownPos_wrapper,
                  "__Qt4Ada_QGraphicsSceneMouseEvent_buttonDownPos_wrapper");

   procedure buttonDownScenePos_wrapper
    (Self   : not null access Qt_Ada.Internals.Events.QEvent;
     Button : Qt4.Mouse_Button;
     Pos    : not null access Qt4.Point_Fs.Internals.QPointF);
   pragma Import
    (C,
     buttonDownScenePos_wrapper,
     "__Qt4Ada_QGraphicsSceneMouseEvent_buttonDownScenePos_wrapper");

   function buttonDownScreenPos_wrapper
    (Self   : not null access Qt_Ada.Internals.Events.QEvent;
     Button : Qt4.Mouse_Button)
       return not null access Qt4.Points.Internals.QPoint;
   pragma Import
    (C,
     buttonDownScreenPos_wrapper,
     "__Qt4Ada_QGraphicsSceneMouseEvent_buttonDownScreenPos_wrapper");

   function buttons_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
      return Qt4.Mouse_Buttons;
   pragma Import
    (C,
     buttons_wrapper,
     "__Qt4Ada_QGraphicsSceneMouseEvent_buttons_wrapper");

   function modifiers_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
      return Qt4.Keyboard_Modifiers;
   pragma Import
    (C,
     modifiers_wrapper,
     "__Qt4Ada_QGraphicsSceneMouseEvent_modifiers_wrapper");

   procedure lastPos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent;
     Pos  : not null access Qt4.Point_Fs.Internals.QPointF);
   pragma Import
    (C, lastPos_wrapper, "__Qt4Ada_QGraphicsSceneMouseEvent_lastPos_wrapper");

   procedure lastScenePos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent;
     Pos  : not null access Qt4.Point_Fs.Internals.QPointF);
   pragma Import
    (C,
     lastScenePos_wrapper,
     "__Qt4Ada_QGraphicsSceneMouseEvent_lastScenePos_wrapper");

   function lastScreenPos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
       return not null access Qt4.Points.Internals.QPoint;
   pragma Import
    (C,
     lastScreenPos_wrapper,
     "__Qt4Ada_QGraphicsSceneMouseEvent_lastScreenPos_wrapper");

   procedure pos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent;
     Pos  : not null access Qt4.Point_Fs.Internals.QPointF);
   pragma Import
    (C, pos_wrapper, "__Qt4Ada_QGraphicsSceneMouseEvent_pos_wrapper");

   procedure scenePos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent;
     Pos  : not null access Qt4.Point_Fs.Internals.QPointF);
   pragma Import
    (C,
     scenePos_wrapper,
     "__Qt4Ada_QGraphicsSceneMouseEvent_scenePos_wrapper");

   function screenPos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
      return not null access Qt4.Points.Internals.QPoint;
   pragma Import
    (C,
     screenPos_wrapper,
     "__Qt4Ada_QGraphicsSceneMouseEvent_screenPos_wrapper");

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access;

   ------------
   -- Button --
   ------------

   function Button
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class)
       return Qt4.Mouse_Button is
   begin
      return button_wrapper (Self.QEvent_View);
   end Button;

   ---------------------
   -- Button_Down_Pos --
   ---------------------

   function Button_Down_Pos
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class;
     B    : Qt4.Mouse_Button)
       return Qt4.Point_Fs.Q_Point_F is
   begin
      return Result : Qt4.Point_Fs.Q_Point_F do
         buttonDownPos_wrapper
          (Self.QEvent_View, B, Qt4.Point_Fs.Internals.QPointF_View (Result));
      end return;
   end Button_Down_Pos;

   ---------------------------
   -- Button_Down_Scene_Pos --
   ---------------------------

   function Button_Down_Scene_Pos
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class;
     B    : Qt4.Mouse_Button)
       return Qt4.Point_Fs.Q_Point_F is
   begin
      return Result : Qt4.Point_Fs.Q_Point_F do
         buttonDownScenePos_wrapper
          (Self.QEvent_View, B, Qt4.Point_Fs.Internals.QPointF_View (Result));
      end return;
   end Button_Down_Scene_Pos;

   ----------------------------
   -- Button_Down_Screen_Pos --
   ----------------------------

   function Button_Down_Screen_Pos
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class;
     B    : Qt4.Mouse_Button)
       return Qt4.Points.Q_Point is
   begin
      return
        Qt4.Points.Internals.Create_Wrapper
         (Qt4.Points.Internals.QPoint_Access
           (buttonDownScreenPos_wrapper (Self.QEvent_View, B)));
   end Button_Down_Screen_Pos;

   -------------
   -- Buttons --
   -------------

   function Buttons
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class)
       return Qt4.Mouse_Buttons is
   begin
      return buttons_wrapper (Self.QEvent_View);
   end Buttons;

   ------------
   -- Create --
   ------------

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access is
   begin
      return Result : constant not null Qt4.Events.Q_Event_Access
        := new Q_Graphics_Scene_Mouse_Event
      do
         Qt_Ada.Internals.Events.Initialize (Result, Object);
      end return;
   end Create;

   --------------
   -- Last_Pos --
   --------------

   function Last_Pos
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class)
       return Qt4.Point_Fs.Q_Point_F is
   begin
      return Result : Qt4.Point_Fs.Q_Point_F do
         lastPos_wrapper
          (Self.QEvent_View, Qt4.Point_Fs.Internals.QPointF_View (Result));
      end return;
   end Last_Pos;

   --------------------
   -- Last_Scene_Pos --
   --------------------

   function Last_Scene_Pos
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class)
       return Qt4.Point_Fs.Q_Point_F is
   begin
      return Result : Qt4.Point_Fs.Q_Point_F do
         lastScenePos_wrapper
          (Self.QEvent_View, Qt4.Point_Fs.Internals.QPointF_View (Result));
      end return;
   end Last_Scene_Pos;

   ---------------------
   -- Last_Screen_Pos --
   ---------------------

   function Last_Screen_Pos
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class)
       return Qt4.Points.Q_Point is
   begin
      return
        Qt4.Points.Internals.Create_Wrapper
         (Qt4.Points.Internals.QPoint_Access
           (lastScreenPos_wrapper (Self.QEvent_View)));
   end Last_Screen_Pos;

   ---------------
   -- Modifiers --
   ---------------

   function Modifiers
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class)
       return Qt4.Keyboard_Modifiers is
   begin
      return modifiers_wrapper (Self.QEvent_View);
   end Modifiers;

   ---------
   -- Pos --
   ---------

   function Pos
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class)
       return Qt4.Point_Fs.Q_Point_F is
   begin
      return Result : Qt4.Point_Fs.Q_Point_F do
         pos_wrapper
          (Self.QEvent_View, Qt4.Point_Fs.Internals.QPointF_View (Result));
      end return;
   end Pos;

   ---------------
   -- Scene_Pos --
   ---------------

   function Scene_Pos
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class)
       return Qt4.Point_Fs.Q_Point_F is
   begin
      return Result : Qt4.Point_Fs.Q_Point_F do
         scenePos_wrapper
          (Self.QEvent_View, Qt4.Point_Fs.Internals.QPointF_View (Result));
      end return;
   end Scene_Pos;

   ----------------
   -- Screen_Pos --
   ----------------

   function Screen_Pos
    (Self : not null access constant Q_Graphics_Scene_Mouse_Event'Class)
       return Qt4.Points.Q_Point is
   begin
      return
        Qt4.Points.Internals.Create_Wrapper
         (Qt4.Points.Internals.QPoint_Access
           (screenPos_wrapper (Self.QEvent_View)));
   end Screen_Pos;

begin
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Graphics_Scene_Mouse_Double_Click, Create'Access);
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Graphics_Scene_Mouse_Move, Create'Access);
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Graphics_Scene_Mouse_Press, Create'Access);
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Graphics_Scene_Mouse_Release, Create'Access);
end Qt4.Graphics_Scene_Mouse_Events;
