------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3786 $ $Date: 2010-10-27 11:09:19 +0400 (Wed, 27 Oct 2010) $
------------------------------------------------------------------------------
with Qt4.Events;
with Qt_Ada.Internals.Events.Registry;

package body Qt4.Graphics_Scene_Wheel_Events is

   function buttons_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
      return Qt4.Mouse_Buttons;
   pragma Import
    (C,
     buttons_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_buttons_wrapper");

   function delta_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
      return Interfaces.C.int;
   pragma Import
    (C,
     delta_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_delta_wrapper");

   function modifiers_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
      return Qt4.Keyboard_Modifiers;
   pragma Import
    (C,
     modifiers_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_modifiers_wrapper");

   procedure pos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent;
     Pos  : not null access Qt4.Point_Fs.Internals.QPointF);
   pragma Import
    (C, pos_wrapper, "__Qt4Ada_QGraphicsSceneWheelEvent_pos_wrapper");

   procedure scenePos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent;
     Pos  : not null access Qt4.Point_Fs.Internals.QPointF);
   pragma Import
    (C,
     scenePos_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_scenePos_wrapper");

   function screenPos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent)
       return not null access Qt4.Points.Internals.QPoint;
   pragma Import
    (C,
     screenPos_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_screenPos_wrapper");

   procedure setButtons_wrapper
    (Self     : not null access Qt_Ada.Internals.Events.QEvent;
     Buttons  : Qt4.Mouse_Buttons);
   pragma Import
    (C,
     setButtons_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_setButtons_wrapper");

   procedure setDelta_wrapper
    (Self      : not null access Qt_Ada.Internals.Events.QEvent;
     The_Delta : Interfaces.C.int);
   pragma Import
    (C,
     setDelta_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_setDelta_wrapper");

   procedure setModifiers_wrapper
    (Self      : not null access Qt_Ada.Internals.Events.QEvent;
     The_Delta : Qt4.Keyboard_Modifiers);
   pragma Import
    (C,
     setModifiers_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_setModifiers_wrapper");

   procedure setPos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent;
     Pos  : not null access constant Qt4.Point_Fs.Internals.QPointF);
   pragma Import
    (C,
     setPos_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_setPos_wrapper");

   procedure setScenePos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent;
     Pos  : not null access constant Qt4.Point_Fs.Internals.QPointF);
   pragma Import
    (C,
     setScenePos_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_setScenePos_wrapper");

   procedure setScreenPos_wrapper
    (Self : not null access Qt_Ada.Internals.Events.QEvent;
     Pos  : not null access constant Qt4.Points.Internals.QPoint);
   pragma Import
    (C,
     setScreenPos_wrapper,
     "__Qt4Ada_QGraphicsSceneWheelEvent_setScreenPos_wrapper");

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access;

   -------------
   -- Buttons --
   -------------

   function Buttons
    (Self : not null access constant Q_Graphics_Scene_Wheel_Event'Class)
       return Qt4.Mouse_Buttons is
   begin
      return buttons_wrapper (Self.QEvent_View);
   end Buttons;

   ------------
   -- Create --
   ------------

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access
   is
   begin
      return Result : constant not null Qt4.Events.Q_Event_Access
        := new Q_Graphics_Scene_Wheel_Event
      do
         Qt_Ada.Internals.Events.Initialize (Result, Object);
      end return;
   end Create;

   ---------------
   -- Get_Delta --
   ---------------

   function Get_Delta
    (Self : not null access constant Q_Graphics_Scene_Wheel_Event'Class)
       return Q_Integer is
   begin
      return Q_Integer (delta_wrapper (Self.QEvent_View));
   end Get_Delta;

   ---------------
   -- Modifiers --
   ---------------

   function Modifiers
    (Self : not null access constant Q_Graphics_Scene_Wheel_Event'Class)
       return Qt4.Keyboard_Modifiers is
   begin
      return modifiers_wrapper (Self.QEvent_View);
   end Modifiers;

   ---------
   -- Pos --
   ---------

   function Pos
    (Self : not null access constant Q_Graphics_Scene_Wheel_Event'Class)
       return Qt4.Point_Fs.Q_Point_F is
   begin
      return Result : Qt4.Point_Fs.Q_Point_F do
         pos_wrapper
          (Self.QEvent_View, Qt4.Point_Fs.Internals.QPointF_View (Result));
      end return;
   end Pos;

   ---------------
   -- Scene_Pos --
   ---------------

   function Scene_Pos
    (Self : not null access constant Q_Graphics_Scene_Wheel_Event'Class)
       return Qt4.Point_Fs.Q_Point_F is
   begin
      return Result : Qt4.Point_Fs.Q_Point_F do
         scenePos_wrapper
          (Self.QEvent_View, Qt4.Point_Fs.Internals.QPointF_View (Result));
      end return;
   end Scene_Pos;

   ----------------
   -- Screen_Pos --
   ----------------

   function Screen_Pos
    (Self : not null access constant Q_Graphics_Scene_Wheel_Event'Class)
       return Qt4.Points.Q_Point is
   begin
      return
        Qt4.Points.Internals.Create_Wrapper
         (Qt4.Points.Internals.QPoint_Access
           (screenPos_wrapper (Self.QEvent_View)));
   end Screen_Pos;

   -----------------
   -- Set_Buttons --
   -----------------

   procedure Set_Buttons
    (Self    : not null access Q_Graphics_Scene_Wheel_Event'Class;
     Buttons : Qt4.Mouse_Buttons) is
   begin
      setButtons_wrapper (Self.QEvent_View, Buttons);
   end Set_Buttons;

   ---------------
   -- Set_Delta --
   ---------------

   procedure Set_Delta
    (Self      : not null access Q_Graphics_Scene_Wheel_Event'Class;
     The_Delta : Q_Integer) is
   begin
      setDelta_wrapper (Self.QEvent_View, Interfaces.C.int (The_Delta));
   end Set_Delta;

   -------------------
   -- Set_Modifiers --
   -------------------

   procedure Set_Modifiers
    (Self      : not null access Q_Graphics_Scene_Wheel_Event'Class;
     Modifiers : Qt4.Keyboard_Modifiers)
   is
   begin
      setModifiers_wrapper (Self.QEvent_View, Modifiers);
   end Set_Modifiers;

   -------------
   -- Set_Pos --
   -------------

   procedure Set_Pos
    (Self : not null access Q_Graphics_Scene_Wheel_Event'Class;
     Pos  : Qt4.Point_Fs.Q_Point_F) is
   begin
      setPos_wrapper
       (Self.QEvent_View, Qt4.Point_Fs.Internals.QPointF_View (Pos));
   end Set_Pos;

   -------------------
   -- Set_Scene_Pos --
   -------------------

   procedure Set_Scene_Pos
    (Self : not  null access Q_Graphics_Scene_Wheel_Event'Class;
     Pos  : Qt4.Point_Fs.Q_Point_F) is
   begin
      setScenePos_wrapper
       (Self.QEvent_View, Qt4.Point_Fs.Internals.QPointF_View (Pos));
   end Set_Scene_Pos;

   --------------------
   -- Set_Screen_Pos --
   --------------------

   procedure Set_Screen_Pos
    (Self : not null access Q_Graphics_Scene_Wheel_Event'Class;
     Pos  : Qt4.Points.Q_Point) is
   begin
      setScreenPos_wrapper
       (Self.QEvent_View, Qt4.Points.Internals.QPoint_View (Pos));
   end Set_Screen_Pos;

begin
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Graphics_Scene_Wheel, Create'Access);
end Qt4.Graphics_Scene_Wheel_Events;
