------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------
with Qt4_Entities;
with Qt4.Item_Selection_Ranges;

package Qt4.Item_Selections is

   package Thin is

      subtype QItemSelection is Qt4_Entities.Cpp_Class;

      function copy_QItemSelection
       (Cpp_Object : not null access Qt4_Entities.Cpp_Class)
        return not null access Qt4_Entities.Cpp_Class;
      pragma Import (C, copy_QItemSelection, "__Qt4Ada_copy_QItemSelection");

   end Thin;

   type Q_Item_Selection is tagged private;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Count
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Count (Self : Q_Item_Selection) return Q_Integer;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Create
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Create return Q_Item_Selection;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Item_At
   --!    <Purpose>
   --!    <Exceptions> Аналог функции QItemSelection::At. Функция называется
   --! Item_At, т.к. At - ключевое слово языка Ада.
   ---------------------------------------------------------------------------
   function Item_At (Self : Q_Item_Selection; Pos : Q_Integer)
     return Qt4.Item_Selection_Ranges.Q_Item_Selection_Range;

   package Internals is

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> Cpp_Object
      --!    <Purpose> Возвращает ссылку на ассоциированный объект C++.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function QItemSelection_View (Self : Q_Item_Selection)
        return not null access Thin.QItemSelection;

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> To_Q_Item_Selection
      --!    <Purpose> Возвращает эквивалентный объект Ada.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function To_Q_Item_Selection
       (Item : not null access constant Thin.QItemSelection)
          return Q_Item_Selection;

   end Internals;

private

   type Q_Item_Selection is new Qt4_Entities.Entity with null record;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Copy_Cpp_Object
   --!    <Purpose> Осуществляет копирование ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Copy_Cpp_Object (Self : not null access Q_Item_Selection)
     return not null access Qt4_Entities.Cpp_Class;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Delete_Cpp_Object
   --!    <Purpose> Осуществляет уничтожение ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   procedure Delete_Cpp_Object (Self : not null access Q_Item_Selection);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Default_Cpp_Object
   --!    <Purpose> Возвращает новый пустой экземпляр класса QItemSelection.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Default_Cpp_Object (Self : not null access Q_Item_Selection)
     return access Qt4_Entities.Cpp_Class;

end Qt4.Item_Selections;
