------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3786 $ $Date: 2010-10-27 11:09:19 +0400 (Wed, 27 Oct 2010) $
------------------------------------------------------------------------------
with Interfaces.C;

with Qt4.Events;
with Qt_Ada.Internals.Events.Registry;

package body Qt4.Key_Events is

   function count_wrapper
    (Cpp_Object : not null access Qt_Ada.Internals.Events.QEvent)
      return Interfaces.C.int;
   pragma Import (C, count_wrapper, "__Qt4Ada_QKeyEvent_count_wrapper");

   function isAutoRepear_wrapper
    (Cpp_Object : not null access Qt_Ada.Internals.Events.QEvent)
      return Qt4.bool;
   pragma Import
    (C, isAutoRepear_wrapper, "__Qt4Ada_QKeyEvent_isAutoRepeat_wrapper");

   function key_wrapper
    (Cpp_Object : not null access Qt_Ada.Internals.Events.QEvent)
      return Qt4.Key;
   pragma Import
    (C, key_wrapper, "__Qt4Ada_QKeyEvent_key_wrapper");

   function nativeModifiers_wrapper
    (Cpp_Object : not null access Qt_Ada.Internals.Events.QEvent)
      return Interfaces.C.unsigned;
   pragma Import
    (C, nativeModifiers_wrapper, "__Qt4Ada_QKeyEvent_nativeModifiers_wrapper");

   function nativeScanCode_wrapper
    (Cpp_Object : not null access Qt_Ada.Internals.Events.QEvent)
      return Interfaces.C.unsigned;
   pragma Import
    (C, nativeScanCode_wrapper, "__Qt4Ada_QKeyEvent_nativeScanCode_wrapper");

   function nativeVirtualKey_wrapper
    (Cpp_Object : not null access Qt_Ada.Internals.Events.QEvent)
      return Interfaces.C.unsigned;
   pragma Import (C,
                  nativeVirtualKey_wrapper,
                  "__Qt4Ada_QKeyEvent_nativeVirtualKey_wrapper");

   function text_wrapper
    (Cpp_Object : not null access Qt_Ada.Internals.Events.QEvent)
      return not null access Qt4.Strings.Internals.QString;
   pragma Import (C, text_wrapper, "__Qt4Ada_QKeyEvent_text_wrapper");

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access;

   -----------
   -- Count --
   -----------

   function Count
    (Self : not null access constant Q_Key_Event'Class) return Q_Integer is
   begin
      return Q_Integer (count_wrapper (Self.QEvent_View));
   end Count;

   ------------
   -- Create --
   ------------

   function Create
    (Object : not null Qt_Ada.Internals.Events.QEvent_Access)
       return not null Qt4.Events.Q_Event_Access is
   begin
      return Result : constant not null Qt4.Events.Q_Event_Access
        := new Q_Key_Event
      do
         Qt_Ada.Internals.Events.Initialize (Result, Object);
      end return;
   end Create;

   --------------------
   -- Is_Auto_Repeat --
   --------------------

   function Is_Auto_Repeat
    (Self : not null access constant Q_Key_Event'Class) return Boolean is
   begin
      return Qt4.To_Boolean (isAutoRepear_wrapper (Self.QEvent_View));
   end Is_Auto_Repeat;

   ---------
   -- Key --
   ---------

   function Key
    (Self : not null access constant Q_Key_Event'Class) return Qt4.Key is
   begin
      return key_wrapper (Self.QEvent_View);
   end Key;

   ----------------------
   -- Native_Modifiers --
   ----------------------

   function Native_Modifiers
    (Self : not null access constant Q_Key_Event'Class)
       return Q_Unsigned_Integer is
   begin
      return Q_Unsigned_Integer (nativeModifiers_wrapper (Self.QEvent_View));
   end Native_Modifiers;

   ----------------------
   -- Native_Scan_Code --
   ----------------------

   function Native_Scan_Code
    (Self : not null access constant Q_Key_Event'Class)
       return Q_Unsigned_Integer is
   begin
      return Q_Unsigned_Integer (nativeScanCode_wrapper (Self.QEvent_View));
   end Native_Scan_Code;

   ------------------------
   -- Native_Virtual_Key --
   ------------------------

   function Native_Virtual_Key
    (Self : not null access constant Q_Key_Event'Class)
       return Q_Unsigned_Integer is
   begin
      return Q_Unsigned_Integer (nativeVirtualKey_wrapper (Self.QEvent_View));
   end Native_Virtual_Key;

   ----------
   -- Text --
   ----------

   function Text
    (Self : not null access constant Q_Key_Event'Class)
       return Qt4.Strings.Q_String is
   begin
      return
        Qt4.Strings.Internals.Create_Wrapper
         (Qt4.Strings.Internals.QString_Access
           (text_wrapper (Self.QEvent_View)));
   end Text;

begin
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Key_Press, Create'Access);
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Key_Release, Create'Access);
   Qt_Ada.Internals.Events.Registry.Register
    (Qt4.Events.Shortcut_Override, Create'Access);
end Qt4.Key_Events;
