------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------

package body Qt4.Palettes is

   function new_QPalette return not null access Qt4_Entities.Cpp_Class;
   pragma Import (C, new_QPalette, "__Qt4Ada_new_QPalette");

   procedure delete_QPalette
    (Cpp_Object : not null access Qt4_Entities.Cpp_Class);
   pragma Import (C, delete_QPalette, "__Qt4Ada_delete_QPalette");

   procedure base_wrapper (Self   : not null access Thin.QPalette;
                           Result : not null access Qt4.Brushes.Internals.QBrush);
   pragma Import (C, base_wrapper, "__Qt4Ada_QPalette_base_wrapper");

   procedure color_wrapper
    (Self   : not null access Thin.QPalette;
     Role   : Color_Role;
     Result : not null access Qt4.Colors.Internals.QColor);
   pragma Import (C, color_wrapper, "__Qt4Ada_QPalette_color_wrapper");

   procedure set_color_wrapper
    (Self : not null access Thin.QPalette;
     Role : Color_Role;
     Col  : not null access Qt4.Colors.Internals.QColor);
   pragma Import (C, set_color_wrapper, "__Qt4Ada_QPalette_set_color_wrapper");

   procedure assign_QPalette (Self   : not null access          Thin.QPalette;
                              Result : not null access constant Thin.QPalette);
   pragma Import (C, assign_QPalette, "__Qt4Ada_assign_QPalette");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Base
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   function Base (Self : Q_Palette) return Qt4.Brushes.Q_Brush is
   begin
      return Result : Qt4.Brushes.Q_Brush do
         base_wrapper (Internals.QPalette_View (Self),
                       Qt4.Brushes.Internals.QBrush_View (Result));
      end return;
   end Base;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Color
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Color (Self : Q_Palette; Role : Color_Role)
     return Qt4.Colors.Q_Color
   is
   begin
      return Result : Qt4.Colors.Q_Color do
         color_wrapper (Internals.QPalette_View (Self),
                        Role,
                        Qt4.Colors.Internals.QColor_View (Result));
      end return;
   end Color;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Copy_Cpp_Object
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   overriding function Copy_Cpp_Object (Self : not null access Q_Palette)
     return not null access Qt4_Entities.Cpp_Class
   is
   begin
      return Thin.copy_QPalette (Self.Cpp_Object);
   end Copy_Cpp_Object;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Create
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   function Create return Q_Palette is
      Result : Q_Palette;

   begin
      return Result;
   end Create;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Default_Cpp_Object
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   overriding function Default_Cpp_Object (Self : not null access Q_Palette)
     return access Qt4_Entities.Cpp_Class
   is
      pragma Unreferenced (Self);
      --  Параметр необходим для диспетчеризации надклассового вызова.

   begin
      return new_QPalette;
   end Default_Cpp_Object;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Delete_Cpp_Object
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   overriding
   procedure Delete_Cpp_Object (Self : not null access Q_Palette)
   is
   begin
      delete_QPalette (Self.Cpp_Object);
   end Delete_Cpp_Object;

   package body Internals is

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> QPalette_View
      --!    <ImplementationNotes>
      ------------------------------------------------------------------------
      function QPalette_View (Self : Q_Palette)
        return not null access Thin.QPalette
      is
      begin
         return Qt4_Entities.Cpp_Object (Self);
      end QPalette_View;

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> To_Q_Palette
      --!    <ImplementationNotes>
      ------------------------------------------------------------------------
      function To_Q_Palette (Item : not null access constant Thin.QPalette)
        return Q_Palette
      is
      begin
         return Result : Q_Palette do
            assign_QPalette (QPalette_View (Result), Item);
         end return;
      end To_Q_Palette;

   end Internals;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Set_Color
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   procedure Set_Color (Self : Q_Palette;
                        Role : Color_Role;
                        Col  : Qt4.Colors.Q_Color)
   is
   begin
      set_color_wrapper
       (Internals.QPalette_View (Self),
        Role,
        Qt4.Colors.Internals.QColor_View (Col));
   end Set_Color;

end Qt4.Palettes;
