------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------
with Qt4.Brushes;
with Qt4.Colors;
with Qt4_Entities;

package Qt4.Palettes is

   type Color_Role is
    (Window_Text,
     Button,
     Light,
     Midlight,
     Dark,
     Mid,
     Text,
     Bright_Text,
     Button_Text,
     Base,
     Window,
     Shadow,
     Highlight,
     Highlighted_Text,
     Link,
     Link_Visited,
     Alternate_Base,
     No_Role);

   pragma Convention (C, Color_Role);
   for Color_Role'Size use Interfaces.C.int'Size;
   for Color_Role use
    (Window_Text      => 0,
     Button           => 1,
     Light            => 2,
     Midlight         => 3,
     Dark             => 4,
     Mid              => 5,
     Text             => 6,
     Bright_Text      => 7,
     Button_Text      => 8,
     Base             => 9,
     Window           => 10,
     Shadow           => 11,
     Highlight        => 12,
     Highlighted_Text => 13,
     Link             => 14,
     Link_Visited     => 15,
     Alternate_Base   => 16,
     No_Role          => 17);

     Foreground    : constant Color_Role := Window_Text;
     N_Color_Roles : constant Color_Role := No_Role;
     Background    : constant Color_Role := Window;

   type Q_Palette is tagged private;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Base
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Base (Self : Q_Palette) return Qt4.Brushes.Q_Brush;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Color
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Color (Self : Q_Palette; Role : Color_Role)
     return Qt4.Colors.Q_Color;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Create
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Create return Q_Palette;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Set_Color
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   procedure Set_Color (Self : Q_Palette;
                        Role : Color_Role;
                        Col  : Qt4.Colors.Q_Color);

   package Thin is

      subtype QPalette is Qt4_Entities.Cpp_Class;

      function copy_QPalette
       (Cpp_Object : not null access Qt4_Entities.Cpp_Class)
        return not null access Qt4_Entities.Cpp_Class;
      pragma Import (C, copy_QPalette, "__Qt4Ada_copy_QPalette");

   end Thin;

   package Internals is

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> Cpp_Object
      --!    <Purpose> Возвращает ссылку на ассоциированный объект C++.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function QPalette_View (Self : Q_Palette)
        return not null access Thin.QPalette;

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> To_Q_Palette
      --!    <Purpose> Возвращает эквивалентный объект Ada.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function To_Q_Palette (Item : not null access constant Thin.QPalette)
        return Q_Palette;

   end Internals;

private

   type Q_Palette is new Qt4_Entities.Entity with null record;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Copy_Cpp_Object
   --!    <Purpose> Осуществляет копирование ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Copy_Cpp_Object (Self : not null access Q_Palette)
     return not null access Qt4_Entities.Cpp_Class;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Delete_Cpp_Object
   --!    <Purpose> Осуществляет уничтожение ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   procedure Delete_Cpp_Object (Self : not null access Q_Palette);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Default_Cpp_Object
   --!    <Purpose> Возвращает новый пустой экземпляр класса QPalette.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Default_Cpp_Object (Self : not null access Q_Palette)
     return access Qt4_Entities.Cpp_Class;

end Qt4.Palettes;
