------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------

package body Qt4.Radial_Gradients is

   function new_QRadialGradient return not null access Qt4_Entities.Cpp_Class;
   pragma Import (C, new_QRadialGradient, "__Qt4Ada_new_QRadialGradient");

   procedure setFocalPoint_wrapper
    (Self : not null access Qt4.Gradients.Thin.QGradient;
     X    : Interfaces.C.double;
     Y    : Interfaces.C.double);
   pragma Import (C,
                  setFocalPoint_wrapper,
                  "__Qt4Ada_QRadialGradient_setFocalPoint_wrapper");

   procedure setCenter_wrapper
    (Self : not null access Qt4.Gradients.Thin.QGradient;
     X    : Interfaces.C.double;
     Y    : Interfaces.C.double);
   pragma Import (C,
                  setCenter_wrapper,
                  "__Qt4Ada_QRadialGradient_setCenter_wrapper");

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Create
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Create (The_Type : Qt4.Gradients.Internals.Gradient_Type)
     return Qt4.Gradients.Q_Gradient'Class;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Create
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   function Create (The_Type : Qt4.Gradients.Internals.Gradient_Type)
     return Qt4.Gradients.Q_Gradient'Class
   is
      Result : Q_Radial_Gradient;
      use type Qt4.Gradients.Internals.Gradient_Type;

   begin
      if The_Type /= Qt4.Gradients.Internals.Radial_Gradient then
         raise Program_Error;
      end if;

      return Result;
   end Create;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Default_Cpp_Object
   --!    <Purpose> Возвращает новый пустой экземпляр класса QRadialGradient.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Default_Cpp_Object (Self : not null access Q_Radial_Gradient)
     return access Qt4_Entities.Cpp_Class
   is
      pragma Unreferenced (Self);
      --  XXX параметр не используется и нужен для диспетчеризации вызова.

   begin
      return new_QRadialGradient;
   end Default_Cpp_Object;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Set_Center
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   procedure Set_Center (Self : Q_Radial_Gradient;
                         X    : Q_Real;
                         Y    : Q_Real)
   is
   begin
      setCenter_wrapper (Qt4.Gradients.Internals.QGradient_View (Self),
                         Interfaces.C.double (X),
                         Interfaces.C.double (Y));
   end Set_Center;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Set_Focal_Point
   --!    <ImplementationNotes>
   ---------------------------------------------------------------------------
   procedure Set_Focal_Point (Self : Q_Radial_Gradient;
                              X    : Q_Real;
                              Y    : Q_Real)
   is
   begin
      setFocalPoint_wrapper (Qt4.Gradients.Internals.QGradient_View (Self),
                             Interfaces.C.double (X),
                             Interfaces.C.double (Y));
   end Set_Focal_Point;

begin
   Qt4.Gradients.Internals.Register
    (Qt4.Gradients.Internals.Radial_Gradient, Create'Access);
end Qt4.Radial_Gradients;
