------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------
with Qt4_Entities;

package Qt4.Size_Policies is

   package Thin is

      subtype QSizePolicy is Qt4_Entities.Cpp_Class;

   end Thin;

   type Policy_Flag is
    (Grow_Flag,
     Expand_Flag,
     Shrink_Flag,
     Ignore_Flag);
   pragma Convention (C, Policy_Flag);
   for Policy_Flag'Size use Interfaces.C.int'Size;
   for Policy_Flag use
    (Grow_Flag   => 1,
     Expand_Flag => 2,
     Shrink_Flag => 4,
     Ignore_Flag => 8);

   type Policy is
    (Fixed,
     Minimum,
     Minimum_Expanding,
     Maximum,
     Preferred,
     Expanding,
     Ignored);
   pragma Convention (C, Policy);
   for Policy'Size use Interfaces.C.int'Size;
   for Policy use
    (Fixed             => 0,
     Minimum           => 1,   --  Grow_Flag
     Minimum_Expanding => 3,   --  GrowFlag    | ExpandFlag
     Maximum           => 4,   --  Shrink_Flag
     Preferred         => 5,   --  GrowFlag    | ShrinkFlag
     Expanding         => 7,   --  GrowFlag    | ShrinkFlag | ExpandFlag
     Ignored           => 13); --  ShrinkFlag  | GrowFlag   | IgnoreFlag

   type Q_Size_Policy is tagged private;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Create
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Create return Q_Size_Policy;

   package Internals is

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> Cpp_Object
      --!    <Purpose> Возвращает ссылку на ассоциированный объект C++.
      --! Вместо этой подпрограммы необходимо использовать QSizePolicy_View.
      --! Со временем эта подпрограмма будет удалена.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function Cpp_Object (Self : Q_Size_Policy)
        return not null access Thin.QSizePolicy;

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> QSizePolicy_View
      --!    <Purpose> Возвращает ссылку на ассоциированный объект C++.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function QSizePolicy_View (Self : Q_Size_Policy)
        return not null access Thin.QSizePolicy
          renames Cpp_Object;

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> To_Q_Size_Policy
      --!    <Purpose> Возвращает эквивалентный объект Ada.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function To_Q_Size_Policy
       (Item : not null access constant Thin.QSizePolicy)
          return Q_Size_Policy;

   end Internals;

private

   type Q_Size_Policy is new Qt4_Entities.Entity with null record;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Copy_Cpp_Object
   --!    <Purpose> Осуществляет копирование ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Copy_Cpp_Object (Self : not null access Q_Size_Policy)
     return not null access Qt4_Entities.Cpp_Class;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Delete_Cpp_Object
   --!    <Purpose> Осуществляет уничтожение ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   procedure Delete_Cpp_Object (Self : not null access Q_Size_Policy);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Default_Cpp_Object
   --!    <Purpose> Возвращает новый пустой экземпляр класса QSizePolicy.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Default_Cpp_Object (Self : not null access Q_Size_Policy)
     return access Qt4_Entities.Cpp_Class;

end Qt4.Size_Policies;
