------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3769 $ $Date: 2010-10-24 03:16:28 +0400 (Sun, 24 Oct 2010) $
------------------------------------------------------------------------------
with Qt4_Entities;

package Qt4.Text_Options is

   package Thin is

      subtype QTextOption is Qt4_Entities.Cpp_Class;

      function copy_QTextOption
       (Cpp_Object : not null access Qt4_Entities.Cpp_Class)
        return not null access Qt4_Entities.Cpp_Class;
      pragma Import (C, copy_QTextOption, "__Qt4Ada_copy_QTextOption");

   end Thin;

   type Text_Option_Wrap_Mode is
    (No_Wrap,
     Word_Wrap,
     Manual_Wrap,
     Wrap_Anywhere,
     Wrap_At_Word_Boundary_Or_Anywhere);
   pragma Convention (C, Text_Option_Wrap_Mode);
   for Text_Option_Wrap_Mode'Size use Interfaces.C.unsigned'Size;

   type Q_Text_Option is tagged private;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Create
   --!    <Purpose>
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   function Create return Q_Text_Option;

   package Internals is

      type QTextOption_Access is access all Thin.QTextOption;

      function QTextOption_View (Self : Q_Text_Option'Class)
        return not null QTextOption_Access;

      ------------------------------------------------------------------------
      --! <Subprogram>
      --!    <Unit> To_Q_Text_Option
      --!    <Purpose> Возвращает эквивалентный объект Ada.
      --!    <Exceptions>
      ------------------------------------------------------------------------
      function To_Q_Text_Option (Item : not null access Thin.QTextOption)
        return Q_Text_Option;

   end Internals;

private

   type Q_Text_Option is new Qt4_Entities.Entity with null record;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Copy_Cpp_Object
   --!    <Purpose> Осуществляет копирование ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Copy_Cpp_Object (Self : not null access Q_Text_Option)
     return not null access Qt4_Entities.Cpp_Class;

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Delete_Cpp_Object
   --!    <Purpose> Осуществляет уничтожение ассоциированного объекта C++.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   procedure Delete_Cpp_Object (Self : not null access Q_Text_Option);

   ---------------------------------------------------------------------------
   --! <Subprogram>
   --!    <Unit> Default_Cpp_Object
   --!    <Purpose> Возвращает новый пустой экземпляр класса QTextOption.
   --!    <Exceptions>
   ---------------------------------------------------------------------------
   overriding
   function Default_Cpp_Object (Self : not null access Q_Text_Option)
     return access Qt4_Entities.Cpp_Class;

end Qt4.Text_Options;
