------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2009-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3689 $ $Date: 2010-01-03 15:38:15 +0300 (Sun, 03 Jan 2010) $
------------------------------------------------------------------------------
with System.Address_To_Access_Conversions;

package Qt_Ada.Internals.Graphics_Items is

   pragma Preelaborate;

   type QGraphicsItem is null record;
   pragma Convention (C, QGraphicsItem);
   --  This type is used to represent C++ obejct of QGraphicsItem subclass
   --  casted to QGraphicsItem.

   package Conversions is
     new System.Address_To_Access_Conversions (QGraphicsItem);

   subtype QGraphicsItem_Access is Conversions.Object_Pointer;

   type Q_Graphics_Item_Base is limited interface;

   type Q_Graphics_Item_Base_Access is access all Q_Graphics_Item_Base'Class;
   pragma Convention (C, Q_Graphics_Item_Base_Access);

   not overriding function QGraphicsItem_View
    (Self : not null access constant Q_Graphics_Item_Base)
       return not null QGraphicsItem_Access
         is abstract;

   procedure Initialize
    (Self   : not null access Q_Graphics_Item_Base'Class;
     Object : not null QGraphicsItem_Access);
   --  Establish association between proxy/director object at Ada side and
   --  corresponding object at C++ side. Initialize Object member when actual
   --  type of Self is derived from Q_Graphics_Item_Impl_Base type.

   type Q_Graphics_Item_Impl_Base is
     abstract limited new Q_Graphics_Item_Base with private;

   not overriding procedure Finalize
    (Self : not null access Q_Graphics_Item_Impl_Base)
       is null;

   overriding function QGraphicsItem_View
    (Self : not null access constant Q_Graphics_Item_Impl_Base)
       return not null QGraphicsItem_Access;

private

   type Q_Graphics_Item_Impl_Base is
     abstract limited new Q_Graphics_Item_Base with
   record
      Object : QGraphicsItem_Access;
   end record;

end Qt_Ada.Internals.Graphics_Items;
