------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2009 Vadim Godunko <vgodunko@gmail.com>                      --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3146 $ $Date: 2009-03-04 19:03:35 +0300 (Wed, 04 Mar 2009) $
------------------------------------------------------------------------------
with Ada.Strings.Wide_Unbounded;

with Asis.Compilation_Units.Relations;

with Amoc.Asis_Implementation;
with Amoc.Model;

package body Amoc.Analizer.Dependencies is

   -------------
   -- Analyze --
   -------------

   procedure Analyze is
   begin
      Amoc.Model.Dependencies.Append
       (Ada.Strings.Wide_Unbounded.To_Unbounded_Wide_String
         (Asis.Compilation_Units.Text_Name (Amoc.Analizer.Primary_Unit)));

      if Amoc.Asis_Implementation.Supports_Semantic_Dependence_Order then
         --  If ASIS implementation supports Semantic_Dependence_Order,
         --  use it for generate full dependencies.

         declare
            Units : constant Asis.Compilation_Unit_List
              := (1 => Amoc.Analizer.Primary_Unit);

            Result : constant Asis.Compilation_Units.Relations.Relationship
              := Asis.Compilation_Units.Relations.Semantic_Dependence_Order
                  (Units,
                   Asis.Nil_Compilation_Unit_List,
                   Amoc.Analizer.Context,
                   Asis.Supporters);

         begin
            for J in Result.Consistent'Range loop
               Amoc.Model.Dependencies.Append
                (Ada.Strings.Wide_Unbounded.To_Unbounded_Wide_String
                  (Asis.Compilation_Units.Text_Name (Result.Consistent (J))));
            end loop;
         end;
      end if;
   end Analyze;

end Amoc.Analizer.Dependencies;
