------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008-2011 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3843 $ $Date: 2011-06-21 14:42:58 +0400 (Tue, 21 Jun 2011) $
------------------------------------------------------------------------------
with Ada.Characters.Conversions;
with Ada.Directories;
with Ada.Wide_Text_IO;

with Asis.Ada_Environments;
with Asis.Declarations;
with Asis.Definitions;
with Asis.Elements;
with Asis.Extensions;
with Asis.Implementation;

with GNAT.OS_Lib;

with Amoc.Diagnoses;
with Amoc.Parameters;

package body Amoc.Asis_Implementation_Gnat is

   --------------------------
   -- Cleanup_Asis_Context --
   --------------------------

   procedure Cleanup_Asis_Context (Context : in out Asis.Context) is
      Base_Name : constant String
        := Ada.Directories.Base_Name
            (Ada.Characters.Conversions.To_String
              (Amoc.Parameters.Source_File_Name));

   begin
      Asis.Ada_Environments.Close (Context);
      Asis.Ada_Environments.Dissociate (Context);
      Asis.Implementation.Finalize;

      Ada.Directories.Delete_File (Base_Name & ".ali");
      Ada.Directories.Delete_File (Base_Name & ".adt");
   end Cleanup_Asis_Context;

   ------------------------
   -- Has_Null_Exclusion --
   ------------------------

   function Has_Null_Exclusion (Element : Asis.Element) return Boolean is
      use type Asis.Trait_Kinds;

   begin
      return Asis.Elements.Trait_Kind (Element) = Asis.A_Null_Exclusion_Trait;
   end Has_Null_Exclusion;

   --------------------------------
   -- Object_Declaration_Subtype --
   --------------------------------

   function Object_Declaration_Subtype
    (Element : Asis.Element) return Asis.Element
       renames Asis.Declarations.Object_Declaration_View;

   --------------------------
   -- Prepare_Asis_Context --
   --------------------------

   procedure Prepare_Asis_Context (Context : in out Asis.Context) is
      Success     : Boolean;
      Source_File : aliased String
        := Ada.Characters.Conversions.To_String
            (Amoc.Parameters.Source_File_Name);
      Source_Dirs : constant Amoc.Parameters.Wide_String_Vectors.Vector
        := Amoc.Parameters.Source_File_Search_Paths;
      Args        : GNAT.OS_Lib.Argument_List
                     (1 .. Integer (Source_Dirs.Length) + 1);

   begin
      Args (Args'First) := new String'("-gnat12");

      for J in 1 .. Integer (Source_Dirs.Length) loop
         Args (J + 1) :=
           new String'
                ("-I"
                   & Ada.Characters.Conversions.To_String
                      (Source_Dirs.Element (J)));
      end loop;

      Asis.Extensions.Compile (Source_File'Unchecked_Access,
                               Args,
                               Success);

      declare
         ADT_File_Name : constant Wide_String
           := Ada.Characters.Conversions.To_Wide_String
               (Ada.Directories.Base_Name (Source_File) & ".adt");

      begin
         if Success then
            if Amoc.Parameters.Verbose then
               Ada.Wide_Text_IO.Put_Line
                ("ADT file " & ADT_File_Name & " successfully created.");
            end if;

         else
            Amoc.Diagnoses.Output
             (Amoc.Parameters.Source_File_Name,
              1,
              1,
              Amoc.Diagnoses.Unable_To_Compile_Source_File);

            raise Amoc.Amoc_Error;
         end if;

         Asis.Implementation.Initialize ("-asis05");
         Asis.Ada_Environments.Associate
          (Context, "AMOCContext", "-C1 " & ADT_File_Name);
         Asis.Ada_Environments.Open (Context);
      end;
   end Prepare_Asis_Context;

   ---------------------
   -- Progenitor_List --
   ---------------------

   function Progenitor_List (Declaration : Asis.Declaration)
     return Asis.Name_List
       renames Asis.Definitions.Definition_Interface_List;

end Amoc.Asis_Implementation_Gnat;
