------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008, 2009 Vadim Godunko <vgodunko@gmail.com>                --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3321 $ $Date: 2009-05-04 01:05:25 +0400 (Mon, 04 May 2009) $
------------------------------------------------------------------------------
with Asis;

package Amoc.Diagnoses is

   type Message_Kind is
    (Duplicate_Pragma_Q_Object_Declaration,
     Pragma_Q_Object_Must_Have_2_Arguments,
     First_Argument_Pragma_Q_Object_Must_Be_Identifier,
     First_Argument_Pragma_Q_Object_Must_Be_Type_Name,
     Second_Argument_Pragma_Q_Object_Must_Be_String,
     Second_Argument_Pragma_Q_Object_Must_Be_Valid,

     Q_Signal_Must_Have_1_Or_2_Parameters,
     Q_Signal_1st_Must_Be_Direct_Name,
     Q_Signal_2nd_Must_Be_String_Literal,
     Q_Signal_Subprogram_Not_Found,
     Q_Signal_Subprogram_Must_Start_From_Emit,
     Q_Signal_Subprogram_Must_Be_Procedure,

     Q_Slot_Must_Have_1_Or_2_Parameters,
     Q_Slot_1st_Must_Be_Direct_Name,
     Q_Slot_2nd_Must_Be_String_Literal,
     Q_Slot_Subprogram_Not_Found,

     Q_Invokable_Must_Have_1_Or_2_Parameters,
     Q_Invokable_1st_Must_Be_Direct_Name,
     Q_Invokable_2nd_Must_Be_String_Literal,
     Q_Invokable_Subprogram_Not_Found,

     Type_Derived_From_Director_Type_In_Visible_Part,
     Unable_To_Find_Type_Derived_From_Q_Object_Director,
     Unknown_Pragma,

     --  GNAT specific messages

     Unable_To_Compile_Source_File);

   procedure Output
    (File_Name : Wide_String;
     Line      : Positive;
     Column    : Positive;
     Message   : Wide_String);
   --  Output file location information and diagnoses message

   procedure Output
    (File_Name : Wide_String;
     Line      : Positive;
     Column    : Positive;
     Message   : Message_Kind);
   --  Output file location information and diagnoses message

   procedure Output
    (Element : Asis.Element;
     Message : Message_Kind);
   --  Output file location information and diagnoses message. File location
   --  information extracted from specified Element.

   procedure Raise_Exception (Item : Asis.Element; Text : Wide_String);
   --  Генерирует исключение с описанием причины его возникновения.

end Amoc.Diagnoses;
