------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008 Vadim Godunko <vgodunko@gmail.com>                      --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 2959 $ $Date: 2008-12-21 12:55:47 +0300 (Sun, 21 Dec 2008) $
------------------------------------------------------------------------------
with Ada.Command_Line;
with Ada.Wide_Text_IO;

with Asis.Implementation;

with Amoc.Analizer;
with Amoc.Generator;
with Amoc.Parameters;
with Amoc.Version;

procedure Amoc.Driver is

   procedure Usage;
   --  Outputs usage information.

   procedure Version;
   --  Outputs version information.

   -----------
   -- Usage --
   -----------

   procedure Usage is
   begin
      Ada.Wide_Text_IO.Put_Line
       (Ada.Wide_Text_IO.Standard_Error,
        "Ada-aware Meta Object Compiler");
      Ada.Wide_Text_IO.Put_Line
       (Ada.Wide_Text_IO.Standard_Error,
        "Usage: amoc [switches] <ada file>");
      Ada.Wide_Text_IO.Put_Line
       (Ada.Wide_Text_IO.Standard_Error,
        "switches:");
      Ada.Wide_Text_IO.Put_Line
       (Ada.Wide_Text_IO.Standard_Error,
        "  -Idir  source files directory");
      Ada.Command_Line.Set_Exit_Status (2);
   end Usage;

   -------------
   -- Version --
   -------------

   procedure Version is

      function Changeset return Wide_String;
      --  Returns formatted changeset number.

      ---------------
      -- Changeset --
      ---------------

      function Changeset return Wide_String is
         Aux : constant Wide_String := Amoc.Version.Qt_Ada_Changeset_Number;

      begin
         if Aux = "" then
            return "";

         else
            return " (" & Aux & ')';
         end if;
      end Changeset;

   begin
      Ada.Wide_Text_IO.Put_Line
       (Ada.Wide_Text_IO.Standard_Error,
        "Ada-aware Meta Object Compiler from QtAda "
          & Amoc.Version.Qt_Ada_Version
          & Changeset
          & ' '
          & Amoc.Version.Qt_Ada_Edition
          & " Edition");
   end Version;

begin
   if Amoc.Parameters.Help then
      Usage;

      return;

   elsif Amoc.Parameters.Version then
      Version;

      return;
   end if;

   Amoc.Analizer.Analyze;
   Amoc.Generator.Generate;
   Amoc.Analizer.Cleanup;

exception
   when Amoc.Amoc_Error =>
      Ada.Command_Line.Set_Exit_Status (5);

   when others =>
      Ada.Wide_Text_IO.Put_Line (Asis.Implementation.Diagnosis);

      raise;
end Amoc.Driver;
