------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2009-2010 Vadim Godunko <vgodunko@gmail.com>                 --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3763 $ $Date: 2010-10-23 23:13:06 +0400 (Sat, 23 Oct 2010) $
------------------------------------------------------------------------------
with Ada.Characters.Conversions;
with Ada.Directories;
with Ada.Strings.Wide_Unbounded;
with Ada.Wide_Text_IO;

with Amoc.Model;
with Amoc.Parameters;

package body Amoc.Generator.Dependencies is

   function Dependencies_File_Name return String;

   ----------------------------
   -- Dependencies_File_Name --
   ----------------------------

   function Dependencies_File_Name return String is
   begin
      return
        Ada.Directories.Base_Name
         (Ada.Characters.Conversions.To_String
           (Amoc.Parameters.Source_File_Name)) & ".d";
   end Dependencies_File_Name;

   --------------
   -- Generate --
   --------------

   procedure Generate is
      File  : Ada.Wide_Text_IO.File_Type;
      First : Boolean;

      procedure Output_Dependency
       (Position : Amoc.Model.Unbounded_Wide_String_Vectors.Cursor);

      procedure Output_Generated
       (Position : Amoc.Model.Unbounded_Wide_String_Vectors.Cursor);

      -----------------------
      -- Output_Dependency --
      -----------------------

      procedure Output_Dependency
       (Position : Amoc.Model.Unbounded_Wide_String_Vectors.Cursor)
      is
         Dep : constant Wide_String
           := Ada.Strings.Wide_Unbounded.To_Wide_String
               (Amoc.Model.Unbounded_Wide_String_Vectors.Element (Position));

      begin
         if not First then
            Ada.Wide_Text_IO.Put_Line (File, " \");

         else
            First := False;
         end if;

         Ada.Wide_Text_IO.Put (File, " " & Dep);
      end Output_Dependency;

      ----------------------
      -- Output_Generated --
      ----------------------

      procedure Output_Generated
       (Position : Amoc.Model.Unbounded_Wide_String_Vectors.Cursor)
      is
         Gen : constant Wide_String
           := Ada.Strings.Wide_Unbounded.To_Wide_String
               (Amoc.Model.Unbounded_Wide_String_Vectors.Element (Position));

      begin
         First := True;

         Ada.Wide_Text_IO.Put (File, Gen & ':');
         Amoc.Model.Dependencies.Iterate (Output_Dependency'Access);
         Ada.Wide_Text_IO.New_Line (File);
      end Output_Generated;

   begin
      Ada.Wide_Text_IO.Create
       (File, Ada.Wide_Text_IO.Out_File, Dependencies_File_Name);

      Amoc.Model.Generated_Files.Iterate (Output_Generated'Access);

      Ada.Wide_Text_IO.Close (File);
   end Generate;

end Amoc.Generator.Dependencies;
