------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008, 2009 Vadim Godunko <vgodunko@gmail.com>                --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3329 $ $Date: 2009-05-04 02:55:39 +0400 (Mon, 04 May 2009) $
------------------------------------------------------------------------------
with Ada.Characters.Conversions;
with Ada.Characters.Handling;
with Ada.Directories;
with Ada.Wide_Text_IO;

with Amoc.Parameters;

package body Amoc.Model is

   ---------------------------
   -- Compilation_Unit_Name --
   ---------------------------

   function Compilation_Unit_Name return Wide_String is

      function To_Upper (Item : Wide_Character) return Wide_Character;
      --  Converts specified character to the upper case form.

      --------------
      -- To_Upper --
      --------------

      function To_Upper (Item : Wide_Character) return Wide_Character is
      begin
         return
           Ada.Characters.Conversions.To_Wide_Character
            (Ada.Characters.Handling.To_Upper
              (Ada.Characters.Conversions.To_Character (Item)));
      end To_Upper;

      Unit : Wide_String
        := Ada.Characters.Conversions.To_Wide_String
            (Ada.Directories.Base_Name
              (Ada.Characters.Conversions.To_String
                (Amoc.Parameters.Source_File_Name)));

   begin
      Unit (Unit'First) := To_Upper (Unit (Unit'First));

      for Index in Unit'First + 1 .. Unit'Last - 1 loop
         if Unit (Index) = '-' then
            Unit (Index) := '.';
            Unit (Index + 1) := To_Upper (Unit (Index + 1));
         end if;

         if Unit (Index) = '_' then
            Unit (Index + 1) := To_Upper (Unit (Index + 1));
         end if;
      end loop;

      return Unit;
   end Compilation_Unit_Name;

   --------------------
   -- Output_Summary --
   --------------------

   procedure Output_Summary is

      procedure Process_Method (Position : Method_Info_Vectors.Cursor);

      --------------------
      -- Process_Method --
      --------------------

      procedure Process_Method (Position : Method_Info_Vectors.Cursor) is
         Element : constant Method_Info
           := Method_Info_Vectors.Element (Position);

      begin
         Ada.Wide_Text_IO.Put_Line
          ("   "
             & Ada.Characters.Conversions.To_Wide_String
                (Ada.Strings.Unbounded.To_String
                  (Element.Signature))
             & " => "
             & Ada.Characters.Conversions.To_Wide_String
                (Ada.Strings.Unbounded.To_String
                  (Element.Subprogram_Name)));
      end Process_Method;

   begin
      Ada.Wide_Text_IO.Put_Line ("Signals:");
      Signals.Iterate (Process_Method'Access);
      Ada.Wide_Text_IO.Put_Line ("Slots:");
      Slots.Iterate (Process_Method'Access);
      Ada.Wide_Text_IO.Put_Line ("Methods:");
      Methods.Iterate (Process_Method'Access);

      Ada.Wide_Text_IO.Put_Line ("Declared interfaces:");

      for J in 1 .. Integer (Declared_Interfaces.Length) loop
         Ada.Wide_Text_IO.Put_Line
          ("   "
             & Ada.Characters.Conversions.To_Wide_String
                (Ada.Strings.Unbounded.To_String
                  (Declared_Interface_Info_Vectors.Element
                    (Declared_Interfaces, J).Name)));
      end loop;

      Ada.Wide_Text_IO.Put_Line ("Declared plugin:");

      if Plugin.Is_Declared then
         Ada.Wide_Text_IO.Put_Line
          ("   "
             & Ada.Characters.Conversions.To_Wide_String
                (To_String (Plugin.Name)));
       end if;

      Ada.Wide_Text_IO.Put_Line ("Properties:");

      for J in 1 .. Integer (Properties.Length) loop
         Ada.Wide_Text_IO.Put_Line
          ("   "
             & Ada.Characters.Conversions.To_Wide_String
                (To_String (Property_Info_Vectors.Element
                             (Properties, J).Name)));
      end loop;
   end Output_Summary;

end Amoc.Model;
