------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008, 2009 Vadim Godunko <vgodunko@gmail.com>                --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3334 $ $Date: 2009-05-04 11:59:21 +0400 (Mon, 04 May 2009) $
------------------------------------------------------------------------------
with Ada.Containers.Vectors;
with Ada.Strings.Unbounded;
with Ada.Strings.Wide_Unbounded;

with Asis;

package Amoc.Model is

   use Ada.Strings.Unbounded;

   type Type_Kinds is (Primitive, Enumeration, Value_Class, QObject_Subclass);

   type Type_Info is record
      Kind           : Type_Kinds;
      Qt_Name        : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
      Ada_Name       : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
      Create_Wrapper : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
      --  Full qualified name of the Create_Wrapper subprogram. Valid only
      --  for Value_Class types.
      To_Pointer     : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
      --  Full qualified name of the To_Pointer subprogram. Valid only for
      --  Value_Class and QObject_Subclass types.
      To_Address     : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
      --  Full qualified name of the To_Address subprogram. Valid only for
      --  Value_Class and QObject_Subclass types.
      Class_View     : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
      --  Full qualified name of the <Class>_View subprogram. Valid only for
      --  Value_Class types.
      Utils_Package  : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
      --  Full qualified name of the utilities package. Valid only for
      --  QObject_Subclass types.
   end record;

   package Type_Vectors is new Ada.Containers.Vectors (Positive, Type_Info);

   type Method_Kinds is (Signal, Slot, Method);

   type Visibility_Kinds is (A_Public, A_Protected, A_Private);

   type Parameter_Info is record
      Name          : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
      --  Parameter name.

      Type_Image    : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
      --  Text image of the parameter's type.

      Type_Id       : Natural;  --  XXX Must be Positive!
      --  Parameter's type

      Is_Classwide  : Boolean;
      --  Classwide type is used for parameter.

      Is_Access     : Boolean;
      Is_Constant   : Boolean;
      Is_Not_Null   : Boolean;
      --  Represent mode/view of the parameter:
      --  Is_Access Is_Constant Is_Not_Null Mode/View
      --    False     False                  in out
      --    False     True                   in
      --    True      False       False      access
      --    True      False       True       not null access
      --    True      True        False      access constant
      --    True      True        True       not null access constant
   end record;

   package Parameter_Vectors is
     new Ada.Containers.Vectors (Positive, Parameter_Info);

   type Method_Info is record
      Subprogram_Name : Ada.Strings.Unbounded.Unbounded_String;
      Signature       : Ada.Strings.Unbounded.Unbounded_String;
      Parameters      : Parameter_Vectors.Vector;
      Kind            : Method_Kinds;
      Visibility      : Visibility_Kinds;
   end record;

   type Declared_Interface_Info is record
      Name       : Ada.Strings.Unbounded.Unbounded_String;
      Identifier : Ada.Strings.Unbounded.Unbounded_String;
   end record;

   type Boolean_Property is record
      Empty : Boolean := False;
      State : Boolean := True;
   end record;

   type Property_Info is record
      Name       : Ada.Strings.Unbounded.Unbounded_String;
      Prop_Type  : Asis.Element;
      Read       : Asis.Element;
      Write      : Asis.Element := Asis.Nil_Element;
      Reset      : Asis.Element := Asis.Nil_Element;
      Designable : Boolean_Property;
      Scriptable : Boolean_Property;
      Stored     : Boolean_Property;
      Editable   : Boolean_Property;
      User       : Boolean_Property;
   end record;

   type Plugin_Class_Info is record
      Name        : Ada.Strings.Unbounded.Unbounded_String;
      Constructor : Ada.Strings.Unbounded.Unbounded_String;
      Is_Declared : Boolean;
   end record;

   package Declared_Interface_Info_Vectors is
     new Ada.Containers.Vectors (Positive, Declared_Interface_Info);

   package Property_Info_Vectors is
     new Ada.Containers.Vectors (Positive, Property_Info);

   package Method_Info_Vectors is
     new Ada.Containers.Vectors (Positive, Method_Info);

   package Unbounded_Wide_String_Vectors is
     new Ada.Containers.Vectors
          (Positive,
           Ada.Strings.Wide_Unbounded.Unbounded_Wide_String,
           Ada.Strings.Wide_Unbounded."=");

   Known_Types              : Type_Vectors.Vector;
   --  Known signal/slot/method parameter types.

   Tagged_Type_Name         : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
   --  Tagged type name.

   Custom_Class_Name        : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
   --  Custom name of the class.

   Parent_Class_Is_Imported : Boolean := False;
   --  Parent class is imported from the C++.

   Parent_Package_Name      : Ada.Strings.Wide_Unbounded.Unbounded_Wide_String;
   --  Package name of the parent class's type declaration.

   Signals                  : Method_Info_Vectors.Vector;
   --  Class's signals.

   Slots                    : Method_Info_Vectors.Vector;
   --  Class's slots.

   Methods                  : Method_Info_Vectors.Vector;
   --  Class's invocable operations.

   Declared_Interfaces      : Declared_Interface_Info_Vectors.Vector;
   --  Объявленные интерфейсы.

   Properties               : Property_Info_Vectors.Vector;
   --  Объявленные свойства.

   Plugin                   : Plugin_Class_Info;
   --  Объявленный плагин.

   Dependencies             : Unbounded_Wide_String_Vectors.Vector;
   --  List of the file names from which main analyzed package dependent.

   Generated_Files          : Unbounded_Wide_String_Vectors.Vector;
   --  List of the generated files.

   function Compilation_Unit_Name return Wide_String;
   --  Returns name of the expected package name for the specified file name.

   procedure Output_Summary;
   --  Output summary information.

end Amoc.Model;
