------------------------------------------------------------------------------
--                                                                          --
--                An Ada2005 binding to the Qt C++ framework                --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2008 Vadim Godunko <vgodunko@gmail.com>                      --
--                                                                          --
-- QtAda is free software;  you can  redistribute it and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. QtAda is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License distributed with QtAda;  see file COPYING.  If not, write --
-- to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, --
-- Boston, MA 02110-1301, USA.                                              --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 2903 $ $Date: 2008-10-08 11:42:39 +0400 (Wed, 08 Oct 2008) $
------------------------------------------------------------------------------
with Ada.Strings.Wide_Fixed;
with Ada.Strings.Wide_Maps;

package body Amoc.Version is

   Package_Version : constant Wide_String := "gpl-3.2.0-20120708-3871";
   Host_Platform   : constant Wide_String := "UNIX";

   GPL_Prefix : constant Wide_String := "gpl-";

   function Has_GPL_Prefix return Boolean;

   --------------------
   -- Has_GPL_Prefix --
   --------------------

   function Has_GPL_Prefix return Boolean is
      Prefix : constant Wide_String
        := Package_Version
            (Package_Version'First
               .. Package_Version'First + GPL_Prefix'Length - 1);

   begin
      return Prefix = GPL_Prefix;
   end Has_GPL_Prefix;

   -----------------------------
   -- Qt_Ada_Changeset_Number --
   -----------------------------

   function Qt_Ada_Changeset_Number return Wide_String is
      First : Positive := Package_Version'First;
      Last  : Natural;

   begin
      if Has_GPL_Prefix then
         First := First + GPL_Prefix'Length;
      end if;

      Ada.Strings.Wide_Fixed.Find_Token
       (Package_Version (First .. Package_Version'Last),
        Ada.Strings.Wide_Maps.To_Set ("-"),
        Ada.Strings.Outside,
        First,
        Last);

      First := Last + 2;

      Ada.Strings.Wide_Fixed.Find_Token
       (Package_Version (First .. Package_Version'Last),
        Ada.Strings.Wide_Maps.To_Set ("-"),
        Ada.Strings.Outside,
        First,
        Last);

      if Last = 0 then
         return "";
      end if;

      First := Last + 2;

      Ada.Strings.Wide_Fixed.Find_Token
       (Package_Version (First .. Package_Version'Last),
        Ada.Strings.Wide_Maps.To_Set ("-"),
        Ada.Strings.Outside,
        First,
        Last);

      return Package_Version (First .. Last);
   end Qt_Ada_Changeset_Number;

   --------------------
   -- Qt_Ada_Edition --
   --------------------

   function Qt_Ada_Edition return Wide_String is
   begin
      if Has_GPL_Prefix then
         return "GPL";

      else
         return "Professional";
      end if;
   end Qt_Ada_Edition;

   ---------------------
   -- Qt_Ada_Platform --
   ---------------------

   function Qt_Ada_Platform return Platform is
   begin
      return Platform'Wide_Value (Host_Platform);
   end Qt_Ada_Platform;

   --------------------
   -- Qt_Ada_Version --
   --------------------

   function Qt_Ada_Version return Wide_String is
      First : Positive := Package_Version'First;
      Last  : Natural;

   begin
      if Has_GPL_Prefix then
         First := First + GPL_Prefix'Length;
      end if;

      Ada.Strings.Wide_Fixed.Find_Token
       (Package_Version (First .. Package_Version'Last),
        Ada.Strings.Wide_Maps.To_Set ("-"),
        Ada.Strings.Outside,
        First,
        Last);

      return Package_Version (First .. Last);
   end Qt_Ada_Version;

end Amoc.Version;
