

with Asis.Gela.Scanners;
with Asis.Gela.Scanner_Tables;
with Gela.Character_Class_Buffers;
with Gela.Classificators;
with Asis.Gela.Classificators_Create;

with Ada.Text_IO;
with Ada.Characters.Handling;

with Asis.Gela.Elements;               use Asis.Gela.Elements;
with Asis.Gela.Elements.Assoc;         use Asis.Gela.Elements.Assoc;
with Asis.Gela.Elements.Clause;        use Asis.Gela.Elements.Clause;
with Asis.Gela.Elements.Clause.Rep;    use Asis.Gela.Elements.Clause.Rep;
with Asis.Gela.Elements.Decl;          use Asis.Gela.Elements.Decl;
with Asis.Gela.Elements.Defs;          use Asis.Gela.Elements.Defs;
with Asis.Gela.Elements.Defs.Accs;     use Asis.Gela.Elements.Defs.Accs;
with Asis.Gela.Elements.Defs.Const;    use Asis.Gela.Elements.Defs.Const;
with Asis.Gela.Elements.Defs.Formal;   use Asis.Gela.Elements.Defs.Formal;
with Asis.Gela.Elements.Defs.Types;    use Asis.Gela.Elements.Defs.Types;
with Asis.Gela.Elements.Defs.Rng;      use Asis.Gela.Elements.Defs.Rng;
with Asis.Gela.Elements.Defs.Sub;      use Asis.Gela.Elements.Defs.Sub;
with Asis.Gela.Elements.Expr;          use Asis.Gela.Elements.Expr;
with Asis.Gela.Elements.Def_Names;     use Asis.Gela.Elements.Def_Names;
with Asis.Gela.Elements.Pathes;        use Asis.Gela.Elements.Pathes;
with Asis.Gela.Elements.Stmt;          use Asis.Gela.Elements.Stmt;
with Asis.Gela.Elements.Helpers;       use Asis.Gela.Elements.Helpers;
with Asis.Gela.Units;                  use Asis.Gela.Units;
with Asis.Gela.Base_Lists;             use Asis.Gela.Base_Lists;
with Asis.Gela.Lists;                  use Asis.Gela.Lists;
with Asis.Gela.Library;

with Asis.Gela.Parser_Utils;           use Asis.Gela.Parser_Utils;

with Asis.Gela.Parser.Goto_Table;
use Asis.Gela.Parser.Goto_Table;
with Asis.Gela.Parser.Tokens;
use Asis.Gela.Parser.Tokens;
with Asis.Gela.Parser.Shift_Reduce;
use Asis.Gela.Parser.Shift_Reduce;

package body Asis.Gela.Parser is 
   package Parser_Goto renames Parser.Goto_Table;
   package Parser_Tokens renames Parser.Tokens;
   package Parser_Shift_Reduce  renames Parser.Shift_Reduce;

   package text_io renames Ada.Text_IO;

   procedure Run
     (The_Context : in     Asis.Context;
      Input       : in     Source_Buffers.Source_Buffer'Class;
      Encoding    : in     Encodings.Encoding;
      Decoder     : in     Decoders.Decoder'Class;
      Line_List   : in out Lines.Vector;
      Result      :    out Asis.Element)
   is

      Chooser : aliased Classificators.Classificator'Class :=
        Classificators_Create (Encoding, Decoder);
      Scanner : Scanners.Scanner (Chooser'Access);

      use Ada.Characters.Handling;

      Last_Compilation   : Asis.Element;
      Get_Current_Line   : Positive := 1;
      Get_Current_Column : Positive := 1;
      Line               : Lines.Line;
      Has_Comment        : Boolean := False;
      Comment            : Source_Buffers.Cursor;
      Token_Stack        : array (1 .. 300) of aliased Token_Node;
      Last_Access_Kind   : Asis.Access_Type_Kinds;
      Last_Overriding_Kind : Asis.Overriding_Indicator_Kinds;

      package Modes is
         package yy is
            value_stack  : array (1 .. 300) of Mode_Kinds;
         end yy;
      end Modes;

      function Get_Token_Value return Wide_String is
         From    : Source_Buffers.Cursor;
         To      : Source_Buffers.Cursor;
         Line    : Wide_String (1 .. 2048);
         Last    : Natural;
      begin
         Scanners.Token_Span (Scanner, From, To);
         Decoders.Decode (Decoder, From, To, Line, Last);
         return Line (1 .. Last);
      end Get_Token_Value;

      function Raw_Value return Gela_String is
         From    : Source_Buffers.Cursor;
         To      : Source_Buffers.Cursor;
      begin
         Scanners.Token_Span (Scanner, From, To);
         return (From, To);
      end Raw_Value;

      function To_String (Raw : Gela_String) return Wide_String is
         Line    : Wide_String (1 .. 2048);
         Last    : Natural;
      begin
         Decoders.Decode (Decoder, Raw.From, Raw.To, Line, Last);
         return Line (1 .. Last);
      end To_String;

      function YYLex return Tokens.Token is
         use type Source_Buffers.Cursor;
         Token : Tokens.Token;
         From  : Source_Buffers.Cursor;
         To    : Source_Buffers.Cursor;
      begin
         Scanners.Next_Token (Scanner, Token);
--text_io.put_Line (Token'Img & To_String (Get_Token_Value));
         Scanners.Token_Span (Scanner, From, To);
         Get_Current_Column :=
           Get_Current_Column + Scanners.Token_Length (Scanner);

         while Token = New_Line_Token or
           Token = Separator_Token or
           Token = Comment_Token
         loop
            if Token = New_Line_Token then
               if Has_Comment then
                  Has_Comment := False;
                  Line.Comment := Comment;
               else
                  Line.Comment := From;
               end if;

               Line.To := From;
               Lines.Vectors.Add (Line_List, Line);

               Line.From := To;
               Get_Current_Line := Get_Current_Line + 1;
               Get_Current_Column := 1;
            elsif Token = Comment_Token then
               Has_Comment := True;
               Comment := From;
            end if;

            Scanners.Next_Token (Scanner, Token);
--text_io.put_Line (Token'Img & To_String (Get_Token_Value));
            Scanners.Token_Span (Scanner, From, To);
            Get_Current_Column :=
              Get_Current_Column + Scanners.Token_Length (Scanner);
         end loop;

         if Token = Identifier_Token then
            Scanners.Enter (Scanner, Scanner_Tables.Default);
         else
            Scanners.Enter (Scanner, Scanner_Tables.Allow_Char);
         end if;

         return Token;
      end YYLex;

      pragma Inline (YYLex);

      procedure YYError (Text : String) is
         Where : constant Text_Position :=
           (Get_Current_Line, Get_Current_Column - 1);
      begin
         Report_Error
           (The_Context.all, Asis.Nil_Compilation_Unit,
            Where, "Syntax Error", Error);
      end YYError;

procedure YYParse is

   -- Rename User Defined Packages to Internal Names.
    package yy_goto_tables         renames
      Parser.Goto_Table;
    package yy_shift_reduce_tables renames
      Parser.Shift_Reduce;
    package yy_tokens              renames
      Parser.Tokens;

   use yy_tokens, yy_goto_tables, yy_shift_reduce_tables;

   procedure yyerrok;
   procedure yyclearin;


   package yy is

       -- the size of the value and state stacks
       stack_size : constant Natural := 300;

       -- subtype rule         is natural;
       subtype parse_state  is natural;
       -- subtype nonterminal  is integer;

       -- encryption constants
       default           : constant := -1;
       first_shift_entry : constant :=  0;
       accept_code       : constant := -3001;
       error_code        : constant := -3000;

       -- stack data used by the parser
       tos                : natural := 0;
       value_stack        : array(0..stack_size) of yy_tokens.yystype;
       state_stack        : array(0..stack_size) of parse_state;

       -- current input symbol and action the parser is on
       action             : integer;
       rule_id            : rule;
       input_symbol       : yy_tokens.token;


       -- error recovery flag
       error_flag : natural := 0;
          -- indicates  3 - (number of valid shifts after an error occurs)

       look_ahead : boolean := true;
       index      : integer;

       -- Is Debugging option on or off
        DEBUG : constant boolean := FALSE;

    end yy;


    function goto_state
      (state : yy.parse_state;
       sym   : nonterminal) return yy.parse_state;

    function parse_action
      (state : yy.parse_state;
       t     : yy_tokens.token) return integer;

    pragma inline(goto_state, parse_action);


    function goto_state(state : yy.parse_state;
                        sym   : nonterminal) return yy.parse_state is
        index : integer;
    begin
        index := goto_offset(state);
        while  integer(goto_matrix(index).nonterm) /= sym loop
            index := index + 1;
        end loop;
        return integer(goto_matrix(index).newstate);
    end goto_state;


    function parse_action(state : yy.parse_state;
                          t     : yy_tokens.token) return integer is
        index      : integer;
        tok_pos    : integer;
        default    : constant integer := -1;
    begin
        tok_pos := yy_tokens.token'pos(t);
        index   := shift_reduce_offset(state);
        while integer(shift_reduce_matrix(index).t) /= tok_pos and then
              integer(shift_reduce_matrix(index).t) /= default
        loop
            index := index + 1;
        end loop;
        return integer(shift_reduce_matrix(index).act);
    end parse_action;

-- error recovery stuff

    procedure handle_error is
      temp_action : integer;
    begin

      if yy.error_flag = 3 then -- no shift yet, clobber input.
      if yy.debug then
          text_io.put_line("Ayacc.YYParse: Error Recovery Clobbers " &
                   yy_tokens.token'image(yy.input_symbol));
      end if;
        if yy.input_symbol = yy_tokens.end_of_input then  -- don't discard,
        if yy.debug then
            text_io.put_line("Ayacc.YYParse: Can't discard END_OF_INPUT, quiting...");
        end if;
        raise yy_tokens.syntax_error;
        end if;

            yy.look_ahead := true;   -- get next token
        return;                  -- and try again...
    end if;

    if yy.error_flag = 0 then -- brand new error
        yyerror("Syntax Error");
    end if;

    yy.error_flag := 3;

    -- find state on stack where error is a valid shift --

    if yy.debug then
        text_io.put_line("Ayacc.YYParse: Looking for state with error as valid shift");
    end if;

    loop
        if yy.debug then
          text_io.put_line("Ayacc.YYParse: Examining State " &
               yy.parse_state'image(yy.state_stack(yy.tos)));
        end if;
        temp_action := parse_action(yy.state_stack(yy.tos), error);

            if temp_action >= yy.first_shift_entry then
                if yy.tos = yy.stack_size then
                    text_io.put_line(" Stack size exceeded on state_stack");
                    raise yy_Tokens.syntax_error;
                end if;
                yy.tos := yy.tos + 1;
                yy.state_stack(yy.tos) := temp_action;
                exit;
            end if;

        Decrement_Stack_Pointer :
        begin
          yy.tos := yy.tos - 1;
        exception
          when Constraint_Error =>
            yy.tos := 0;
        end Decrement_Stack_Pointer;

        if yy.tos = 0 then
          if yy.debug then
            text_io.put_line("Ayacc.YYParse: Error recovery popped entire stack, aborting...");
          end if;
          raise yy_tokens.syntax_error;
        end if;
    end loop;

    if yy.debug then
        text_io.put_line("Ayacc.YYParse: Shifted error token in state " &
              yy.parse_state'image(yy.state_stack(yy.tos)));
    end if;

    end handle_error;

   -- print debugging information for a shift operation
   procedure shift_debug(state_id: yy.parse_state; lexeme: yy_tokens.token) is
   begin
       text_io.put_line("Ayacc.YYParse: Shift "& yy.parse_state'image(state_id)&" on input symbol "&
               yy_tokens.token'image(lexeme) );
   end;

   -- print debugging information for a reduce operation
   procedure reduce_debug(rule_id: rule; state_id: yy.parse_state) is
   begin
       text_io.put_line("Ayacc.YYParse: Reduce by rule "&rule'image(rule_id)&" goto state "&
               yy.parse_state'image(state_id));
   end;

   -- make the parser believe that 3 valid shifts have occured.
   -- used for error recovery.
   procedure yyerrok is
   begin
       yy.error_flag := 0;
   end yyerrok;

   -- called to clear input symbol that caused an error.
   procedure yyclearin is
   begin
       -- yy.input_symbol := yylex;
       yy.look_ahead := true;
   end yyclearin;


begin
    -- initialize by pushing state 0 and getting the first input symbol
    yy.state_stack(yy.tos) := 0;


    loop

        yy.index := shift_reduce_offset(yy.state_stack(yy.tos));
        if integer(shift_reduce_matrix(yy.index).t) = yy.default then
            yy.action := integer(shift_reduce_matrix(yy.index).act);
        else
            if yy.look_ahead then
                yy.look_ahead   := false;

                yy.input_symbol := yylex;
            end if;
            yy.action :=
             parse_action(yy.state_stack(yy.tos), yy.input_symbol);
        end if;


        if yy.action >= yy.first_shift_entry then  -- SHIFT

            if yy.debug then
                shift_debug(yy.action, yy.input_symbol);
            end if;

            -- Enter new state
            if yy.tos = yy.stack_size then
                text_io.put_line(" Stack size exceeded on state_stack");
                raise yy_Tokens.syntax_error;
            end if;
            yy.tos := yy.tos + 1;
            yy.state_stack(yy.tos) := yy.action;
              yy.value_stack(yy.tos) := yylval;

        if yy.error_flag > 0 then  -- indicate a valid shift
            yy.error_flag := yy.error_flag - 1;
        end if;

            -- Advance lookahead
            yy.look_ahead := true;

        elsif yy.action = yy.error_code then       -- ERROR

            handle_error;

        elsif yy.action = yy.accept_code then
            if yy.debug then
                text_io.put_line("Ayacc.YYParse: Accepting Grammar...");
            end if;
            exit;

        else -- Reduce Action

            -- Convert action into a rule
            yy.rule_id  := -1 * yy.action;

            -- Execute User Action
            -- user_action(yy.rule_id);


                case yy.rule_id is

when  1 =>
--#line  118

   declare
      Node :  constant Identifier_Ptr := new Identifier_Node;
      Value : constant Wide_String := Get_Token_Value;
   begin
      Set_Name_Image (Node.all, Value);
      Set_Start_Position (Node.all,
        (Get_Current_Line, Get_Current_Column - Value'Length));
      Set_End_Position (Node.all,
        (Get_Current_Line, Get_Current_Column - 1));
      
yyval := YYSTYPE (Node);
   end;
   

when  2 =>
--#line  133

   declare
      Node :  constant Integer_Literal_Ptr := new Integer_Literal_Node;
      Value : constant Wide_String := Get_Token_Value;
   begin
      Set_Value_Image (Node.all, Value);
      Set_Start_Position (Node.all,
        (Get_Current_Line, Get_Current_Column - Value'Length));
      Set_End_Position (Node.all,
        (Get_Current_Line, Get_Current_Column - 1));
      
yyval := YYSTYPE (Node);
   end;
   

when  3 =>
--#line  148

   declare
      Node :  constant Real_Literal_Ptr := new Real_Literal_Node;
      Value : constant Wide_String := Get_Token_Value;
   begin
      Set_Value_Image (Node.all, Value);
      Set_Start_Position (Node.all,
        (Get_Current_Line, Get_Current_Column - Value'Length));
      Set_End_Position (Node.all,
        (Get_Current_Line, Get_Current_Column - 1));
      
yyval := YYSTYPE (Node);
   end;
   

when  4 =>
--#line  163

   declare
      Node :  constant Character_Literal_Ptr := new Character_Literal_Node;
      Value : constant Wide_String := Get_Token_Value;
   begin
      Set_Name_Image (Node.all, Value);
      Set_Start_Position (Node.all,
        (Get_Current_Line, Get_Current_Column - Value'Length));
      Set_End_Position (Node.all,
        (Get_Current_Line, Get_Current_Column - 1));
      
yyval := YYSTYPE (Node);
   end;
   

when  5 =>
--#line  178

   declare
      Node :  constant String_Literal_Ptr := new String_Literal_Node;
      Value : constant Wide_String := Get_Token_Value;
   begin
      Set_Value_Image (Node.all, Value);
      Set_Start_Position (Node.all,
        (Get_Current_Line, Get_Current_Column - Value'Length));
      Set_End_Position (Node.all,
        (Get_Current_Line, Get_Current_Column - 1));
      
yyval := YYSTYPE (Node);
   end;
   

when  6 =>
--#line  193

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  7 =>
--#line  207

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  8 =>
--#line  221

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  9 =>
--#line  235

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  10 =>
--#line  249

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  11 =>
--#line  263

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  12 =>
--#line  277

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  13 =>
--#line  291

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  14 =>
--#line  305

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  15 =>
--#line  319

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  16 =>
--#line  333

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  17 =>
--#line  347

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  18 =>
--#line  361

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  19 =>
--#line  375

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  20 =>
--#line  389

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  21 =>
--#line  403

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  22 =>
--#line  417

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  23 =>
--#line  431

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  24 =>
--#line  445

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  25 =>
--#line  459

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  26 =>
--#line  473

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  27 =>
--#line  487

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  28 =>
--#line  501

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  29 =>
--#line  515

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  30 =>
--#line  529

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  31 =>
--#line  543

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 1);
      
yyval := YYSTYPE (Node);
   end;
   

when  32 =>
--#line  557

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  33 =>
--#line  571

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  34 =>
--#line  585

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 8);
      
yyval := YYSTYPE (Node);
   end;
   

when  35 =>
--#line  599

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 6);
      
yyval := YYSTYPE (Node);
   end;
   

when  36 =>
--#line  613

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 6);
      
yyval := YYSTYPE (Node);
   end;
   

when  37 =>
--#line  627

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 7);
      
yyval := YYSTYPE (Node);
   end;
   

when  38 =>
--#line  641

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  39 =>
--#line  655

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  40 =>
--#line  669

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  41 =>
--#line  683

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  42 =>
--#line  697

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  43 =>
--#line  711

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  44 =>
--#line  725

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  45 =>
--#line  739

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 8);
      
yyval := YYSTYPE (Node);
   end;
   

when  46 =>
--#line  753

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 7);
      
yyval := YYSTYPE (Node);
   end;
   

when  47 =>
--#line  767

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  48 =>
--#line  781

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  49 =>
--#line  795

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 6);
      
yyval := YYSTYPE (Node);
   end;
   

when  50 =>
--#line  809

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  51 =>
--#line  823

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  52 =>
--#line  837

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  53 =>
--#line  851

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  54 =>
--#line  865

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  55 =>
--#line  879

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 9);
      
yyval := YYSTYPE (Node);
   end;
   

when  56 =>
--#line  893

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  57 =>
--#line  907

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  58 =>
--#line  921

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 8);
      
yyval := YYSTYPE (Node);
   end;
   

when  59 =>
--#line  935

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 7);
      
yyval := YYSTYPE (Node);
   end;
   

when  60 =>
--#line  949

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  61 =>
--#line  963

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  62 =>
--#line  977

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  63 =>
--#line  991

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 9);
      
yyval := YYSTYPE (Node);
   end;
   

when  64 =>
--#line  1005

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  65 =>
--#line  1019

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 7);
      
yyval := YYSTYPE (Node);
   end;
   

when  66 =>
--#line  1033

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  67 =>
--#line  1047

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  68 =>
--#line  1061

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  69 =>
--#line  1075

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  70 =>
--#line  1089

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  71 =>
--#line  1103

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  72 =>
--#line  1117

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 2);
      
yyval := YYSTYPE (Node);
   end;
   

when  73 =>
--#line  1131

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 6);
      
yyval := YYSTYPE (Node);
   end;
   

when  74 =>
--#line  1145

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  75 =>
--#line  1159

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 10);
      
yyval := YYSTYPE (Node);
   end;
   

when  76 =>
--#line  1173

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 7);
      
yyval := YYSTYPE (Node);
   end;
   

when  77 =>
--#line  1187

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 6);
      
yyval := YYSTYPE (Node);
   end;
   

when  78 =>
--#line  1201

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 7);
      
yyval := YYSTYPE (Node);
   end;
   

when  79 =>
--#line  1215

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 9);
      
yyval := YYSTYPE (Node);
   end;
   

when  80 =>
--#line  1229

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 9);
      
yyval := YYSTYPE (Node);
   end;
   

when  81 =>
--#line  1243

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  82 =>
--#line  1257

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  83 =>
--#line  1271

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 6);
      
yyval := YYSTYPE (Node);
   end;
   

when  84 =>
--#line  1285

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  85 =>
--#line  1299

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 7);
      
yyval := YYSTYPE (Node);
   end;
   

when  86 =>
--#line  1313

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 7);
      
yyval := YYSTYPE (Node);
   end;
   

when  87 =>
--#line  1327

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 6);
      
yyval := YYSTYPE (Node);
   end;
   

when  88 =>
--#line  1341

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 7);
      
yyval := YYSTYPE (Node);
   end;
   

when  89 =>
--#line  1355

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 6);
      
yyval := YYSTYPE (Node);
   end;
   

when  90 =>
--#line  1369

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 8);
      
yyval := YYSTYPE (Node);
   end;
   

when  91 =>
--#line  1383

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 7);
      
yyval := YYSTYPE (Node);
   end;
   

when  92 =>
--#line  1397

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 12);
      
yyval := YYSTYPE (Node);
   end;
   

when  93 =>
--#line  1411

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 6);
      
yyval := YYSTYPE (Node);
   end;
   

when  94 =>
--#line  1425

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  95 =>
--#line  1439

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 9);
      
yyval := YYSTYPE (Node);
   end;
   

when  96 =>
--#line  1453

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  97 =>
--#line  1467

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  98 =>
--#line  1481

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  99 =>
--#line  1495

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  100 =>
--#line  1509

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  101 =>
--#line  1523

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 5);
      
yyval := YYSTYPE (Node);
   end;
   

when  102 =>
--#line  1537

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 4);
      
yyval := YYSTYPE (Node);
   end;
   

when  103 =>
--#line  1551

   declare
      Node :  constant Token_Ptr := Token_Stack (yy.tos)'Unchecked_Access;
   begin
      Init_Token (Element => Node.all,
                  Line    => Get_Current_Line,
                  Column  => Get_Current_Column,
                  Image   => Raw_Value,
                  Length  => 3);
      
yyval := YYSTYPE (Node);
   end;
   

when  104 =>
--#line  1567

   declare
      New_Node : constant Pragma_Ptr :=
        new Pragma_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Pragma_Name_Image (New_Node.all, Name_Image (Identifier_Ptr (
yy.value_stack(yy.tos-2)).all));
      Set_Pragma_Argument_Associations (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  105 =>
--#line  1580

   declare
      New_Node : constant Pragma_Ptr :=
        new Pragma_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Pragma_Name_Image (New_Node.all, Name_Image (Identifier_Ptr (
yy.value_stack(yy.tos-1)).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  106 =>
--#line  1595

   declare
      New_Node : constant Pragma_Argument_Association_Ptr :=
        new Pragma_Argument_Association_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Formal_Parameter (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Actual_Parameter (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  107 =>
--#line  1608

   declare
      New_Node : constant Pragma_Argument_Association_Ptr :=
        new Pragma_Argument_Association_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Actual_Parameter (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  108 =>
--#line  1623
 
yyval := 
yy.value_stack(yy.tos);

when  109 =>
--#line  1625
 
yyval := 
yy.value_stack(yy.tos);

when  110 =>
--#line  1627
 
yyval := 
yy.value_stack(yy.tos);

when  111 =>
--#line  1629
 
yyval := 
yy.value_stack(yy.tos);

when  112 =>
--#line  1631
 
yyval := 
yy.value_stack(yy.tos);

when  113 =>
--#line  1633
 
yyval := 
yy.value_stack(yy.tos);

when  114 =>
--#line  1635
 
yyval := 
yy.value_stack(yy.tos);

when  115 =>
--#line  1637
 
yyval := 
yy.value_stack(yy.tos);

when  116 =>
--#line  1639
 
yyval := 
yy.value_stack(yy.tos);

when  117 =>
--#line  1641
 
yyval := 
yy.value_stack(yy.tos);

when  118 =>
--#line  1643
 
yyval := 
yy.value_stack(yy.tos);

when  119 =>
--#line  1645
 
yyval := 
yy.value_stack(yy.tos);

when  120 =>
--#line  1650

   declare
      New_Node : constant Defining_Identifier_Ptr :=
        new Defining_Identifier_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Defining_Name_Image (New_Node.all, Name_Image (Identifier_Ptr (
yy.value_stack(yy.tos)).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  121 =>
--#line  1665
 
yyval := 
yy.value_stack(yy.tos);

when  122 =>
--#line  1667
 
yyval := 
yy.value_stack(yy.tos);

when  123 =>
--#line  1669
 
yyval := 
yy.value_stack(yy.tos);

when  124 =>
--#line  1671
 
yyval := 
yy.value_stack(yy.tos);

when  125 =>
--#line  1677

   declare
      New_Node : constant Ordinary_Type_Declaration_Ptr :=
        new Ordinary_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Discriminant_Part (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  126 =>
--#line  1691

   declare
      New_Node : constant Ordinary_Type_Declaration_Ptr :=
        new Ordinary_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  127 =>
--#line  1704
 
yyval := 
yy.value_stack(yy.tos);

when  128 =>
--#line  1706
 
yyval := 
yy.value_stack(yy.tos);

when  129 =>
--#line  1711
 
yyval := 
yy.value_stack(yy.tos);

when  130 =>
--#line  1713
 
yyval := 
yy.value_stack(yy.tos);

when  131 =>
--#line  1715
 
yyval := 
yy.value_stack(yy.tos);

when  132 =>
--#line  1717
 
yyval := 
yy.value_stack(yy.tos);

when  133 =>
--#line  1719
 
yyval := 
yy.value_stack(yy.tos);

when  134 =>
--#line  1721
 
yyval := 
yy.value_stack(yy.tos);

when  135 =>
--#line  1723
 
yyval := 
yy.value_stack(yy.tos);

when  136 =>
--#line  1725
 
yyval := 
yy.value_stack(yy.tos);

when  137 =>
--#line  1730

   declare
      New_Node : constant Subtype_Declaration_Ptr :=
        new Subtype_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  138 =>
--#line  1746

   declare
      New_Node : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Null_Exclusion (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Subtype_Constraint (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  139 =>
--#line  1760

   declare
      New_Node : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Null_Exclusion (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  140 =>
--#line  1773

   declare
      New_Node : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Subtype_Constraint (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  141 =>
--#line  1786

   declare
      New_Node : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  142 =>
--#line  1801
 
yyval := 
yy.value_stack(yy.tos);

when  143 =>
--#line  1806
 
yyval := 
yy.value_stack(yy.tos);

when  144 =>
--#line  1808
 
yyval := 
yy.value_stack(yy.tos);

when  145 =>
--#line  1813
 
yyval := 
yy.value_stack(yy.tos);

when  146 =>
--#line  1815
 
yyval := 
yy.value_stack(yy.tos);

when  147 =>
--#line  1817
 
yyval := 
yy.value_stack(yy.tos);

when  148 =>
--#line  1822
 
yyval := 
yy.value_stack(yy.tos);

when  149 =>
--#line  1824
 
yyval := 
yy.value_stack(yy.tos);

when  150 =>
--#line  1830

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-6));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  151 =>
--#line  1846

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  152 =>
--#line  1861

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  153 =>
--#line  1877

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  154 =>
--#line  1892

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  155 =>
--#line  1907

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  156 =>
--#line  1921

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  157 =>
--#line  1935

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  158 =>
--#line  1949

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-6));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  159 =>
--#line  1965

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  160 =>
--#line  1980

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  161 =>
--#line  1996

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  162 =>
--#line  2011

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  163 =>
--#line  2026

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  164 =>
--#line  2040

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  165 =>
--#line  2054

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  166 =>
--#line  2068

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-6));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  167 =>
--#line  2084

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  168 =>
--#line  2099

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  169 =>
--#line  2115

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  170 =>
--#line  2130

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  171 =>
--#line  2145

   declare
      New_Node : constant Constant_Declaration_Ptr :=
        new Constant_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  172 =>
--#line  2159

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  173 =>
--#line  2174

   declare
      New_Node : constant Variable_Declaration_Ptr :=
        new Variable_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  174 =>
--#line  2187
 
yyval := 
yy.value_stack(yy.tos);

when  175 =>
--#line  2189
 
yyval := 
yy.value_stack(yy.tos);

when  176 =>
--#line  2194

   declare
      Wrap1 : constant Primary_Defining_Name_Lists.List :=
        Primary_Defining_Name_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Defining_Name_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  177 =>
--#line  2204

   declare
      Wrap1 : constant Primary_Defining_Name_Lists.List :=
        new Primary_Defining_Name_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Defining_Name_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  178 =>
--#line  2218

   declare
      New_Node : constant Integer_Number_Declaration_Ptr :=
        new Integer_Number_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  179 =>
--#line  2235

   declare
      New_Node : constant Derived_Record_Extension_Ptr :=
        new Derived_Record_Extension_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Has_Limited (New_Node.all, True);
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  180 =>
--#line  2252

   declare
      New_Node : constant Derived_Record_Extension_Ptr :=
        new Derived_Record_Extension_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Has_Limited (New_Node.all, True);
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  181 =>
--#line  2269

   declare
      New_Node : constant Derived_Record_Extension_Ptr :=
        new Derived_Record_Extension_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  182 =>
--#line  2285

   declare
      New_Node : constant Derived_Record_Extension_Ptr :=
        new Derived_Record_Extension_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  183 =>
--#line  2301

   declare
      New_Node : constant Derived_Record_Extension_Ptr :=
        new Derived_Record_Extension_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Has_Limited (New_Node.all, True);
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  184 =>
--#line  2316

   declare
      New_Node : constant Derived_Record_Extension_Ptr :=
        new Derived_Record_Extension_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Has_Limited (New_Node.all, True);
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  185 =>
--#line  2331

   declare
      New_Node : constant Derived_Record_Extension_Ptr :=
        new Derived_Record_Extension_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  186 =>
--#line  2345

   declare
      New_Node : constant Derived_Record_Extension_Ptr :=
        new Derived_Record_Extension_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  187 =>
--#line  2358

   declare
      New_Node : constant Derived_Type_Ptr :=
        new Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Has_Limited (New_Node.all, True);
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  188 =>
--#line  2373

   declare
      New_Node : constant Derived_Type_Ptr :=
        new Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  189 =>
--#line  2387

   declare
      New_Node : constant Derived_Type_Ptr :=
        new Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Limited (New_Node.all, True);
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  190 =>
--#line  2400

   declare
      New_Node : constant Derived_Type_Ptr :=
        new Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Parent_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  191 =>
--#line  2415

   declare
      Wrap1 : constant Constraint_Ptr :=
        Constraint_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
   end;


when  192 =>
--#line  2428
 
yyval := 
yy.value_stack(yy.tos);

when  193 =>
--#line  2430

   declare
      New_Node : constant Simple_Expression_Range_Ptr :=
        new Simple_Expression_Range_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Lower_Bound (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Upper_Bound (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  194 =>
--#line  2447

   declare
      Wrap1 : constant Enumeration_Type_Ptr :=
        new Enumeration_Type_Node;
      Wrap2 : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Primary_Declaration_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Enumeration_Literal_Declarations (Wrap1.all, T (Wrap2));
   end;


when  195 =>
--#line  2463

   declare
      Wrap1 : constant Enumeration_Type_Ptr :=
        new Enumeration_Type_Node;
      Wrap2 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Primary_Declaration_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Enumeration_Literal_Declarations (Wrap1.all, T (Wrap2));
   end;


when  196 =>
--#line  2481

   declare
      Wrap1 : constant Enumeration_Literal_Specification_Ptr :=
        new Enumeration_Literal_Specification_Node;
      Wrap2 : constant Defining_Enumeration_Literal_Ptr :=
        new Defining_Enumeration_Literal_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Defining_Name_Image (Wrap2.all, Defining_Name_Image (Defining_Identifier_Ptr (
yy.value_stack(yy.tos)).all));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Name (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  197 =>
--#line  2498

   declare
      New_Node : constant Enumeration_Literal_Specification_Ptr :=
        new Enumeration_Literal_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  198 =>
--#line  2513

   declare
      New_Node : constant Defining_Character_Literal_Ptr :=
        new Defining_Character_Literal_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Defining_Name_Image (New_Node.all, Name_Image (Character_Literal_Ptr (
yy.value_stack(yy.tos)).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  199 =>
--#line  2528
 
yyval := 
yy.value_stack(yy.tos);

when  200 =>
--#line  2530
 
yyval := 
yy.value_stack(yy.tos);

when  201 =>
--#line  2535

   declare
      New_Node : constant Signed_Integer_Type_Ptr :=
        new Signed_Integer_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Integer_Constraint (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  202 =>
--#line  2550

   declare
      New_Node : constant Modular_Type_Ptr :=
        new Modular_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Mod_Static_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  203 =>
--#line  2565
 
yyval := 
yy.value_stack(yy.tos);

when  204 =>
--#line  2567
 
yyval := 
yy.value_stack(yy.tos);

when  205 =>
--#line  2572

   declare
      New_Node : constant Floating_Point_Ptr :=
        new Floating_Point_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Digits_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Real_Range_Constraint (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  206 =>
--#line  2585

   declare
      New_Node : constant Floating_Point_Ptr :=
        new Floating_Point_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Digits_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  207 =>
--#line  2600

   declare
      New_Node : constant Simple_Expression_Range_Ptr :=
        new Simple_Expression_Range_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Lower_Bound (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Upper_Bound (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  208 =>
--#line  2616
 
yyval := 
yy.value_stack(yy.tos);

when  209 =>
--#line  2618
 
yyval := 
yy.value_stack(yy.tos);

when  210 =>
--#line  2623

   declare
      New_Node : constant Ordinary_Fixed_Point_Ptr :=
        new Ordinary_Fixed_Point_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Delta_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Real_Range_Constraint (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  211 =>
--#line  2639

   declare
      New_Node : constant Decimal_Fixed_Point_Ptr :=
        new Decimal_Fixed_Point_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Delta_Expression (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Digits_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Real_Range_Constraint (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  212 =>
--#line  2653

   declare
      New_Node : constant Decimal_Fixed_Point_Ptr :=
        new Decimal_Fixed_Point_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Delta_Expression (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Digits_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  213 =>
--#line  2669

   declare
      New_Node : constant Digits_Constraint_Ptr :=
        new Digits_Constraint_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Digits_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Real_Range_Constraint (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  214 =>
--#line  2682

   declare
      New_Node : constant Digits_Constraint_Ptr :=
        new Digits_Constraint_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Digits_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  215 =>
--#line  2697
 
yyval := 
yy.value_stack(yy.tos);

when  216 =>
--#line  2699
 
yyval := 
yy.value_stack(yy.tos);

when  217 =>
--#line  2705

   declare
      Wrap1 : constant Unconstrained_Array_Ptr :=
        new Unconstrained_Array_Node;
      Wrap2 : constant Primary_Identifier_Lists.List :=
        Primary_Identifier_Lists.List (
yy.value_stack(yy.tos-3));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Primary_Identifier_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Array_Component_Definition (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Index_Subtype_Definitions (Wrap1.all, T (Wrap2));
   end;


when  218 =>
--#line  2722

   declare
      Wrap1 : constant Unconstrained_Array_Ptr :=
        new Unconstrained_Array_Node;
      Wrap2 : constant Primary_Identifier_Lists.List :=
        new Primary_Identifier_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Primary_Identifier_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Array_Component_Definition (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Index_Subtype_Definitions (Wrap1.all, T (Wrap2));
   end;


when  219 =>
--#line  2741
 
yyval := 
yy.value_stack(yy.tos-2);

when  220 =>
--#line  2748

   declare
      Wrap1 : constant Constrained_Array_Ptr :=
        new Constrained_Array_Node;
      Wrap2 : constant Primary_Definition_Lists.List :=
        Primary_Definition_Lists.List (
yy.value_stack(yy.tos-3));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Primary_Definition_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Array_Component_Definition (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Discrete_Subtype_Definitions (Wrap1.all, T (Wrap2));
   end;


when  221 =>
--#line  2765

   declare
      Wrap1 : constant Constrained_Array_Ptr :=
        new Constrained_Array_Node;
      Wrap2 : constant Primary_Definition_Lists.List :=
        new Primary_Definition_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Primary_Definition_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Array_Component_Definition (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Discrete_Subtype_Definitions (Wrap1.all, T (Wrap2));
   end;


when  222 =>
--#line  2784

   declare
      New_Node : constant S_Discrete_Subtype_Indication_Ptr :=
        new S_Discrete_Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  223 =>
--#line  2796

   declare
      New_Node : constant S_Discrete_Range_Attribute_Reference_Ptr :=
        new S_Discrete_Range_Attribute_Reference_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Range_Attribute (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  224 =>
--#line  2808

   declare
      New_Node : constant S_Discrete_Simple_Expression_Range_Ptr :=
        new S_Discrete_Simple_Expression_Range_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Lower_Bound (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Upper_Bound (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  225 =>
--#line  2824

   declare
      New_Node : constant Component_Definition_Ptr :=
        new Component_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Component_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  226 =>
--#line  2837

   declare
      New_Node : constant Component_Definition_Ptr :=
        new Component_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Component_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  227 =>
--#line  2849

   declare
      New_Node : constant Component_Definition_Ptr :=
        new Component_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Trait_Kind (New_Node.all, An_Aliased_Trait);
      Set_Component_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  228 =>
--#line  2862

   declare
      New_Node : constant Component_Definition_Ptr :=
        new Component_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Component_Subtype_Indication (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  229 =>
--#line  2878
 
yyval := 
yy.value_stack(yy.tos-2);

when  230 =>
--#line  2880
 
yyval := 
yy.value_stack(yy.tos-1);

when  231 =>
--#line  2885

   declare
      New_Node : constant Discrete_Subtype_Indication_Ptr :=
        new Discrete_Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Subtype_Constraint (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  232 =>
--#line  2898

   declare
      New_Node : constant Discrete_Range_Attribute_Reference_Ptr :=
        new Discrete_Range_Attribute_Reference_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Range_Attribute (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  233 =>
--#line  2910

   declare
      New_Node : constant Discrete_Simple_Expression_Range_Ptr :=
        new Discrete_Simple_Expression_Range_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Lower_Bound (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Upper_Bound (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  234 =>
--#line  2926
 
yyval := 
yy.value_stack(yy.tos);

when  235 =>
--#line  2928
 
yyval := 
yy.value_stack(yy.tos);

when  236 =>
--#line  2933

   declare
      New_Node : constant Unknown_Discriminant_Part_Ptr :=
        new Unknown_Discriminant_Part_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  237 =>
--#line  2948

   declare
      Wrap1 : constant Known_Discriminant_Part_Ptr :=
        new Known_Discriminant_Part_Node;
      Wrap2 : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Primary_Declaration_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Discriminants (Wrap1.all, T (Wrap2));
   end;


when  238 =>
--#line  2963

   declare
      Wrap1 : constant Known_Discriminant_Part_Ptr :=
        new Known_Discriminant_Part_Node;
      Wrap2 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Primary_Declaration_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Discriminants (Wrap1.all, T (Wrap2));
   end;


when  239 =>
--#line  2982

   declare
      Wrap1 : constant Discriminant_Specification_Ptr :=
        new Discriminant_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Initialization_Expression (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  240 =>
--#line  3002

   declare
      Wrap1 : constant Discriminant_Specification_Ptr :=
        new Discriminant_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  241 =>
--#line  3022

   declare
      Wrap1 : constant Discriminant_Specification_Ptr :=
        new Discriminant_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Initialization_Expression (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  242 =>
--#line  3041

   declare
      Wrap1 : constant Discriminant_Specification_Ptr :=
        new Discriminant_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  243 =>
--#line  3060

   declare
      New_Node : constant Discriminant_Specification_Ptr :=
        new Discriminant_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Trait_Kind (New_Node.all, An_Access_Definition_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  244 =>
--#line  3075

   declare
      New_Node : constant Discriminant_Specification_Ptr :=
        new Discriminant_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, An_Access_Definition_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  245 =>
--#line  3092
 
yyval := 
yy.value_stack(yy.tos);

when  246 =>
--#line  3098
 
yyval := 
yy.value_stack(yy.tos-2);

when  247 =>
--#line  3100
 
yyval := 
yy.value_stack(yy.tos-1);

when  248 =>
--#line  3105
 
yyval := 
yy.value_stack(yy.tos);

when  249 =>
--#line  3107
 
yyval := 
yy.value_stack(yy.tos);

when  250 =>
--#line  3112

   declare
      New_Node : constant Tagged_Record_Type_Ptr :=
        new Tagged_Record_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Has_Tagged (New_Node.all, True);
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  251 =>
--#line  3129

   declare
      New_Node : constant Tagged_Record_Type_Ptr :=
        new Tagged_Record_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Has_Tagged (New_Node.all, True);
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  252 =>
--#line  3144

   declare
      New_Node : constant Tagged_Record_Type_Ptr :=
        new Tagged_Record_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Tagged (New_Node.all, True);
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  253 =>
--#line  3159

   declare
      New_Node : constant Tagged_Record_Type_Ptr :=
        new Tagged_Record_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Tagged (New_Node.all, True);
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  254 =>
--#line  3172

   declare
      New_Node : constant Record_Type_Ptr :=
        new Record_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  255 =>
--#line  3186

   declare
      New_Node : constant Record_Type_Ptr :=
        new Record_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Record_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  256 =>
--#line  3201

   declare
      New_Node : constant Record_Definition_Ptr :=
        new Record_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Record_Components (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  257 =>
--#line  3213

   declare
      New_Node : constant Null_Record_Definition_Ptr :=
        new Null_Record_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  258 =>
--#line  3227

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Declaration_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  259 =>
--#line  3237

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  260 =>
--#line  3246

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Declaration_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  261 =>
--#line  3256

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  262 =>
--#line  3265

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
      Wrap2 : constant Null_Component_Ptr :=
        new Null_Component_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Pragmas (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Declaration_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  263 =>
--#line  3280

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
      Wrap2 : constant Null_Component_Ptr :=
        new Null_Component_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Declaration_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  264 =>
--#line  3297
 
yyval := 
yy.value_stack(yy.tos);

when  265 =>
--#line  3299
 
yyval := 
yy.value_stack(yy.tos);

when  266 =>
--#line  3305

   declare
      New_Node : constant Component_Declaration_Ptr :=
        new Component_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  267 =>
--#line  3320

   declare
      New_Node : constant Component_Declaration_Ptr :=
        new Component_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  268 =>
--#line  3337

   declare
      Wrap1 : constant Variant_Part_Ptr :=
        new Variant_Part_Node;
      Wrap2 : constant Primary_Variant_Lists.List :=
        Primary_Variant_Lists.List (
yy.value_stack(yy.tos-3));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Discriminant_Direct_Name (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Pragmas (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Primary_Variant_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Variants (Wrap1.all, T (Wrap2));
   end;


when  269 =>
--#line  3354

   declare
      Wrap1 : constant Variant_Part_Ptr :=
        new Variant_Part_Node;
      Wrap2 : constant Primary_Variant_Lists.List :=
        new Primary_Variant_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Discriminant_Direct_Name (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Pragmas (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Primary_Variant_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Variants (Wrap1.all, T (Wrap2));
   end;


when  270 =>
--#line  3371

   declare
      Wrap1 : constant Variant_Part_Ptr :=
        new Variant_Part_Node;
      Wrap2 : constant Primary_Variant_Lists.List :=
        Primary_Variant_Lists.List (
yy.value_stack(yy.tos-3));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Discriminant_Direct_Name (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Primary_Variant_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Variants (Wrap1.all, T (Wrap2));
   end;


when  271 =>
--#line  3387

   declare
      Wrap1 : constant Variant_Part_Ptr :=
        new Variant_Part_Node;
      Wrap2 : constant Primary_Variant_Lists.List :=
        new Primary_Variant_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Discriminant_Direct_Name (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Primary_Variant_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Variants (Wrap1.all, T (Wrap2));
   end;


when  272 =>
--#line  3406

   declare
      New_Node : constant Variant_Ptr :=
        new Variant_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Variant_Choices (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Record_Components (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  273 =>
--#line  3422

   declare
      Wrap1 : constant Primary_Choise_Lists.List :=
        Primary_Choise_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Choise_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  274 =>
--#line  3432

   declare
      Wrap1 : constant Primary_Choise_Lists.List :=
        new Primary_Choise_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Choise_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  275 =>
--#line  3445
 
yyval := 
yy.value_stack(yy.tos);

when  276 =>
--#line  3447
 
yyval := 
yy.value_stack(yy.tos);

when  277 =>
--#line  3449

   declare
      New_Node : constant Others_Choice_Ptr :=
        new Others_Choice_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  278 =>
--#line  3463
 
yyval := 
yy.value_stack(yy.tos);

when  279 =>
--#line  3469

   if 
yy.value_stack(yy.tos-3).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Declaration_Ptr :=
           new Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
         Set_Has_Abstract (New_Node.all, True);
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Declaration_Ptr :=
           new Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
         Set_Has_Abstract (New_Node.all, True);
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  280 =>
--#line  3499

   if 
yy.value_stack(yy.tos-3).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Declaration_Ptr :=
           new Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
         Set_Has_Abstract (New_Node.all, True);
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Declaration_Ptr :=
           new Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
         Set_Has_Abstract (New_Node.all, True);
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  281 =>
--#line  3530

   declare
      New_Node : constant Interface_Type_Ptr :=
        new Interface_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Synchronized (New_Node.all, True);
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  282 =>
--#line  3543

   declare
      New_Node : constant Interface_Type_Ptr :=
        new Interface_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Synchronized (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  283 =>
--#line  3555

   declare
      New_Node : constant Interface_Type_Ptr :=
        new Interface_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Protected (New_Node.all, True);
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  284 =>
--#line  3568

   declare
      New_Node : constant Interface_Type_Ptr :=
        new Interface_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Protected (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  285 =>
--#line  3580

   declare
      New_Node : constant Interface_Type_Ptr :=
        new Interface_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Task (New_Node.all, True);
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  286 =>
--#line  3593

   declare
      New_Node : constant Interface_Type_Ptr :=
        new Interface_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Task (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  287 =>
--#line  3605

   declare
      New_Node : constant Interface_Type_Ptr :=
        new Interface_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Limited (New_Node.all, True);
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  288 =>
--#line  3618

   declare
      New_Node : constant Interface_Type_Ptr :=
        new Interface_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Limited (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  289 =>
--#line  3630

   declare
      New_Node : constant Interface_Type_Ptr :=
        new Interface_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  290 =>
--#line  3642

   declare
      New_Node : constant Interface_Type_Ptr :=
        new Interface_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  291 =>
--#line  3656

   declare
      Wrap1 : constant Primary_Expression_Lists.List :=
        Primary_Expression_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Expression_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  292 =>
--#line  3666

   declare
      Wrap1 : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Expression_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  293 =>
--#line  3679

   declare
      Wrap1 : constant Access_Type_Ptr :=
        Access_Type_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Null_Exclusion (Wrap1.all, True);
   end;


when  294 =>
--#line  3690

   declare
      Wrap1 : constant Access_Type_Ptr :=
        Access_Type_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  295 =>
--#line  3699

   declare
      Wrap1 : constant Access_Type_Ptr :=
        Access_Type_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Null_Exclusion (Wrap1.all, True);
   end;


when  296 =>
--#line  3710

   declare
      Wrap1 : constant Access_Type_Ptr :=
        Access_Type_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  297 =>
--#line  3722

   declare
      New_Node : constant Access_Type_Ptr :=
        new Access_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Access_Type_Kind (New_Node.all, Last_Access_Kind);
      Set_Access_To_Object_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  298 =>
--#line  3735

   declare
      New_Node : constant Access_Type_Ptr :=
        new Access_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Access_To_Object_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  299 =>
--#line  3750

   declare
      Wrap1 : constant Access_Type_Ptr :=
        new Access_Type_Node;
      Wrap2 : constant Primary_Parameter_Lists.List :=
        Primary_Parameter_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Access_Type_Kind (Wrap1.all, An_Access_To_Protected_Procedure);
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Access_To_Subprogram_Parameter_Profile (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  300 =>
--#line  3767

   declare
      Wrap1 : constant Access_Type_Ptr :=
        new Access_Type_Node;
      Wrap2 : constant Primary_Parameter_Lists.List :=
        Primary_Parameter_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Access_To_Subprogram_Parameter_Profile (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  301 =>
--#line  3783

   declare
      New_Node : constant Access_Type_Ptr :=
        new Access_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Access_Type_Kind (New_Node.all, An_Access_To_Protected_Function);
      Set_Access_To_Function_Result_Subtype (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  302 =>
--#line  3796

   declare
      New_Node : constant Access_Type_Ptr :=
        new Access_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Access_To_Function_Result_Subtype (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  304 =>
--#line  3815

   declare
      New_Node : constant Anonymous_Access_To_Constant_Ptr :=
        new Anonymous_Access_To_Constant_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Has_Null_Exclusion (New_Node.all, True);
      Set_Anonymous_Access_To_Object_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  305 =>
--#line  3828

   declare
      New_Node : constant Anonymous_Access_To_Variable_Ptr :=
        new Anonymous_Access_To_Variable_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Null_Exclusion (New_Node.all, True);
      Set_Anonymous_Access_To_Object_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  306 =>
--#line  3841

   declare
      New_Node : constant Anonymous_Access_To_Constant_Ptr :=
        new Anonymous_Access_To_Constant_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Anonymous_Access_To_Object_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  307 =>
--#line  3853

   declare
      New_Node : constant Anonymous_Access_To_Variable_Ptr :=
        new Anonymous_Access_To_Variable_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Anonymous_Access_To_Object_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  308 =>
--#line  3865

   declare
      New_Node : constant Anonymous_Access_To_Protected_Procedure_Ptr :=
        new Anonymous_Access_To_Protected_Procedure_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Has_Null_Exclusion (New_Node.all, True);
      Set_Access_To_Subprogram_Parameter_Profile (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  309 =>
--#line  3878

   declare
      New_Node : constant Anonymous_Access_To_Procedure_Ptr :=
        new Anonymous_Access_To_Procedure_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Has_Null_Exclusion (New_Node.all, True);
      Set_Access_To_Subprogram_Parameter_Profile (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  310 =>
--#line  3891

   declare
      New_Node : constant Anonymous_Access_To_Protected_Procedure_Ptr :=
        new Anonymous_Access_To_Protected_Procedure_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Access_To_Subprogram_Parameter_Profile (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  311 =>
--#line  3903

   declare
      New_Node : constant Anonymous_Access_To_Procedure_Ptr :=
        new Anonymous_Access_To_Procedure_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Access_To_Subprogram_Parameter_Profile (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  312 =>
--#line  3915

   declare
      New_Node : constant Anonymous_Access_To_Protected_Function_Ptr :=
        new Anonymous_Access_To_Protected_Function_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Has_Null_Exclusion (New_Node.all, True);
      Set_Access_To_Function_Result_Subtype (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  313 =>
--#line  3928

   declare
      New_Node : constant Anonymous_Access_To_Function_Ptr :=
        new Anonymous_Access_To_Function_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Has_Null_Exclusion (New_Node.all, True);
      Set_Access_To_Function_Result_Subtype (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  314 =>
--#line  3941

   declare
      New_Node : constant Anonymous_Access_To_Protected_Function_Ptr :=
        new Anonymous_Access_To_Protected_Function_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Access_To_Function_Result_Subtype (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  315 =>
--#line  3953

   declare
      New_Node : constant Anonymous_Access_To_Function_Ptr :=
        new Anonymous_Access_To_Function_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Access_To_Function_Result_Subtype (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  316 =>
--#line  3969

   declare
      New_Node : constant Incomplete_Type_Declaration_Ptr :=
        new Incomplete_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Discriminant_Part (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  317 =>
--#line  3983

   declare
      New_Node : constant Incomplete_Type_Declaration_Ptr :=
        new Incomplete_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Discriminant_Part (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  318 =>
--#line  3996

   declare
      New_Node : constant Incomplete_Type_Declaration_Ptr :=
        new Incomplete_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  319 =>
--#line  4009

   declare
      New_Node : constant Incomplete_Type_Declaration_Ptr :=
        new Incomplete_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  320 =>
--#line  4024

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  321 =>
--#line  4033

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  322 =>
--#line  4045
 
yyval := 
yy.value_stack(yy.tos);

when  323 =>
--#line  4047
 
yyval := 
yy.value_stack(yy.tos);

when  324 =>
--#line  4052
 
yyval := 
yy.value_stack(yy.tos);

when  325 =>
--#line  4054
 
yyval := 
yy.value_stack(yy.tos);

when  326 =>
--#line  4056
 
yyval := 
yy.value_stack(yy.tos);

when  327 =>
--#line  4061
 
yyval := 
yy.value_stack(yy.tos);

when  328 =>
--#line  4063
 
yyval := 
yy.value_stack(yy.tos);

when  329 =>
--#line  4068
 
yyval := 
yy.value_stack(yy.tos);

when  330 =>
--#line  4070
 
yyval := 
yy.value_stack(yy.tos);

when  331 =>
--#line  4072
 
yyval := 
yy.value_stack(yy.tos);

when  332 =>
--#line  4074
 
yyval := 
yy.value_stack(yy.tos);

when  333 =>
--#line  4079
 
yyval := 
yy.value_stack(yy.tos);

when  334 =>
--#line  4081
 
yyval := 
yy.value_stack(yy.tos);

when  335 =>
--#line  4083
 
yyval := 
yy.value_stack(yy.tos);

when  336 =>
--#line  4085
 
yyval := 
yy.value_stack(yy.tos);

when  337 =>
--#line  4087
 
yyval := 
yy.value_stack(yy.tos);

when  338 =>
--#line  4089
 
yyval := 
yy.value_stack(yy.tos);

when  339 =>
--#line  4094
 
yyval := 
yy.value_stack(yy.tos);

when  340 =>
--#line  4096
 
yyval := 
yy.value_stack(yy.tos);

when  341 =>
--#line  4101
 
yyval := 
yy.value_stack(yy.tos);

when  342 =>
--#line  4106

   declare
      New_Node : constant Explicit_Dereference_Ptr :=
        new Explicit_Dereference_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Prefix (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  343 =>
--#line  4121
 
yyval := 
yy.value_stack(yy.tos);

when  344 =>
--#line  4127
 
yyval := 
yy.value_stack(yy.tos-4);

when  345 =>
--#line  4129
 
yyval := 
yy.value_stack(yy.tos-3);

when  346 =>
--#line  4134
 
yyval := 
yy.value_stack(yy.tos-3);

when  347 =>
--#line  4139

   declare
      New_Node : constant Selected_Component_Ptr :=
        new Selected_Component_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Prefix (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Selector (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  348 =>
--#line  4155
 
yyval := 
yy.value_stack(yy.tos);

when  349 =>
--#line  4157
 
yyval := 
yy.value_stack(yy.tos);

when  350 =>
--#line  4159
 
yyval := 
yy.value_stack(yy.tos);

when  351 =>
--#line  4164

   declare
      New_Node : constant Attribute_Reference_Ptr :=
        new Attribute_Reference_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Prefix (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Attribute_Designator_Identifier (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  352 =>
--#line  4180
 
yyval := 
yy.value_stack(yy.tos);

when  353 =>
--#line  4182

   declare
      Wrap1 : constant Identifier_Ptr :=
        new Identifier_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (Wrap1.all, To_String (Raw_Image (
yy.value_stack(yy.tos).all)));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  354 =>
--#line  4194

   declare
      Wrap1 : constant Identifier_Ptr :=
        new Identifier_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (Wrap1.all, To_String (Raw_Image (
yy.value_stack(yy.tos).all)));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  355 =>
--#line  4206

   declare
      Wrap1 : constant Identifier_Ptr :=
        new Identifier_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (Wrap1.all, To_String (Raw_Image (
yy.value_stack(yy.tos).all)));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  356 =>
--#line  4218

   declare
      Wrap1 : constant Identifier_Ptr :=
        new Identifier_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (Wrap1.all, To_String (Raw_Image (
yy.value_stack(yy.tos).all)));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  357 =>
--#line  4233

   declare
      Wrap1 : constant Range_Attribute_Reference_Ptr :=
        new Range_Attribute_Reference_Node;
      Wrap2 : constant Attribute_Reference_Ptr :=
        Attribute_Reference_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Prefix (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Range_Attribute (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  358 =>
--#line  4252

   declare
      Wrap1 : constant Attribute_Reference_Ptr :=
        new Attribute_Reference_Node;
      Wrap2 : constant Identifier_Ptr :=
        new Identifier_Node;
      Wrap3 : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Name_Image (Wrap2.all, To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Primary_Expression_Lists.Add (Wrap3.all, 
yy.value_stack(yy.tos));
      Set_Attribute_Designator_Expressions (Wrap1.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Attribute_Designator_Identifier (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  359 =>
--#line  4273

   declare
      Wrap1 : constant Attribute_Reference_Ptr :=
        new Attribute_Reference_Node;
      Wrap2 : constant Identifier_Ptr :=
        new Identifier_Node;
      Wrap3 : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (Wrap2.all, To_String (Raw_Image (
yy.value_stack(yy.tos).all)));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Attribute_Designator_Expressions (Wrap1.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Attribute_Designator_Identifier (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  360 =>
--#line  4296
 
yyval := 
yy.value_stack(yy.tos);

when  361 =>
--#line  4298
 
yyval := 
yy.value_stack(yy.tos);

when  362 =>
--#line  4304

   declare
      New_Node : constant Record_Aggregate_Ptr :=
        new Record_Aggregate_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Record_Component_Associations (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  363 =>
--#line  4319

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        Primary_Association_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  364 =>
--#line  4329

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  365 =>
--#line  4339

   declare
      New_Node : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  366 =>
--#line  4353

   declare
      New_Node : constant Record_Component_Association_Ptr :=
        new Record_Component_Association_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Record_Component_Choices (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Component_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  367 =>
--#line  4366

   declare
      New_Node : constant Record_Component_Association_Ptr :=
        new Record_Component_Association_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Component_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  368 =>
--#line  4378

   declare
      New_Node : constant Record_Component_Association_Ptr :=
        new Record_Component_Association_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Component_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  369 =>
--#line  4390

   declare
      Wrap1 : constant Record_Component_Association_Ptr :=
        new Record_Component_Association_Node;
      Wrap2 : constant Box_Expression_Ptr :=
        new Box_Expression_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Record_Component_Choices (Wrap1.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Component_Expression (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  370 =>
--#line  4411
 
yyval := 
yy.value_stack(yy.tos-1);

when  371 =>
--#line  4413
 
yyval := 
yy.value_stack(yy.tos);

when  373 =>
--#line  4420

   declare
      New_Node : constant Extension_Aggregate_Ptr :=
        new Extension_Aggregate_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Extension_Aggregate_Expression (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Record_Component_Associations (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  374 =>
--#line  4436
 
yyval := 
yy.value_stack(yy.tos);

when  375 =>
--#line  4441
 
yyval := 
yy.value_stack(yy.tos);

when  376 =>
--#line  4443
 
yyval := 
yy.value_stack(yy.tos);

when  377 =>
--#line  4449
 
yyval := 
yy.value_stack(yy.tos-4);

when  378 =>
--#line  4451
 
yyval := 
yy.value_stack(yy.tos-3);

when  379 =>
--#line  4454
 
yyval := 
yy.value_stack(yy.tos-6);

when  380 =>
--#line  4457
 
yyval := 
yy.value_stack(yy.tos-5);

when  381 =>
--#line  4460
 
yyval := 
yy.value_stack(yy.tos-6);

when  382 =>
--#line  4463
 
yyval := 
yy.value_stack(yy.tos-5);

when  383 =>
--#line  4469
 
yyval := 
yy.value_stack(yy.tos-2);

when  384 =>
--#line  4471
 
yyval := 
yy.value_stack(yy.tos-1);

when  385 =>
--#line  4476
 
yyval := 
yy.value_stack(yy.tos-2);

when  386 =>
--#line  4478
 
yyval := 
yy.value_stack(yy.tos-2);

when  387 =>
--#line  4483
 
yyval := 
yy.value_stack(yy.tos);

when  388 =>
--#line  4485

   declare
      Call_Node1 : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Call_Node2 : constant Function_Call_Ptr :=
        Function_Call_Ptr (
yy.value_stack(yy.tos));
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos-1));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node1.all, T (Arg_List));
      Set_Start_Position (Call_Node1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Call_Node1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-2).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Prefix (Call_Node1.all, T (Sym_Node));
      Set_Is_Prefix_Call (Call_Node1.all, False);
      Push_Argument (Call_Node2.all, T (Call_Node1));
      Set_Start_Position (Call_Node2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      
yyval := YYSTYPE (Call_Node2);
   end;


when  389 =>
--#line  4522

   declare
      Call_Node1 : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node1.all, T (Arg_List));
      Set_Start_Position (Call_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Call_Node1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node1.all, T (Sym_Node));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Is_Prefix_Call (Call_Node1.all, False);
      
yyval := YYSTYPE (Call_Node1);
   end;


when  390 =>
--#line  4555

   declare
      Node1 : constant And_Then_Short_Circuit_Ptr:=
        new And_Then_Short_Circuit_Node;
      Node2 : constant And_Then_Short_Circuit_Ptr:=
        And_Then_Short_Circuit_Ptr (
yy.value_stack(yy.tos));
   begin
      Set_Short_Circuit_Operation_Left_Expression (Node1.all, 
yy.value_stack(yy.tos-4));
      Set_Start_Position (Node1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_End_Position (Node1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Short_Circuit_Operation_Right_Expression (Node1.all, 
yy.value_stack(yy.tos-1));
      Push_Argument (Node2.all, T (Node1));
      Set_Start_Position (Node2.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      
yyval := YYSTYPE (Node2);
   end;


when  391 =>
--#line  4572

   declare
      Node1 : constant And_Then_Short_Circuit_Ptr:=
        new And_Then_Short_Circuit_Node;
   begin
      Set_Short_Circuit_Operation_Left_Expression (Node1.all, 
yy.value_stack(yy.tos-3));
      Set_Short_Circuit_Operation_Right_Expression (Node1.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Node1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Node1.all, End_Position (
yy.value_stack(yy.tos).all));
      
yyval := YYSTYPE (Node1);
   end;


when  392 =>
--#line  4585

   declare
      Call_Node1 : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Call_Node2 : constant Function_Call_Ptr :=
        Function_Call_Ptr (
yy.value_stack(yy.tos));
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos-1));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node1.all, T (Arg_List));
      Set_Start_Position (Call_Node1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Call_Node1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-2).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Prefix (Call_Node1.all, T (Sym_Node));
      Set_Is_Prefix_Call (Call_Node1.all, False);
      Push_Argument (Call_Node2.all, T (Call_Node1));
      Set_Start_Position (Call_Node2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      
yyval := YYSTYPE (Call_Node2);
   end;


when  393 =>
--#line  4622

   declare
      Call_Node1 : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node1.all, T (Arg_List));
      Set_Start_Position (Call_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Call_Node1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node1.all, T (Sym_Node));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Is_Prefix_Call (Call_Node1.all, False);
      
yyval := YYSTYPE (Call_Node1);
   end;


when  394 =>
--#line  4655

   declare
      Node1 : constant Or_Else_Short_Circuit_Ptr:=
        new Or_Else_Short_Circuit_Node;
      Node2 : constant Or_Else_Short_Circuit_Ptr:=
        Or_Else_Short_Circuit_Ptr (
yy.value_stack(yy.tos));
   begin
      Set_Short_Circuit_Operation_Left_Expression (Node1.all, 
yy.value_stack(yy.tos-4));
      Set_Start_Position (Node1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_End_Position (Node1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Short_Circuit_Operation_Right_Expression (Node1.all, 
yy.value_stack(yy.tos-1));
      Push_Argument (Node2.all, T (Node1));
      Set_Start_Position (Node2.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      
yyval := YYSTYPE (Node2);
   end;


when  395 =>
--#line  4672

   declare
      Node1 : constant Or_Else_Short_Circuit_Ptr:=
        new Or_Else_Short_Circuit_Node;
   begin
      Set_Short_Circuit_Operation_Left_Expression (Node1.all, 
yy.value_stack(yy.tos-3));
      Set_Short_Circuit_Operation_Right_Expression (Node1.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Node1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Node1.all, End_Position (
yy.value_stack(yy.tos).all));
      
yyval := YYSTYPE (Node1);
   end;


when  396 =>
--#line  4685

   declare
      Call_Node1 : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Call_Node2 : constant Function_Call_Ptr :=
        Function_Call_Ptr (
yy.value_stack(yy.tos));
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos-1));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node1.all, T (Arg_List));
      Set_Start_Position (Call_Node1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Call_Node1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-2).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Prefix (Call_Node1.all, T (Sym_Node));
      Set_Is_Prefix_Call (Call_Node1.all, False);
      Push_Argument (Call_Node2.all, T (Call_Node1));
      Set_Start_Position (Call_Node2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      
yyval := YYSTYPE (Call_Node2);
   end;


when  397 =>
--#line  4722

   declare
      Call_Node1 : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node1.all, T (Arg_List));
      Set_Start_Position (Call_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Call_Node1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node1.all, T (Sym_Node));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Is_Prefix_Call (Call_Node1.all, False);
      
yyval := YYSTYPE (Call_Node1);
   end;


when  398 =>
--#line  4758

   declare
      Call_Node : constant Function_Call_Ptr :=
        Function_Call_Ptr (
yy.value_stack(yy.tos));
   begin
      Push_Argument (Call_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      
yyval := YYSTYPE (Call_Node);
   end;


when  400 =>
--#line  4770

   declare
      New_Node : constant Not_In_Range_Membership_Test_Ptr :=
        new Not_In_Range_Membership_Test_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Membership_Test_Expression (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Membership_Test_Range (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  401 =>
--#line  4783

   declare
      New_Node : constant In_Range_Membership_Test_Ptr :=
        new In_Range_Membership_Test_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Membership_Test_Expression (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Membership_Test_Range (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  402 =>
--#line  4796

   declare
      New_Node : constant Not_In_Type_Membership_Test_Ptr :=
        new Not_In_Type_Membership_Test_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Membership_Test_Expression (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Membership_Test_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  403 =>
--#line  4809

   declare
      New_Node : constant In_Type_Membership_Test_Ptr :=
        new In_Type_Membership_Test_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Membership_Test_Expression (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Membership_Test_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  404 =>
--#line  4825

   declare
      Call_Node1 : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Call_Node2 : constant Function_Call_Ptr :=
        Function_Call_Ptr (
yy.value_stack(yy.tos));
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-1));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Set_Function_Call_Parameters (Call_Node1.all, T (Arg_List));
      Set_Prefix (Call_Node1.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Call_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Call_Node1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Is_Prefix_Call (Call_Node1.all, False);
      Push_Argument (Call_Node2.all, T (Call_Node1));
      Set_Start_Position (Call_Node2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      
yyval := YYSTYPE (Call_Node2);
   end;


when  405 =>
--#line  4851

   declare
      Call_Node : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List  : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
   begin
      Set_Actual_Parameter (Arg_Node.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Prefix (Call_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  406 =>
--#line  4873

   declare
      Call_Node : constant Function_Call_Ptr :=
        Function_Call_Ptr (
yy.value_stack(yy.tos));
   begin
      Push_Argument (Call_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      
yyval := YYSTYPE (Call_Node);
   end;


when  408 =>
--#line  4888

   declare
      Call_Node : constant Function_Call_Ptr :=
        Function_Call_Ptr (
yy.value_stack(yy.tos));
   begin
      Push_Argument (Call_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      
yyval := YYSTYPE (Call_Node);
   end;


when  410 =>
--#line  4903

   declare
      Call_Node : constant Function_Call_Ptr :=
        Function_Call_Ptr (
yy.value_stack(yy.tos));
   begin
      Push_Argument (Call_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      
yyval := YYSTYPE (Call_Node);
   end;


when  412 =>
--#line  4915

   declare
      Call_Node  : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node   : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, T (Sym_Node));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  413 =>
--#line  4944

   declare
      Call_Node  : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node   : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, T (Sym_Node));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  414 =>
--#line  4976
 
yyval := 
yy.value_stack(yy.tos);

when  415 =>
--#line  4978
 
yyval := 
yy.value_stack(yy.tos);

when  416 =>
--#line  4980

   declare
      New_Node : constant Null_Literal_Ptr :=
        new Null_Literal_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  417 =>
--#line  4991
 
yyval := 
yy.value_stack(yy.tos);

when  418 =>
--#line  4993
 
yyval := 
yy.value_stack(yy.tos);

when  419 =>
--#line  4995
 
yyval := 
yy.value_stack(yy.tos);

when  420 =>
--#line  4997
 
yyval := 
yy.value_stack(yy.tos);

when  424 =>
--#line  5008

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """=""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  425 =>
--#line  5020

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """/=""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  426 =>
--#line  5032

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """<""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  427 =>
--#line  5044

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """<=""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  428 =>
--#line  5056

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """>""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  429 =>
--#line  5068

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """>=""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  430 =>
--#line  5083

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """+""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  431 =>
--#line  5095

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """-""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  432 =>
--#line  5107

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """&""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  433 =>
--#line  5122

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """+""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  434 =>
--#line  5134

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """-""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  435 =>
--#line  5149

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """*""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  436 =>
--#line  5161

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, """/""");
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  437 =>
--#line  5173

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos).all)) & '"');
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  438 =>
--#line  5185

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos).all)) & '"');
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  442 =>
--#line  5206
 
yyval := 
yy.value_stack(yy.tos-3);

when  443 =>
--#line  5208
 
yyval := 
yy.value_stack(yy.tos-3);

when  444 =>
--#line  5213

   declare
      New_Node : constant Qualified_Expression_Ptr :=
        new Qualified_Expression_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Converted_Or_Qualified_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Converted_Or_Qualified_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  445 =>
--#line  5229

   declare
      Wrap1 : constant Allocation_From_Subtype_Ptr :=
        new Allocation_From_Subtype_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Allocator_Subtype_Indication (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  446 =>
--#line  5247

   declare
      New_Node : constant Allocation_From_Qualified_Expression_Ptr :=
        new Allocation_From_Qualified_Expression_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Allocator_Qualified_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  447 =>
--#line  5262

   declare
      Wrap1 : constant Primary_Statement_Lists.List :=
        Primary_Statement_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Statement_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  448 =>
--#line  5272

   declare
      Wrap1 : constant Primary_Statement_Lists.List :=
        new Primary_Statement_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Statement_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  449 =>
--#line  5285

   Set_Label_Names (Statement_Node (
yy.value_stack(yy.tos).all), 
yy.value_stack(yy.tos-1));
   
yyval := 
yy.value_stack(yy.tos);


when  451 =>
--#line  5291

   Set_Label_Names (Statement_Node (
yy.value_stack(yy.tos).all), 
yy.value_stack(yy.tos-1));
   
yyval := 
yy.value_stack(yy.tos);


when  453 =>
--#line  5300
 
yyval := 
yy.value_stack(yy.tos);

when  454 =>
--#line  5302
 
yyval := 
yy.value_stack(yy.tos);

when  455 =>
--#line  5304
 
yyval := 
yy.value_stack(yy.tos);

when  456 =>
--#line  5306
 
yyval := 
yy.value_stack(yy.tos);

when  457 =>
--#line  5308
 
yyval := 
yy.value_stack(yy.tos);

when  458 =>
--#line  5310
 
yyval := 
yy.value_stack(yy.tos);

when  459 =>
--#line  5312
 
yyval := 
yy.value_stack(yy.tos);

when  460 =>
--#line  5314
 
yyval := 
yy.value_stack(yy.tos);

when  461 =>
--#line  5316
 
yyval := 
yy.value_stack(yy.tos);

when  462 =>
--#line  5318
 
yyval := 
yy.value_stack(yy.tos);

when  463 =>
--#line  5320
 
yyval := 
yy.value_stack(yy.tos);

when  464 =>
--#line  5322
 
yyval := 
yy.value_stack(yy.tos);

when  465 =>
--#line  5327
 
yyval := 
yy.value_stack(yy.tos);

when  466 =>
--#line  5329
 
yyval := 
yy.value_stack(yy.tos);

when  467 =>
--#line  5331
 
yyval := 
yy.value_stack(yy.tos);

when  468 =>
--#line  5333
 
yyval := 
yy.value_stack(yy.tos);

when  469 =>
--#line  5335
 
yyval := 
yy.value_stack(yy.tos);

when  470 =>
--#line  5337
 
yyval := 
yy.value_stack(yy.tos);

when  471 =>
--#line  5339
 
yyval := 
yy.value_stack(yy.tos);

when  472 =>
--#line  5344

   declare
      New_Node : constant Null_Statement_Ptr :=
        new Null_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  473 =>
--#line  5358
 
yyval := 
yy.value_stack(yy.tos-1);

when  474 =>
--#line  5363
 
yyval := 
yy.value_stack(yy.tos);

when  475 =>
--#line  5368

   declare
      New_Node : constant Assignment_Statement_Ptr :=
        new Assignment_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Assignment_Variable_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Assignment_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  476 =>
--#line  5385

   declare
      Wrap1 : constant If_Statement_Ptr :=
        new If_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-4));
      Wrap3 : constant If_Path_Ptr :=
        new If_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Condition_Expression (Wrap3.all, 
yy.value_stack(yy.tos-7));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-5).all));
      Primary_Path_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  477 =>
--#line  5409

   declare
      Wrap1 : constant If_Statement_Ptr :=
        new If_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-3));
      Wrap3 : constant If_Path_Ptr :=
        new If_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Condition_Expression (Wrap3.all, 
yy.value_stack(yy.tos-6));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-4).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  478 =>
--#line  5432

   declare
      Wrap1 : constant If_Statement_Ptr :=
        new If_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant If_Path_Ptr :=
        new If_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Condition_Expression (Wrap3.all, 
yy.value_stack(yy.tos-6));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-4).all));
      Primary_Path_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  479 =>
--#line  5455

   declare
      Wrap1 : constant If_Statement_Ptr :=
        new If_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant If_Path_Ptr :=
        new If_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Condition_Expression (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  480 =>
--#line  5480
 
yyval := 
yy.value_stack(yy.tos);

when  481 =>
--#line  5486

   declare
      Wrap1 : constant Case_Statement_Ptr :=
        new Case_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-3));
      Wrap3 : constant Case_Path_Ptr :=
        Case_Path_Ptr (
yy.value_stack(yy.tos-4));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Case_Expression (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  482 =>
--#line  5507

   declare
      Wrap1 : constant Case_Statement_Ptr :=
        new Case_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Case_Path_Ptr :=
        Case_Path_Ptr (
yy.value_stack(yy.tos-3));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Case_Expression (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  483 =>
--#line  5528

   declare
      Wrap1 : constant Case_Statement_Ptr :=
        new Case_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-3));
      Wrap3 : constant Case_Path_Ptr :=
        Case_Path_Ptr (
yy.value_stack(yy.tos-4));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Case_Expression (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  484 =>
--#line  5547

   declare
      Wrap1 : constant Case_Statement_Ptr :=
        new Case_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Case_Path_Ptr :=
        Case_Path_Ptr (
yy.value_stack(yy.tos-3));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Case_Expression (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  485 =>
--#line  5568

   declare
      New_Node : constant Case_Path_Ptr :=
        new Case_Path_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Case_Statement_Alternative_Choices (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Sequence_Of_Statements (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  486 =>
--#line  5585

   if 
yy.value_stack(yy.tos-6).all in Expression_Node'Class then
      declare
         New_Node : constant While_Loop_Statement_Ptr :=
           new While_Loop_Statement_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
         Set_Statement_Identifier (New_Node.all, 
yy.value_stack(yy.tos-7));
         Set_While_Condition (New_Node.all, 
yy.value_stack(yy.tos-6));
         Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Back_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant For_Loop_Statement_Ptr :=
           new For_Loop_Statement_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
         Set_Statement_Identifier (New_Node.all, 
yy.value_stack(yy.tos-7));
         Set_Loop_Parameter_Specification (New_Node.all, 
yy.value_stack(yy.tos-6));
         Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Back_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  487 =>
--#line  5616

   if 
yy.value_stack(yy.tos-5).all in Expression_Node'Class then
      declare
         New_Node : constant While_Loop_Statement_Ptr :=
           new While_Loop_Statement_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
         Set_Statement_Identifier (New_Node.all, 
yy.value_stack(yy.tos-6));
         Set_While_Condition (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant For_Loop_Statement_Ptr :=
           new For_Loop_Statement_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
         Set_Statement_Identifier (New_Node.all, 
yy.value_stack(yy.tos-6));
         Set_Loop_Parameter_Specification (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  488 =>
--#line  5645

   declare
      New_Node : constant Loop_Statement_Ptr :=
        new Loop_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Statement_Identifier (New_Node.all, 
yy.value_stack(yy.tos-6));
      Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Back_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  489 =>
--#line  5660

   declare
      New_Node : constant Loop_Statement_Ptr :=
        new Loop_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Statement_Identifier (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  490 =>
--#line  5674

   if 
yy.value_stack(yy.tos-6).all in Expression_Node'Class then
      declare
         New_Node : constant While_Loop_Statement_Ptr :=
           new While_Loop_Statement_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
         Set_While_Condition (New_Node.all, 
yy.value_stack(yy.tos-6));
         Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Back_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant For_Loop_Statement_Ptr :=
           new For_Loop_Statement_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
         Set_Loop_Parameter_Specification (New_Node.all, 
yy.value_stack(yy.tos-6));
         Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Back_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  491 =>
--#line  5703

   if 
yy.value_stack(yy.tos-5).all in Expression_Node'Class then
      declare
         New_Node : constant While_Loop_Statement_Ptr :=
           new While_Loop_Statement_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
         Set_While_Condition (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant For_Loop_Statement_Ptr :=
           new For_Loop_Statement_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
         Set_Loop_Parameter_Specification (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  492 =>
--#line  5729

   declare
      New_Node : constant Loop_Statement_Ptr :=
        new Loop_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Back_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  493 =>
--#line  5742

   declare
      New_Node : constant Loop_Statement_Ptr :=
        new Loop_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Loop_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  494 =>
--#line  5757
 
yyval := 
yy.value_stack(yy.tos);

when  495 =>
--#line  5759
 
yyval := 
yy.value_stack(yy.tos);

when  496 =>
--#line  5764

   declare
      New_Node : constant Loop_Parameter_Specification_Ptr :=
        new Loop_Parameter_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Trait_Kind (New_Node.all, A_Reverse_Trait);
      Set_Specification_Subtype_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  497 =>
--#line  5778

   declare
      New_Node : constant Loop_Parameter_Specification_Ptr :=
        new Loop_Parameter_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Specification_Subtype_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  498 =>
--#line  5796

   declare
      New_Node : constant Block_Statement_Ptr :=
        new Block_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Statement_Identifier (New_Node.all, 
yy.value_stack(yy.tos-6));
      Set_Block_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Back_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  499 =>
--#line  5812

   declare
      New_Node : constant Block_Statement_Ptr :=
        new Block_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Statement_Identifier (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Block_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  500 =>
--#line  5827

   declare
      New_Node : constant Block_Statement_Ptr :=
        new Block_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Statement_Identifier (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Back_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  501 =>
--#line  5842

   declare
      New_Node : constant Block_Statement_Ptr :=
        new Block_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Statement_Identifier (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  502 =>
--#line  5856

   declare
      New_Node : constant Block_Statement_Ptr :=
        new Block_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Block_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Back_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  503 =>
--#line  5871

   declare
      New_Node : constant Block_Statement_Ptr :=
        new Block_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Block_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  504 =>
--#line  5885

   declare
      New_Node : constant Block_Statement_Ptr :=
        new Block_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Back_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  505 =>
--#line  5898

   declare
      New_Node : constant Block_Statement_Ptr :=
        new Block_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  506 =>
--#line  5913

   declare
      New_Node : constant Exit_Statement_Ptr :=
        new Exit_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Exit_Loop_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Exit_Condition (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  507 =>
--#line  5926

   declare
      New_Node : constant Exit_Statement_Ptr :=
        new Exit_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Exit_Loop_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  508 =>
--#line  5938

   declare
      New_Node : constant Exit_Statement_Ptr :=
        new Exit_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Exit_Condition (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  509 =>
--#line  5950

   declare
      New_Node : constant Exit_Statement_Ptr :=
        new Exit_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  510 =>
--#line  5964

   declare
      New_Node : constant Goto_Statement_Ptr :=
        new Goto_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Goto_Label (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  511 =>
--#line  5979

   if 
yy.value_stack(yy.tos-1).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Declaration_Ptr :=
           new Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Declaration_Ptr :=
           new Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  512 =>
--#line  6005

   if 
yy.value_stack(yy.tos-1).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Declaration_Ptr :=
           new Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Declaration_Ptr :=
           new Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  513 =>
--#line  6032
 
yyval := 
yy.value_stack(yy.tos);

when  514 =>
--#line  6034
 
yyval := 
yy.value_stack(yy.tos);

when  515 =>
--#line  6039

   declare
      New_Node : constant Procedure_Specification_Ptr :=
        new Procedure_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Profile (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  516 =>
--#line  6055

   declare
      New_Node : constant Function_Specification_Ptr :=
        new Function_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Profile (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  517 =>
--#line  6068

   declare
      New_Node : constant Function_Specification_Ptr :=
        new Function_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  518 =>
--#line  6083
 
yyval := 
yy.value_stack(yy.tos);

when  519 =>
--#line  6085
 
yyval := 
yy.value_stack(yy.tos);

when  520 =>
--#line  6090
 
yyval := 
yy.value_stack(yy.tos);

when  521 =>
--#line  6092
 
yyval := 
yy.value_stack(yy.tos);

when  522 =>
--#line  6097
 
yyval := 
yy.value_stack(yy.tos);

when  523 =>
--#line  6102

   declare
      New_Node : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (New_Node.all, Value_Image (String_Literal_Ptr (
yy.value_stack(yy.tos)).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  524 =>
--#line  6117

   declare
      New_Node : constant Defining_Operator_Symbol_Ptr :=
        new Defining_Operator_Symbol_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Defining_Name_Image (New_Node.all, Value_Image (String_Literal_Ptr (
yy.value_stack(yy.tos)).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  525 =>
--#line  6132

   declare
      Wrap1 : constant Primary_Parameter_Lists.List :=
        Primary_Parameter_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  526 =>
--#line  6141

   declare
      Wrap1 : constant Primary_Parameter_Lists.List :=
        new Primary_Parameter_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  527 =>
--#line  6153

   declare
      Wrap1 : constant Function_Profile_Ptr :=
        new Function_Profile_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Parameter_Profile (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Null_Exclusion (Wrap2.all, True);
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Result_Profile (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  528 =>
--#line  6173

   declare
      Wrap1 : constant Function_Profile_Ptr :=
        new Function_Profile_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Parameter_Profile (Wrap1.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Result_Profile (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  529 =>
--#line  6192

   declare
      Wrap1 : constant Function_Profile_Ptr :=
        new Function_Profile_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Null_Exclusion (Wrap2.all, True);
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Result_Profile (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  530 =>
--#line  6211

   declare
      Wrap1 : constant Function_Profile_Ptr :=
        new Function_Profile_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Result_Profile (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  531 =>
--#line  6229

   declare
      New_Node : constant Function_Profile_Ptr :=
        new Function_Profile_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Parameter_Profile (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Result_Profile (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  532 =>
--#line  6242

   declare
      New_Node : constant Function_Profile_Ptr :=
        new Function_Profile_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Result_Profile (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  533 =>
--#line  6258

   declare
      Wrap1 : constant Primary_Parameter_Lists.List :=
        Primary_Parameter_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Parameter_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-2));
   end;


when  534 =>
--#line  6268

   declare
      Wrap1 : constant Primary_Parameter_Lists.List :=
        new Primary_Parameter_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Parameter_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  535 =>
--#line  6282

   declare
      Wrap1 : constant Parameter_Specification_Ptr :=
        new Parameter_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Mode_Kind (Wrap1.all, Modes.
yy.value_stack(yy.tos-3));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Initialization_Expression (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  536 =>
--#line  6304

   declare
      Wrap1 : constant Parameter_Specification_Ptr :=
        new Parameter_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Mode_Kind (Wrap1.all, Modes.
yy.value_stack(yy.tos-2));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  537 =>
--#line  6325

   declare
      Wrap1 : constant Parameter_Specification_Ptr :=
        new Parameter_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Mode_Kind (Wrap1.all, Modes.
yy.value_stack(yy.tos-2));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Initialization_Expression (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  538 =>
--#line  6345

   declare
      Wrap1 : constant Parameter_Specification_Ptr :=
        new Parameter_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Mode_Kind (Wrap1.all, Modes.
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  539 =>
--#line  6366

   declare
      Wrap1 : constant Parameter_Specification_Ptr :=
        new Parameter_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Initialization_Expression (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  540 =>
--#line  6386

   declare
      Wrap1 : constant Parameter_Specification_Ptr :=
        new Parameter_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  541 =>
--#line  6406

   declare
      Wrap1 : constant Parameter_Specification_Ptr :=
        new Parameter_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Initialization_Expression (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  542 =>
--#line  6425

   declare
      Wrap1 : constant Parameter_Specification_Ptr :=
        new Parameter_Specification_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  543 =>
--#line  6444

   declare
      New_Node : constant Parameter_Specification_Ptr :=
        new Parameter_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Trait_Kind (New_Node.all, An_Access_Definition_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  544 =>
--#line  6459

   declare
      New_Node : constant Parameter_Specification_Ptr :=
        new Parameter_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, An_Access_Definition_Trait);
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  545 =>
--#line  6478

   if 
yy.value_stack(yy.tos-7).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Body_Declaration_Ptr :=
           new Procedure_Body_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-8).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-7));
         Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Body_Declaration_Ptr :=
           new Function_Body_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-8).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-7));
         Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  546 =>
--#line  6511

   if 
yy.value_stack(yy.tos-6).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Body_Declaration_Ptr :=
           new Procedure_Body_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-6));
         Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Body_Declaration_Ptr :=
           new Function_Body_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-6));
         Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  547 =>
--#line  6542

   if 
yy.value_stack(yy.tos-7).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Body_Declaration_Ptr :=
           new Procedure_Body_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-7));
         Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Body_Declaration_Ptr :=
           new Function_Body_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-7));
         Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  548 =>
--#line  6573

   if 
yy.value_stack(yy.tos-6).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Body_Declaration_Ptr :=
           new Procedure_Body_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-6));
         Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Body_Declaration_Ptr :=
           new Function_Body_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-6));
         Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  549 =>
--#line  6604
 
yyval := 
yy.value_stack(yy.tos-1);

when  550 =>
--#line  6606
 
yyval := 
yy.value_stack(yy.tos-2);

when  551 =>
--#line  6611

   declare
      New_Node : constant Function_Call_Ptr :=
        new Function_Call_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Prefix (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Record_Aggregate (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  552 =>
--#line  6628
 
yyval := 
yy.value_stack(yy.tos-2);

when  553 =>
--#line  6630
 
yyval := 
yy.value_stack(yy.tos-1);

when  554 =>
--#line  6635
 
yyval := 
yy.value_stack(yy.tos);

when  555 =>
--#line  6637
 
yyval := 
yy.value_stack(yy.tos);

when  556 =>
--#line  6642
 
yyval := 
yy.value_stack(yy.tos);

when  557 =>
--#line  6644
 
yyval := 
yy.value_stack(yy.tos);

when  558 =>
--#line  6649

   declare
      New_Node : constant Simple_Return_Statement_Ptr :=
        new Simple_Return_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Return_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  559 =>
--#line  6661

   declare
      New_Node : constant Simple_Return_Statement_Ptr :=
        new Simple_Return_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  560 =>
--#line  6677

   declare
      Wrap1 : constant Extended_Return_Statement_Ptr :=
        new Extended_Return_Statement_Node;
      Wrap2 : constant Return_Object_Specification_Ptr :=
        new Return_Object_Specification_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (Wrap2.all, 
yy.value_stack(yy.tos-6));
      Set_Trait_Kind (Wrap2.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Initialization_Expression (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Handled_Statements (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Return_Object_Specification (Wrap1.all, T (Wrap2));
   end;


when  561 =>
--#line  6699

   declare
      Wrap1 : constant Extended_Return_Statement_Ptr :=
        new Extended_Return_Statement_Node;
      Wrap2 : constant Return_Object_Specification_Ptr :=
        new Return_Object_Specification_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (Wrap2.all, 
yy.value_stack(yy.tos-5));
      Set_Trait_Kind (Wrap2.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Return_Object_Specification (Wrap1.all, T (Wrap2));
   end;


when  562 =>
--#line  6720

   declare
      Wrap1 : constant Extended_Return_Statement_Ptr :=
        new Extended_Return_Statement_Node;
      Wrap2 : constant Return_Object_Specification_Ptr :=
        new Return_Object_Specification_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (Wrap2.all, 
yy.value_stack(yy.tos-5));
      Set_Trait_Kind (Wrap2.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Handled_Statements (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Return_Object_Specification (Wrap1.all, T (Wrap2));
   end;


when  563 =>
--#line  6741

   declare
      Wrap1 : constant Extended_Return_Statement_Ptr :=
        new Extended_Return_Statement_Node;
      Wrap2 : constant Return_Object_Specification_Ptr :=
        new Return_Object_Specification_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Trait_Kind (Wrap2.all, An_Aliased_Trait);
      Set_Object_Declaration_Subtype (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Return_Object_Specification (Wrap1.all, T (Wrap2));
   end;


when  564 =>
--#line  6762

   declare
      Wrap1 : constant Extended_Return_Statement_Ptr :=
        new Extended_Return_Statement_Node;
      Wrap2 : constant Return_Object_Specification_Ptr :=
        new Return_Object_Specification_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (Wrap2.all, 
yy.value_stack(yy.tos-5));
      Set_Object_Declaration_Subtype (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Initialization_Expression (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Handled_Statements (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Return_Object_Specification (Wrap1.all, T (Wrap2));
   end;


when  565 =>
--#line  6783

   declare
      Wrap1 : constant Extended_Return_Statement_Ptr :=
        new Extended_Return_Statement_Node;
      Wrap2 : constant Return_Object_Specification_Ptr :=
        new Return_Object_Specification_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Object_Declaration_Subtype (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Return_Object_Specification (Wrap1.all, T (Wrap2));
   end;


when  566 =>
--#line  6803

   declare
      Wrap1 : constant Extended_Return_Statement_Ptr :=
        new Extended_Return_Statement_Node;
      Wrap2 : constant Return_Object_Specification_Ptr :=
        new Return_Object_Specification_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Object_Declaration_Subtype (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Handled_Statements (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Return_Object_Specification (Wrap1.all, T (Wrap2));
   end;


when  567 =>
--#line  6823

   declare
      Wrap1 : constant Extended_Return_Statement_Ptr :=
        new Extended_Return_Statement_Node;
      Wrap2 : constant Return_Object_Specification_Ptr :=
        new Return_Object_Specification_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Name (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Object_Declaration_Subtype (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Return_Object_Specification (Wrap1.all, T (Wrap2));
   end;


when  568 =>
--#line  6844
 
yyval := 
yy.value_stack(yy.tos);

when  569 =>
--#line  6846
 
yyval := 
yy.value_stack(yy.tos);

when  570 =>
--#line  6852

   if 
yy.value_stack(yy.tos-3).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Declaration_Ptr :=
           new Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Is_Null_Procedure (New_Node.all, True);
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Declaration_Ptr :=
           new Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Is_Null_Procedure (New_Node.all, True);
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  571 =>
--#line  6880

   if 
yy.value_stack(yy.tos-3).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Declaration_Ptr :=
           new Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Is_Null_Procedure (New_Node.all, True);
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Declaration_Ptr :=
           new Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Is_Null_Procedure (New_Node.all, True);
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  572 =>
--#line  6909

   declare
      New_Node : constant Package_Declaration_Ptr :=
        new Package_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Package_Specification (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  573 =>
--#line  6925

   declare
      New_Node : constant Package_Specification_Ptr :=
        new Package_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Visible_Part_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Private_Part_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  574 =>
--#line  6941

   declare
      New_Node : constant Package_Specification_Ptr :=
        new Package_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Visible_Part_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Private_Part_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  575 =>
--#line  6956

   declare
      New_Node : constant Package_Specification_Ptr :=
        new Package_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Visible_Part_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  576 =>
--#line  6971

   declare
      New_Node : constant Package_Specification_Ptr :=
        new Package_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Visible_Part_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  577 =>
--#line  6985

   declare
      New_Node : constant Package_Specification_Ptr :=
        new Package_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Private_Part_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  578 =>
--#line  7000

   declare
      New_Node : constant Package_Specification_Ptr :=
        new Package_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Private_Part_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  579 =>
--#line  7013

   declare
      New_Node : constant Package_Specification_Ptr :=
        new Package_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  580 =>
--#line  7026

   declare
      New_Node : constant Package_Specification_Ptr :=
        new Package_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  581 =>
--#line  7043

   declare
      New_Node : constant Package_Body_Declaration_Ptr :=
        new Package_Body_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-6));
      Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  582 =>
--#line  7059

   declare
      New_Node : constant Package_Body_Declaration_Ptr :=
        new Package_Body_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  583 =>
--#line  7074

   declare
      New_Node : constant Package_Body_Declaration_Ptr :=
        new Package_Body_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  584 =>
--#line  7089

   declare
      New_Node : constant Package_Body_Declaration_Ptr :=
        new Package_Body_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  585 =>
--#line  7106

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Tagged_Private_Type_Definition_Ptr :=
        new Tagged_Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Has_Tagged (Wrap2.all, True);
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  586 =>
--#line  7131

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Tagged_Private_Type_Definition_Ptr :=
        new Tagged_Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Has_Tagged (Wrap2.all, True);
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  587 =>
--#line  7154

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Tagged_Private_Type_Definition_Ptr :=
        new Tagged_Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Has_Tagged (Wrap2.all, True);
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  588 =>
--#line  7177

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Tagged_Private_Type_Definition_Ptr :=
        new Tagged_Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Tagged (Wrap2.all, True);
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  589 =>
--#line  7198

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Private_Type_Definition_Ptr :=
        new Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  590 =>
--#line  7220

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Private_Type_Definition_Ptr :=
        new Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  591 =>
--#line  7240

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Tagged_Private_Type_Definition_Ptr :=
        new Tagged_Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Has_Tagged (Wrap2.all, True);
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  592 =>
--#line  7264

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Tagged_Private_Type_Definition_Ptr :=
        new Tagged_Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Has_Tagged (Wrap2.all, True);
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  593 =>
--#line  7286

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Tagged_Private_Type_Definition_Ptr :=
        new Tagged_Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Has_Tagged (Wrap2.all, True);
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  594 =>
--#line  7307

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Tagged_Private_Type_Definition_Ptr :=
        new Tagged_Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Tagged (Wrap2.all, True);
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  595 =>
--#line  7326

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Private_Type_Definition_Ptr :=
        new Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  596 =>
--#line  7346

   declare
      Wrap1 : constant Private_Type_Declaration_Ptr :=
        new Private_Type_Declaration_Node;
      Wrap2 : constant Private_Type_Definition_Ptr :=
        new Private_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  597 =>
--#line  7369

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-11).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-10));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-9));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Has_Synchronized (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  598 =>
--#line  7395

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-10).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-9));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Has_Synchronized (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  599 =>
--#line  7420

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-11).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-10));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-9));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  600 =>
--#line  7447

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-10).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-9));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  601 =>
--#line  7473

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-10).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-9));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  602 =>
--#line  7497

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  603 =>
--#line  7521

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-10).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-9));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Has_Synchronized (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  604 =>
--#line  7544

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Has_Synchronized (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  605 =>
--#line  7567

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-10).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-9));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  606 =>
--#line  7591

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  607 =>
--#line  7614

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  608 =>
--#line  7636

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Discriminant_Part (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  609 =>
--#line  7658

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-10).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-9));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Has_Synchronized (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  610 =>
--#line  7682

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Has_Synchronized (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  611 =>
--#line  7705

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-10).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-9));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  612 =>
--#line  7730

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  613 =>
--#line  7754

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  614 =>
--#line  7777

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Trait_Kind (Wrap2.all, An_Abstract_Trait);
      Set_Has_Abstract (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  615 =>
--#line  7799

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Has_Synchronized (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  616 =>
--#line  7821

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Has_Synchronized (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  617 =>
--#line  7842

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-8));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  618 =>
--#line  7865

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Trait_Kind (Wrap2.all, A_Limited_Trait);
      Set_Has_Limited (Wrap2.all, True);
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  619 =>
--#line  7887

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  620 =>
--#line  7908

   declare
      Wrap1 : constant Private_Extension_Declaration_Ptr :=
        new Private_Extension_Declaration_Node;
      Wrap2 : constant Private_Extension_Definition_Ptr :=
        new Private_Extension_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Name (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Ancestor_Subtype_Indication (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Has_Private (Wrap2.all, True);
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Type_Declaration_View (Wrap1.all, T (Wrap2));
   end;


when  621 =>
--#line  7930
 
yyval := 
yy.value_stack(yy.tos);

when  622 =>
--#line  7932
 
yyval := 
yy.value_stack(yy.tos);

when  623 =>
--#line  7937

   declare
      Wrap1 : constant Use_Package_Clause_Ptr :=
        new Use_Package_Clause_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        Primary_Expression_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Clause_Names (Wrap1.all, T (Wrap2));
   end;


when  624 =>
--#line  7952

   declare
      Wrap1 : constant Use_Package_Clause_Ptr :=
        new Use_Package_Clause_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Clause_Names (Wrap1.all, T (Wrap2));
   end;


when  625 =>
--#line  7970

   declare
      Wrap1 : constant Use_Type_Clause_Ptr :=
        new Use_Type_Clause_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        Primary_Expression_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Clause_Names (Wrap1.all, T (Wrap2));
   end;


when  626 =>
--#line  7985

   declare
      Wrap1 : constant Use_Type_Clause_Ptr :=
        new Use_Type_Clause_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Clause_Names (Wrap1.all, T (Wrap2));
   end;


when  627 =>
--#line  8003
 
yyval := 
yy.value_stack(yy.tos);

when  628 =>
--#line  8005
 
yyval := 
yy.value_stack(yy.tos);

when  629 =>
--#line  8007
 
yyval := 
yy.value_stack(yy.tos);

when  630 =>
--#line  8009
 
yyval := 
yy.value_stack(yy.tos);

when  631 =>
--#line  8011
 
yyval := 
yy.value_stack(yy.tos);

when  632 =>
--#line  8017

   declare
      Wrap1 : constant Object_Renaming_Declaration_Ptr :=
        new Object_Renaming_Declaration_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Renamed_Entity (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  633 =>
--#line  8038

   declare
      Wrap1 : constant Object_Renaming_Declaration_Ptr :=
        new Object_Renaming_Declaration_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Renamed_Entity (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  634 =>
--#line  8058

   declare
      New_Node : constant Object_Renaming_Declaration_Ptr :=
        new Object_Renaming_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Renamed_Entity (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  635 =>
--#line  8076

   declare
      New_Node : constant Exception_Renaming_Declaration_Ptr :=
        new Exception_Renaming_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Renamed_Entity (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  636 =>
--#line  8092

   declare
      New_Node : constant Package_Renaming_Declaration_Ptr :=
        new Package_Renaming_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Renamed_Entity (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  637 =>
--#line  8109

   if 
yy.value_stack(yy.tos-3).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Renaming_Declaration_Ptr :=
           new Procedure_Renaming_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Renamed_Entity (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Renaming_Declaration_Ptr :=
           new Function_Renaming_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Renamed_Entity (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  638 =>
--#line  8137

   if 
yy.value_stack(yy.tos-3).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Renaming_Declaration_Ptr :=
           new Procedure_Renaming_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Renamed_Entity (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Renaming_Declaration_Ptr :=
           new Function_Renaming_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Renamed_Entity (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  639 =>
--#line  8167

   declare
      New_Node : constant Generic_Package_Renaming_Declaration_Ptr :=
        new Generic_Package_Renaming_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Empty_Generic_Part (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Renamed_Entity (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  640 =>
--#line  8182

   if 
yy.value_stack(yy.tos-3).all in Procedure_Specification_Node then
      declare
         New_Node : constant Generic_Procedure_Renaming_Declaration_Ptr :=
           new Generic_Procedure_Renaming_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Empty_Generic_Part (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Renamed_Entity (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Generic_Function_Renaming_Declaration_Ptr :=
           new Generic_Function_Renaming_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Empty_Generic_Part (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Renamed_Entity (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  641 =>
--#line  8214

   declare
      New_Node : constant Task_Type_Declaration_Ptr :=
        new Task_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Discriminant_Part (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  642 =>
--#line  8230

   declare
      New_Node : constant Task_Type_Declaration_Ptr :=
        new Task_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Discriminant_Part (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  643 =>
--#line  8245

   declare
      New_Node : constant Task_Type_Declaration_Ptr :=
        new Task_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Discriminant_Part (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  644 =>
--#line  8259

   declare
      New_Node : constant Task_Type_Declaration_Ptr :=
        new Task_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  645 =>
--#line  8273

   declare
      New_Node : constant Task_Type_Declaration_Ptr :=
        new Task_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  646 =>
--#line  8286

   declare
      New_Node : constant Task_Type_Declaration_Ptr :=
        new Task_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  647 =>
--#line  8302

   declare
      New_Node : constant Single_Task_Declaration_Ptr :=
        new Single_Task_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  648 =>
--#line  8316

   declare
      New_Node : constant Single_Task_Declaration_Ptr :=
        new Single_Task_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  649 =>
--#line  8329

   declare
      New_Node : constant Single_Task_Declaration_Ptr :=
        new Single_Task_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  650 =>
--#line  8344

   declare
      New_Node : constant Task_Definition_Ptr :=
        new Task_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Visible_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Private_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Identifier (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  651 =>
--#line  8358

   declare
      New_Node : constant Task_Definition_Ptr :=
        new Task_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Visible_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Private_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  652 =>
--#line  8371

   declare
      New_Node : constant Task_Definition_Ptr :=
        new Task_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Visible_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Identifier (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  653 =>
--#line  8384

   declare
      New_Node : constant Task_Definition_Ptr :=
        new Task_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Visible_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  654 =>
--#line  8396

   declare
      New_Node : constant Task_Definition_Ptr :=
        new Task_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Private_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Identifier (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  655 =>
--#line  8409

   declare
      New_Node : constant Task_Definition_Ptr :=
        new Task_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Private_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  656 =>
--#line  8421

   declare
      New_Node : constant Task_Definition_Ptr :=
        new Task_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Identifier (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  657 =>
--#line  8433

   declare
      New_Node : constant Task_Definition_Ptr :=
        new Task_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  658 =>
--#line  8447
 
yyval := 
yy.value_stack(yy.tos);

when  659 =>
--#line  8449
 
yyval := 
yy.value_stack(yy.tos);

when  660 =>
--#line  8455

   declare
      New_Node : constant Task_Body_Declaration_Ptr :=
        new Task_Body_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-7));
      Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Compound_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  661 =>
--#line  8471

   declare
      New_Node : constant Task_Body_Declaration_Ptr :=
        new Task_Body_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-6));
      Set_Body_Declarative_Items (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  662 =>
--#line  8489

   declare
      New_Node : constant Protected_Type_Declaration_Ptr :=
        new Protected_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Discriminant_Part (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  663 =>
--#line  8505

   declare
      New_Node : constant Protected_Type_Declaration_Ptr :=
        new Protected_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Discriminant_Part (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  664 =>
--#line  8520

   declare
      New_Node : constant Protected_Type_Declaration_Ptr :=
        new Protected_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  665 =>
--#line  8535

   declare
      New_Node : constant Protected_Type_Declaration_Ptr :=
        new Protected_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  666 =>
--#line  8552

   declare
      New_Node : constant Single_Protected_Declaration_Ptr :=
        new Single_Protected_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  667 =>
--#line  8567

   declare
      New_Node : constant Single_Protected_Declaration_Ptr :=
        new Single_Protected_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  668 =>
--#line  8584

   declare
      New_Node : constant Protected_Definition_Ptr :=
        new Protected_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Visible_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Private_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Identifier (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  669 =>
--#line  8599

   declare
      New_Node : constant Protected_Definition_Ptr :=
        new Protected_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Visible_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Private_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  670 =>
--#line  8612

   declare
      New_Node : constant Protected_Definition_Ptr :=
        new Protected_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Visible_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Identifier (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  671 =>
--#line  8625

   declare
      New_Node : constant Protected_Definition_Ptr :=
        new Protected_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Visible_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  672 =>
--#line  8637

   declare
      New_Node : constant Protected_Definition_Ptr :=
        new Protected_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Private_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Identifier (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  673 =>
--#line  8650

   declare
      New_Node : constant Protected_Definition_Ptr :=
        new Protected_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Private_Part_Items (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  674 =>
--#line  8662

   declare
      New_Node : constant Protected_Definition_Ptr :=
        new Protected_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Identifier (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  675 =>
--#line  8674

   declare
      New_Node : constant Protected_Definition_Ptr :=
        new Protected_Definition_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  676 =>
--#line  8688
 
yyval := 
yy.value_stack(yy.tos);

when  677 =>
--#line  8690
 
yyval := 
yy.value_stack(yy.tos);

when  678 =>
--#line  8692
 
yyval := 
yy.value_stack(yy.tos);

when  679 =>
--#line  8697
 
yyval := 
yy.value_stack(yy.tos);

when  680 =>
--#line  8699
 
yyval := 
yy.value_stack(yy.tos);

when  681 =>
--#line  8705

   declare
      New_Node : constant Protected_Body_Declaration_Ptr :=
        new Protected_Body_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Protected_Operation_Items (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  682 =>
--#line  8720

   declare
      New_Node : constant Protected_Body_Declaration_Ptr :=
        new Protected_Body_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Protected_Operation_Items (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  683 =>
--#line  8734

   declare
      New_Node : constant Protected_Body_Declaration_Ptr :=
        new Protected_Body_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Identifier (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  684 =>
--#line  8747

   declare
      New_Node : constant Protected_Body_Declaration_Ptr :=
        new Protected_Body_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  685 =>
--#line  8762
 
yyval := 
yy.value_stack(yy.tos);

when  686 =>
--#line  8764
 
yyval := 
yy.value_stack(yy.tos);

when  687 =>
--#line  8766
 
yyval := 
yy.value_stack(yy.tos);

when  688 =>
--#line  8768
 
yyval := 
yy.value_stack(yy.tos);

when  689 =>
--#line  8774

   declare
      New_Node : constant Entry_Declaration_Ptr :=
        new Entry_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Entry_Family_Definition (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Parameter_Profile (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  690 =>
--#line  8790

   declare
      New_Node : constant Entry_Declaration_Ptr :=
        new Entry_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Parameter_Profile (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  691 =>
--#line  8805

   declare
      New_Node : constant Entry_Declaration_Ptr :=
        new Entry_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Entry_Family_Definition (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Parameter_Profile (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  692 =>
--#line  8819

   declare
      New_Node : constant Entry_Declaration_Ptr :=
        new Entry_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Parameter_Profile (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  693 =>
--#line  8837

   declare
      New_Node : constant Accept_Statement_Ptr :=
        new Accept_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Accept_Entry_Direct_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Accept_Entry_Index (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Accept_Parameters (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  694 =>
--#line  8853

   declare
      New_Node : constant Accept_Statement_Ptr :=
        new Accept_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Accept_Entry_Direct_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Accept_Entry_Index (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Accept_Parameters (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  695 =>
--#line  8868

   declare
      New_Node : constant Accept_Statement_Ptr :=
        new Accept_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Accept_Entry_Direct_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Accept_Parameters (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Handled_Statements (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  696 =>
--#line  8882

   declare
      New_Node : constant Accept_Statement_Ptr :=
        new Accept_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Accept_Entry_Direct_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Accept_Parameters (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  697 =>
--#line  8898
 
yyval := 
yy.value_stack(yy.tos);

when  698 =>
--#line  8905

   declare
      Wrap1 : constant Entry_Body_Declaration_Ptr :=
        new Entry_Body_Declaration_Node;
      Wrap2 : constant Function_Specification_Ptr :=
        Function_Specification_Ptr (
yy.value_stack(yy.tos-8));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-10).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Names (Wrap2.all, 
yy.value_stack(yy.tos-9));
      Set_Entry_Barrier (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Body_Declarative_Items (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Handled_Statements (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Compound_Name (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Specification (Wrap1.all, T (Wrap2));
   end;


when  699 =>
--#line  8927

   declare
      Wrap1 : constant Entry_Body_Declaration_Ptr :=
        new Entry_Body_Declaration_Node;
      Wrap2 : constant Function_Specification_Ptr :=
        Function_Specification_Ptr (
yy.value_stack(yy.tos-7));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-9).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Names (Wrap2.all, 
yy.value_stack(yy.tos-8));
      Set_Entry_Barrier (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Body_Declarative_Items (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Handled_Statements (Wrap1.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Specification (Wrap1.all, T (Wrap2));
   end;


when  700 =>
--#line  8949

   declare
      Wrap1 : constant Function_Specification_Ptr :=
        new Function_Specification_Node;
      Wrap2 : constant Function_Profile_Ptr :=
        new Function_Profile_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Result_Profile (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_Parameter_Profile (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Profile (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  701 =>
--#line  8967

   declare
      Wrap1 : constant Function_Specification_Ptr :=
        new Function_Specification_Node;
      Wrap2 : constant Function_Profile_Ptr :=
        new Function_Profile_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Parameter_Profile (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Profile (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  702 =>
--#line  8987
 
yyval := 
yy.value_stack(yy.tos);

when  703 =>
--#line  8992

   declare
      New_Node : constant Entry_Index_Specification_Ptr :=
        new Entry_Index_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Specification_Subtype_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  704 =>
--#line  9008

   declare
      New_Node : constant Procedure_Call_Statement_Ptr :=
        new Procedure_Call_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Called_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  705 =>
--#line  9023

   declare
      New_Node : constant Requeue_Statement_With_Abort_Ptr :=
        new Requeue_Statement_With_Abort_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Requeue_Entry_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  706 =>
--#line  9035

   declare
      New_Node : constant Requeue_Statement_Ptr :=
        new Requeue_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Requeue_Entry_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  707 =>
--#line  9050
 
yyval := 
yy.value_stack(yy.tos);

when  708 =>
--#line  9052
 
yyval := 
yy.value_stack(yy.tos);

when  709 =>
--#line  9057

   declare
      New_Node : constant Delay_Until_Statement_Ptr :=
        new Delay_Until_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Delay_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  710 =>
--#line  9072

   declare
      New_Node : constant Delay_Relative_Statement_Ptr :=
        new Delay_Relative_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Delay_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  711 =>
--#line  9087
 
yyval := 
yy.value_stack(yy.tos);

when  712 =>
--#line  9089
 
yyval := 
yy.value_stack(yy.tos);

when  713 =>
--#line  9095

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-4));
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Guard (Wrap3.all, 
yy.value_stack(yy.tos-7));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-6));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-5).all));
      Primary_Path_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  714 =>
--#line  9120

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-3));
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Guard (Wrap3.all, 
yy.value_stack(yy.tos-6));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-4).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  715 =>
--#line  9144

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Guard (Wrap3.all, 
yy.value_stack(yy.tos-6));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-4).all));
      Primary_Path_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  716 =>
--#line  9169

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Guard (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  717 =>
--#line  9193

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-4));
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Guard (Wrap3.all, 
yy.value_stack(yy.tos-6));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-5).all));
      Primary_Path_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  718 =>
--#line  9217

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-3));
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Guard (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-4).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  719 =>
--#line  9240

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Guard (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-4).all));
      Primary_Path_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  720 =>
--#line  9263

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Guard (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  721 =>
--#line  9286

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-4));
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-6));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-5).all));
      Primary_Path_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  722 =>
--#line  9310

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-3));
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-4).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  723 =>
--#line  9333

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-4).all));
      Primary_Path_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  724 =>
--#line  9356

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  725 =>
--#line  9379

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-4));
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-5));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-5).all));
      Primary_Path_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  726 =>
--#line  9402

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-3));
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-4).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  727 =>
--#line  9424

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-4));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-4).all));
      Primary_Path_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  728 =>
--#line  9446

   declare
      Wrap1 : constant Selective_Accept_Statement_Ptr :=
        new Selective_Accept_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  729 =>
--#line  9470
 
yyval := 
yy.value_stack(yy.tos-1);

when  730 =>
--#line  9475
 
yyval := 
yy.value_stack(yy.tos);

when  731 =>
--#line  9477
 
yyval := 
yy.value_stack(yy.tos);

when  732 =>
--#line  9479
 
yyval := 
yy.value_stack(yy.tos);

when  733 =>
--#line  9481
 
yyval := 
yy.value_stack(yy.tos);

when  734 =>
--#line  9486

   declare
      Wrap1 : constant Primary_Statement_Lists.List :=
        Primary_Statement_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Statement_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  735 =>
--#line  9496

   declare
      Wrap1 : constant Primary_Statement_Lists.List :=
        new Primary_Statement_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Statement_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  736 =>
--#line  9509

   declare
      Wrap1 : constant Primary_Statement_Lists.List :=
        Primary_Statement_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Statement_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  737 =>
--#line  9519

   declare
      Wrap1 : constant Primary_Statement_Lists.List :=
        new Primary_Statement_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Statement_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  738 =>
--#line  9532

   declare
      Wrap1 : constant Primary_Statement_Lists.List :=
        Primary_Statement_Lists.List (
yy.value_stack(yy.tos));
      Wrap2 : constant Terminate_Alternative_Statement_Ptr :=
        new Terminate_Alternative_Statement_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Statement_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  739 =>
--#line  9546

   declare
      Wrap1 : constant Primary_Statement_Lists.List :=
        new Primary_Statement_Lists.List_Node;
      Wrap2 : constant Terminate_Alternative_Statement_Ptr :=
        new Terminate_Alternative_Statement_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Statement_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  740 =>
--#line  9564
 
yyval := 
yy.value_stack(yy.tos-5);

when  741 =>
--#line  9569

   declare
      Wrap1 : constant Primary_Statement_Lists.List :=
        Primary_Statement_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Statement_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  742 =>
--#line  9579

   declare
      Wrap1 : constant Primary_Statement_Lists.List :=
        new Primary_Statement_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Statement_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  743 =>
--#line  9592
 
yyval := 
yy.value_stack(yy.tos);

when  744 =>
--#line  9598
 
yyval := 
yy.value_stack(yy.tos-5);

when  745 =>
--#line  9604

   declare
      Wrap1 : constant Asynchronous_Select_Statement_Ptr :=
        new Asynchronous_Select_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
      Wrap4 : constant Then_Abort_Path_Ptr :=
        new Then_Abort_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-8).all));
      Set_Pragmas (Wrap3.all, 
yy.value_stack(yy.tos-7));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-6));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-6).all));
      Set_Start_Position (Wrap4.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Sequence_Of_Statements (Wrap4.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap4.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap4));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  746 =>
--#line  9633

   declare
      Wrap1 : constant Asynchronous_Select_Statement_Ptr :=
        new Asynchronous_Select_Statement_Node;
      Wrap2 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap3 : constant Select_Path_Ptr :=
        new Select_Path_Node;
      Wrap4 : constant Then_Abort_Path_Ptr :=
        new Then_Abort_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Sequence_Of_Statements (Wrap3.all, 
yy.value_stack(yy.tos-6));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-6).all));
      Set_Start_Position (Wrap4.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Sequence_Of_Statements (Wrap4.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (Wrap4.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap4));
      Primary_Path_Lists.Add (Wrap2.all, T (Wrap3));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Statement_Paths (Wrap1.all, T (Wrap2));
   end;


when  747 =>
--#line  9663
 
yyval := 
yy.value_stack(yy.tos-1);

when  748 =>
--#line  9665
 
yyval := 
yy.value_stack(yy.tos);

when  749 =>
--#line  9670
 
yyval := 
yy.value_stack(yy.tos);

when  750 =>
--#line  9672
 
yyval := 
yy.value_stack(yy.tos);

when  751 =>
--#line  9677
 
yyval := 
yy.value_stack(yy.tos);

when  752 =>
--#line  9682

   declare
      Wrap1 : constant Abort_Statement_Ptr :=
        new Abort_Statement_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        Primary_Expression_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Aborted_Tasks (Wrap1.all, T (Wrap2));
   end;


when  753 =>
--#line  9697

   declare
      Wrap1 : constant Abort_Statement_Ptr :=
        new Abort_Statement_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Aborted_Tasks (Wrap1.all, T (Wrap2));
   end;


when  754 =>
--#line  9715

   declare
      Wrap1 : constant Primary_Unit_Lists.List :=
        Primary_Unit_Lists.List (
yy.value_stack(yy.tos));
   begin
      Last_Compilation := T (Wrap1);
   end;


when  755 =>
--#line  9724

   declare
      Wrap1 : constant Primary_Unit_Lists.List :=
        new Primary_Unit_Lists.List_Node;
   begin
      Last_Compilation := T (Wrap1);
   end;


when  756 =>
--#line  9736

   declare
      Wrap1 : constant Any_Compilation_Unit_Ptr :=
        Any_Compilation_Unit_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Context_Clause_Elements (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  757 =>
--#line  9747

   declare
      Wrap1 : constant Any_Compilation_Unit_Ptr :=
        Any_Compilation_Unit_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Context_Clause_Elements (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  758 =>
--#line  9758
 
yyval := 
yy.value_stack(yy.tos);

when  759 =>
--#line  9760

   declare
      Wrap1 : constant Any_Compilation_Unit_Ptr :=
        Any_Compilation_Unit_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  760 =>
--#line  9769

   declare
      Wrap1 : constant Any_Compilation_Unit_Ptr :=
        Any_Compilation_Unit_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  761 =>
--#line  9781

   declare
      New_Node : constant Any_Compilation_Unit_Ptr :=
        Any_Compilation_Unit_Ptr (New_Compilation_Unit (The_Context));
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Unit_Declaration (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  762 =>
--#line  9793

   declare
      New_Node : constant Any_Compilation_Unit_Ptr :=
        Any_Compilation_Unit_Ptr (New_Compilation_Unit (The_Context));
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Unit_Declaration (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  763 =>
--#line  9805

   declare
      New_Node : constant Any_Compilation_Unit_Ptr :=
        Any_Compilation_Unit_Ptr (New_Compilation_Unit (The_Context));
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Unit_Declaration (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  764 =>
--#line  9820
 
yyval := 
yy.value_stack(yy.tos);

when  765 =>
--#line  9822
 
yyval := 
yy.value_stack(yy.tos);

when  766 =>
--#line  9824
 
yyval := 
yy.value_stack(yy.tos);

when  767 =>
--#line  9826
 
yyval := 
yy.value_stack(yy.tos);

when  768 =>
--#line  9831
 
yyval := 
yy.value_stack(yy.tos);

when  769 =>
--#line  9833
 
yyval := 
yy.value_stack(yy.tos);

when  770 =>
--#line  9835
 
yyval := 
yy.value_stack(yy.tos);

when  771 =>
--#line  9840
 
yyval := 
yy.value_stack(yy.tos);

when  772 =>
--#line  9842
 
yyval := 
yy.value_stack(yy.tos);

when  773 =>
--#line  9847
 
yyval := 
yy.value_stack(yy.tos);

when  774 =>
--#line  9852

   declare
      Wrap1 : constant Primary_Clause_Lists.List :=
        Primary_Clause_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Clause_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  775 =>
--#line  9862

   declare
      Wrap1 : constant Primary_Clause_Lists.List :=
        new Primary_Clause_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Clause_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  776 =>
--#line  9875
 
yyval := 
yy.value_stack(yy.tos);

when  777 =>
--#line  9877
 
yyval := 
yy.value_stack(yy.tos);

when  778 =>
--#line  9879

   declare
      New_Node : constant Private_Indicator_Ptr :=
        new Private_Indicator_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  779 =>
--#line  9893
 
yyval := 
yy.value_stack(yy.tos);

when  780 =>
--#line  9895
 
yyval := 
yy.value_stack(yy.tos);

when  781 =>
--#line  9900

   declare
      Wrap1 : constant With_Clause_Ptr :=
        new With_Clause_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        Primary_Expression_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Has_Limited (Wrap1.all, True);
      Set_Has_Private (Wrap1.all, True);
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Clause_Names (Wrap1.all, T (Wrap2));
   end;


when  782 =>
--#line  9917

   declare
      Wrap1 : constant With_Clause_Ptr :=
        new With_Clause_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Has_Limited (Wrap1.all, True);
      Set_Has_Private (Wrap1.all, True);
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Clause_Names (Wrap1.all, T (Wrap2));
   end;


when  783 =>
--#line  9934

   declare
      Wrap1 : constant With_Clause_Ptr :=
        new With_Clause_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        Primary_Expression_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Has_Limited (Wrap1.all, True);
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Clause_Names (Wrap1.all, T (Wrap2));
   end;


when  784 =>
--#line  9950

   declare
      Wrap1 : constant With_Clause_Ptr :=
        new With_Clause_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Has_Limited (Wrap1.all, True);
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Clause_Names (Wrap1.all, T (Wrap2));
   end;


when  785 =>
--#line  9969

   declare
      Wrap1 : constant With_Clause_Ptr :=
        new With_Clause_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        Primary_Expression_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Clause_Names (Wrap1.all, T (Wrap2));
   end;


when  786 =>
--#line  9984

   declare
      Wrap1 : constant With_Clause_Ptr :=
        new With_Clause_Node;
      Wrap2 : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Primary_Expression_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Clause_Names (Wrap1.all, T (Wrap2));
   end;


when  787 =>
--#line  10002
 
yyval := 
yy.value_stack(yy.tos);

when  788 =>
--#line  10004
 
yyval := 
yy.value_stack(yy.tos);

when  789 =>
--#line  10006
 
yyval := 
yy.value_stack(yy.tos);

when  790 =>
--#line  10008
 
yyval := 
yy.value_stack(yy.tos);

when  791 =>
--#line  10014

   if 
yy.value_stack(yy.tos-3).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Body_Stub_Ptr :=
           new Procedure_Body_Stub_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Body_Stub_Ptr :=
           new Function_Body_Stub_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  792 =>
--#line  10040

   if 
yy.value_stack(yy.tos-3).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Body_Stub_Ptr :=
           new Procedure_Body_Stub_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Body_Stub_Ptr :=
           new Function_Body_Stub_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  793 =>
--#line  10068

   declare
      New_Node : constant Package_Body_Stub_Ptr :=
        new Package_Body_Stub_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  794 =>
--#line  10083

   declare
      New_Node : constant Task_Body_Stub_Ptr :=
        new Task_Body_Stub_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  795 =>
--#line  10098

   declare
      New_Node : constant Protected_Body_Stub_Ptr :=
        new Protected_Body_Stub_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  796 =>
--#line  10114

   declare
      New_Node : constant Any_Compilation_Unit_Ptr :=
        Any_Compilation_Unit_Ptr (New_Compilation_Unit (The_Context));
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Separate_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Unit_Declaration (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  797 =>
--#line  10130

   declare
      New_Node : constant Exception_Declaration_Ptr :=
        new Exception_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  798 =>
--#line  10145

   declare
      New_Node : constant Handled_Statements_Ptr :=
        new Handled_Statements_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Statements (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Exception_Handlers (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  799 =>
--#line  10158

   declare
      New_Node : constant Handled_Statements_Ptr :=
        new Handled_Statements_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Statements (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  800 =>
--#line  10174

   declare
      Wrap1 : constant Exception_Handler_Ptr :=
        new Exception_Handler_Node;
      Wrap2 : constant Primary_Choise_Lists.List :=
        Primary_Choise_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Choice_Parameter_Specification (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Primary_Choise_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Handler_Statements (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Exception_Choices (Wrap1.all, T (Wrap2));
   end;


when  801 =>
--#line  10192

   declare
      Wrap1 : constant Exception_Handler_Ptr :=
        new Exception_Handler_Node;
      Wrap2 : constant Primary_Choise_Lists.List :=
        new Primary_Choise_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Choice_Parameter_Specification (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Primary_Choise_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_Handler_Statements (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Exception_Choices (Wrap1.all, T (Wrap2));
   end;


when  802 =>
--#line  10210

   declare
      Wrap1 : constant Exception_Handler_Ptr :=
        new Exception_Handler_Node;
      Wrap2 : constant Primary_Choise_Lists.List :=
        Primary_Choise_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Primary_Choise_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Handler_Statements (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Exception_Choices (Wrap1.all, T (Wrap2));
   end;


when  803 =>
--#line  10226

   declare
      Wrap1 : constant Exception_Handler_Ptr :=
        new Exception_Handler_Node;
      Wrap2 : constant Primary_Choise_Lists.List :=
        new Primary_Choise_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Primary_Choise_Lists.Add (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_Handler_Statements (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Exception_Choices (Wrap1.all, T (Wrap2));
   end;


when  804 =>
--#line  10245

   declare
      New_Node : constant Choice_Parameter_Specification_Ptr :=
        new Choice_Parameter_Specification_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  805 =>
--#line  10260
 
yyval := 
yy.value_stack(yy.tos);

when  806 =>
--#line  10262

   declare
      New_Node : constant Others_Choice_Ptr :=
        new Others_Choice_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  807 =>
--#line  10276

   declare
      New_Node : constant Raise_Statement_Ptr :=
        new Raise_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  808 =>
--#line  10287

   declare
      New_Node : constant Raise_Statement_Ptr :=
        new Raise_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Raised_Exception (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Raise_Statement_Message (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  809 =>
--#line  10300

   declare
      New_Node : constant Raise_Statement_Ptr :=
        new Raise_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Raised_Exception (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  810 =>
--#line  10315
 
yyval := 
yy.value_stack(yy.tos);

when  811 =>
--#line  10317
 
yyval := 
yy.value_stack(yy.tos);

when  812 =>
--#line  10322

   if 
yy.value_stack(yy.tos-1).all in Procedure_Specification_Node then
      declare
         New_Node : constant Generic_Procedure_Declaration_Ptr :=
           new Generic_Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
         Set_Generic_Formal_Part (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Generic_Function_Declaration_Ptr :=
           new Generic_Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
         Set_Generic_Formal_Part (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  813 =>
--#line  10351

   declare
      New_Node : constant Generic_Package_Declaration_Ptr :=
        new Generic_Package_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Generic_Formal_Part (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  814 =>
--#line  10367

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
   end;


when  815 =>
--#line  10377

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  816 =>
--#line  10391
 
yyval := 
yy.value_stack(yy.tos);

when  817 =>
--#line  10393
 
yyval := 
yy.value_stack(yy.tos);

when  818 =>
--#line  10395
 
yyval := 
yy.value_stack(yy.tos);

when  819 =>
--#line  10397
 
yyval := 
yy.value_stack(yy.tos);

when  820 =>
--#line  10403

   declare
      New_Node : constant Package_Instantiation_Ptr :=
        new Package_Instantiation_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Generic_Actual_Part (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  821 =>
--#line  10418

   declare
      New_Node : constant Package_Instantiation_Ptr :=
        new Package_Instantiation_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  822 =>
--#line  10432

   if 
yy.value_stack(yy.tos-5).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Instantiation_Ptr :=
           new Procedure_Instantiation_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_Generic_Actual_Part (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Instantiation_Ptr :=
           new Function_Instantiation_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_Generic_Actual_Part (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  823 =>
--#line  10463

   if 
yy.value_stack(yy.tos-4).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Instantiation_Ptr :=
           new Procedure_Instantiation_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Instantiation_Ptr :=
           new Function_Instantiation_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
         Set_Overriding_Indicator_Kind (New_Node.all, Last_Overriding_Kind);
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  824 =>
--#line  10492

   if 
yy.value_stack(yy.tos-5).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Instantiation_Ptr :=
           new Procedure_Instantiation_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_Generic_Actual_Part (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Instantiation_Ptr :=
           new Function_Instantiation_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_Generic_Actual_Part (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  825 =>
--#line  10520

   if 
yy.value_stack(yy.tos-4).all in Procedure_Specification_Node then
      declare
         New_Node : constant Procedure_Instantiation_Ptr :=
           new Procedure_Instantiation_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Function_Instantiation_Ptr :=
           new Function_Instantiation_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  826 =>
--#line  10550

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        Primary_Association_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-2));
   end;


when  827 =>
--#line  10560

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  828 =>
--#line  10573

   declare
      New_Node : constant Generic_Association_Ptr :=
        new Generic_Association_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Formal_Parameter (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Actual_Parameter (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  829 =>
--#line  10586

   declare
      New_Node : constant Generic_Association_Ptr :=
        new Generic_Association_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Actual_Parameter (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  830 =>
--#line  10601
 
yyval := 
yy.value_stack(yy.tos);

when  831 =>
--#line  10607

   declare
      Wrap1 : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-6));
      Set_Mode_Kind (Wrap1.all, Modes.
yy.value_stack(yy.tos-4));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Initialization_Expression (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  832 =>
--#line  10629

   declare
      Wrap1 : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Mode_Kind (Wrap1.all, Modes.
yy.value_stack(yy.tos-3));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  833 =>
--#line  10650

   declare
      Wrap1 : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Mode_Kind (Wrap1.all, Modes.
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Initialization_Expression (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  834 =>
--#line  10670

   declare
      Wrap1 : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Mode_Kind (Wrap1.all, Modes.
yy.value_stack(yy.tos-2));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  835 =>
--#line  10690

   declare
      Wrap1 : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Initialization_Expression (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  836 =>
--#line  10711

   declare
      Wrap1 : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Has_Null_Exclusion (Wrap1.all, True);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  837 =>
--#line  10731

   declare
      Wrap1 : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-4));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Initialization_Expression (Wrap1.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  838 =>
--#line  10750

   declare
      Wrap1 : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
      Wrap2 : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Object_Declaration_Subtype (Wrap1.all, T (Wrap2));
   end;


when  839 =>
--#line  10769

   declare
      New_Node : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_Mode_Kind (New_Node.all, Modes.
yy.value_stack(yy.tos-3));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  840 =>
--#line  10785

   declare
      New_Node : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Mode_Kind (New_Node.all, Modes.
yy.value_stack(yy.tos-2));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  841 =>
--#line  10800

   declare
      New_Node : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Initialization_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  842 =>
--#line  10814

   declare
      New_Node : constant Formal_Object_Declaration_Ptr :=
        new Formal_Object_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Names (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Object_Declaration_Subtype (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  843 =>
--#line  10831

   declare
      New_Node : constant Formal_Type_Declaration_Ptr :=
        new Formal_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Discriminant_Part (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  844 =>
--#line  10846

   declare
      New_Node : constant Formal_Type_Declaration_Ptr :=
        new Formal_Type_Declaration_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Type_Declaration_View (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  845 =>
--#line  10862
 
yyval := 
yy.value_stack(yy.tos);

when  846 =>
--#line  10864
 
yyval := 
yy.value_stack(yy.tos);

when  847 =>
--#line  10866
 
yyval := 
yy.value_stack(yy.tos);

when  848 =>
--#line  10868
 
yyval := 
yy.value_stack(yy.tos);

when  849 =>
--#line  10870
 
yyval := 
yy.value_stack(yy.tos);

when  850 =>
--#line  10872
 
yyval := 
yy.value_stack(yy.tos);

when  851 =>
--#line  10874
 
yyval := 
yy.value_stack(yy.tos);

when  852 =>
--#line  10876
 
yyval := 
yy.value_stack(yy.tos);

when  853 =>
--#line  10878
 
yyval := 
yy.value_stack(yy.tos);

when  854 =>
--#line  10880
 
yyval := 
yy.value_stack(yy.tos);

when  855 =>
--#line  10882
 
yyval := 
yy.value_stack(yy.tos);

when  856 =>
--#line  10887

   declare
      New_Node : constant Formal_Tagged_Private_Type_Ptr :=
        new Formal_Tagged_Private_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Has_Tagged (New_Node.all, True);
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  857 =>
--#line  10904

   declare
      New_Node : constant Formal_Tagged_Private_Type_Ptr :=
        new Formal_Tagged_Private_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Has_Tagged (New_Node.all, True);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  858 =>
--#line  10919

   declare
      New_Node : constant Formal_Tagged_Private_Type_Ptr :=
        new Formal_Tagged_Private_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Tagged (New_Node.all, True);
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  859 =>
--#line  10934

   declare
      New_Node : constant Formal_Tagged_Private_Type_Ptr :=
        new Formal_Tagged_Private_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Has_Tagged (New_Node.all, True);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  860 =>
--#line  10947

   declare
      New_Node : constant Formal_Private_Type_Ptr :=
        new Formal_Private_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  861 =>
--#line  10961

   declare
      New_Node : constant Formal_Private_Type_Ptr :=
        new Formal_Private_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  862 =>
--#line  10976

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Has_Synchronized (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  863 =>
--#line  10994

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Has_Synchronized (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  864 =>
--#line  11011

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Has_Synchronized (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  865 =>
--#line  11026

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  866 =>
--#line  11045

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  867 =>
--#line  11063

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  868 =>
--#line  11079

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  869 =>
--#line  11096

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  870 =>
--#line  11112

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Trait_Kind (New_Node.all, An_Abstract_Trait);
      Set_Has_Abstract (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  871 =>
--#line  11126

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Has_Synchronized (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  872 =>
--#line  11142

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Has_Synchronized (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  873 =>
--#line  11157

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Has_Synchronized (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  874 =>
--#line  11170

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  875 =>
--#line  11187

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  876 =>
--#line  11203

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Trait_Kind (New_Node.all, A_Limited_Trait);
      Set_Has_Limited (New_Node.all, True);
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  877 =>
--#line  11217

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Progenitor_List (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  878 =>
--#line  11232

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Trait_Kind (New_Node.all, A_Private_Trait);
      Set_Has_Private (New_Node.all, True);
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  879 =>
--#line  11246

   declare
      New_Node : constant Formal_Derived_Type_Ptr :=
        new Formal_Derived_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  880 =>
--#line  11261

   declare
      New_Node : constant Formal_Discrete_Type_Ptr :=
        new Formal_Discrete_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  881 =>
--#line  11275

   declare
      New_Node : constant Formal_Signed_Integer_Type_Ptr :=
        new Formal_Signed_Integer_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  882 =>
--#line  11289

   declare
      New_Node : constant Formal_Modular_Type_Ptr :=
        new Formal_Modular_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  883 =>
--#line  11303

   declare
      New_Node : constant Formal_Floating_Point_Ptr :=
        new Formal_Floating_Point_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  884 =>
--#line  11317

   declare
      New_Node : constant Formal_Ordinary_Fixed_Point_Ptr :=
        new Formal_Ordinary_Fixed_Point_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  885 =>
--#line  11331

   declare
      New_Node : constant Formal_Decimal_Fixed_Point_Ptr :=
        new Formal_Decimal_Fixed_Point_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  886 =>
--#line  11345

   if 
yy.value_stack(yy.tos).all in Unconstrained_Array_Node then
      declare
         New_Node : constant Formal_Unconstrained_Array_Ptr :=
           new Formal_Unconstrained_Array_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
         Set_Array_Definition (New_Node.all, 
yy.value_stack(yy.tos));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Formal_Constrained_Array_Ptr :=
           new Formal_Constrained_Array_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
         Set_Array_Definition (New_Node.all, 
yy.value_stack(yy.tos));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  887 =>
--#line  11372

   declare
      New_Node : constant Formal_Access_Type_Ptr :=
        new Formal_Access_Type_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Access_Definition (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  888 =>
--#line  11387
 
yyval := 
yy.value_stack(yy.tos);

when  889 =>
--#line  11392
 
yyval := 
yy.value_stack(yy.tos);

when  890 =>
--#line  11394
 
yyval := 
yy.value_stack(yy.tos);

when  891 =>
--#line  11400

   if 
yy.value_stack(yy.tos-2).all in Procedure_Specification_Node then
      declare
         New_Node : constant Formal_Procedure_Declaration_Ptr :=
           new Formal_Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_Formal_Subprogram_Default (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Formal_Function_Declaration_Ptr :=
           new Formal_Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_Formal_Subprogram_Default (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  892 =>
--#line  11426

   if 
yy.value_stack(yy.tos-1).all in Procedure_Specification_Node then
      declare
         New_Node : constant Formal_Procedure_Declaration_Ptr :=
           new Formal_Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Formal_Function_Declaration_Ptr :=
           new Formal_Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  893 =>
--#line  11454

   if 
yy.value_stack(yy.tos-4).all in Procedure_Specification_Node then
      declare
         New_Node : constant Formal_Procedure_Declaration_Ptr :=
           new Formal_Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Has_Abstract (New_Node.all, True);
         Set_Formal_Subprogram_Default (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Formal_Function_Declaration_Ptr :=
           new Formal_Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-4));
         Set_Has_Abstract (New_Node.all, True);
         Set_Formal_Subprogram_Default (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  894 =>
--#line  11482

   if 
yy.value_stack(yy.tos-3).all in Procedure_Specification_Node then
      declare
         New_Node : constant Formal_Procedure_Declaration_Ptr :=
           new Formal_Procedure_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Has_Abstract (New_Node.all, True);
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Formal_Function_Declaration_Ptr :=
           new Formal_Function_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
         Set_Specification (New_Node.all, 
yy.value_stack(yy.tos-3));
         Set_Has_Abstract (New_Node.all, True);
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  895 =>
--#line  11511
 
yyval := 
yy.value_stack(yy.tos);

when  896 =>
--#line  11513

   declare
      New_Node : constant Box_Expression_Ptr :=
        new Box_Expression_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  897 =>
--#line  11524

   declare
      New_Node : constant Null_Literal_Ptr :=
        new Null_Literal_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  898 =>
--#line  11538
 
yyval := 
yy.value_stack(yy.tos);

when  899 =>
--#line  11544

   if 
yy.value_stack(yy.tos-1).all in Primary_Association_Lists.List_Node then
      declare
         New_Node : constant Formal_Package_Declaration_Ptr :=
           new Formal_Package_Declaration_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
         Set_Name (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_Generic_Actual_Part (New_Node.all, 
yy.value_stack(yy.tos-1));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   else
      declare
         New_Node : constant Formal_Package_Declaration_With_Box_Ptr :=
           new Formal_Package_Declaration_With_Box_Node;
      begin
         
yyval := YYSTYPE (New_Node);
         Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
         Set_Name (New_Node.all, 
yy.value_stack(yy.tos-5));
         Set_Generic_Unit_Name (New_Node.all, 
yy.value_stack(yy.tos-2));
         Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      end;
   end if;


when  901 =>
--#line  11575

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Wrap2 : constant Generic_Association_Ptr :=
        new Generic_Association_Node;
      Wrap3 : constant Others_Choice_Ptr :=
        new Others_Choice_Node;
      Wrap4 : constant Box_Expression_Ptr :=
        new Box_Expression_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos-3).all));
      Set_Start_Position (Wrap4.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap4.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Actual_Parameter (Wrap2.all, T (Wrap4));
      Set_End_Position (Wrap2.all, End_Position (T (Wrap4).all));
      Set_Start_Position (Wrap2.all, Start_Position (T (Wrap3).all));
      Set_Formal_Parameter (Wrap2.all, T (Wrap3));
      Primary_Association_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  902 =>
--#line  11599

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  903 =>
--#line  11609

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        Primary_Association_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-3));
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  904 =>
--#line  11621

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        Primary_Association_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-2));
   end;


when  905 =>
--#line  11632

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-2));
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  906 =>
--#line  11643

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  907 =>
--#line  11656
 
yyval := 
yy.value_stack(yy.tos);

when  908 =>
--#line  11658

   declare
      Wrap1 : constant Generic_Association_Ptr :=
        new Generic_Association_Node;
      Wrap2 : constant Box_Expression_Ptr :=
        new Box_Expression_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Formal_Parameter (Wrap1.all, 
yy.value_stack(yy.tos-2));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Actual_Parameter (Wrap1.all, T (Wrap2));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap2).all));
   end;


when  909 =>
--#line  11679
 
yyval := 
yy.value_stack(yy.tos);

when  910 =>
--#line  11681
 
yyval := 
yy.value_stack(yy.tos);

when  911 =>
--#line  11683
 
yyval := 
yy.value_stack(yy.tos);

when  912 =>
--#line  11688
 
yyval := 
yy.value_stack(yy.tos);

when  913 =>
--#line  11690

   declare
      New_Node : constant Attribute_Reference_Ptr :=
        new Attribute_Reference_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Prefix (New_Node.all, 
yy.value_stack(yy.tos-2));
      Set_Attribute_Designator_Identifier (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  914 =>
--#line  11706

   declare
      New_Node : constant At_Clause_Ptr :=
        new At_Clause_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-5).all));
      Set_Representation_Clause_Name (New_Node.all, 
yy.value_stack(yy.tos-4));
      Set_Representation_Clause_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  915 =>
--#line  11719

   declare
      New_Node : constant Attribute_Definition_Clause_Ptr :=
        new Attribute_Definition_Clause_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Representation_Clause_Name (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_Representation_Clause_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  916 =>
--#line  11735
 
yyval := 
yy.value_stack(yy.tos-3);

when  917 =>
--#line  11740
 
yyval := 
yy.value_stack(yy.tos);

when  918 =>
--#line  11746

   declare
      New_Node : constant Record_Representation_Clause_Ptr :=
        new Record_Representation_Clause_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Representation_Clause_Name (New_Node.all, 
yy.value_stack(yy.tos-6));
      Set_Component_Clauses (New_Node.all, 
yy.value_stack(yy.tos-3));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  919 =>
--#line  11759

   declare
      New_Node : constant Record_Representation_Clause_Ptr :=
        new Record_Representation_Clause_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-6).all));
      Set_Representation_Clause_Name (New_Node.all, 
yy.value_stack(yy.tos-5));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  920 =>
--#line  11775

   declare
      Wrap1 : constant Component_Clause_Ptr :=
        new Component_Clause_Node;
      Wrap2 : constant Discrete_Simple_Expression_Range_Ptr :=
        new Discrete_Simple_Expression_Range_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-7).all));
      Set_Representation_Clause_Name (Wrap1.all, 
yy.value_stack(yy.tos-7));
      Set_Component_Clause_Position (Wrap1.all, 
yy.value_stack(yy.tos-5));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-4).all));
      Set_Lower_Bound (Wrap2.all, 
yy.value_stack(yy.tos-3));
      Set_Upper_Bound (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Component_Clause_Range (Wrap1.all, T (Wrap2));
   end;


when  921 =>
--#line  11795
 
yyval := 
yy.value_stack(yy.tos);

when  922 =>
--#line  11797
 
yyval := 
yy.value_stack(yy.tos);

when  923 =>
--#line  11802
 
yyval := 
yy.value_stack(yy.tos);

when  924 =>
--#line  11807
 
yyval := 
yy.value_stack(yy.tos);

when  925 =>
--#line  11812
 
yyval := 
yy.value_stack(yy.tos);

when  926 =>
--#line  11817

   declare
      New_Node : constant Code_Statement_Ptr :=
        new Code_Statement_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Qualified_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  927 =>
--#line  11832
 
yyval := 
yy.value_stack(yy.tos);

when  928 =>
--#line  11834
 
yyval := 
yy.value_stack(yy.tos-2);

when  929 =>
--#line  11839
 
yyval := 
yy.value_stack(yy.tos);

when  930 =>
--#line  11841
 
yyval := 
yy.value_stack(yy.tos);

when  931 =>
--#line  11846

   declare
      New_Node : constant Delta_Constraint_Ptr :=
        new Delta_Constraint_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Delta_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Real_Range_Constraint (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  932 =>
--#line  11859

   declare
      New_Node : constant Delta_Constraint_Ptr :=
        new Delta_Constraint_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Delta_Expression (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  933 =>
--#line  11874
 
yyval := 
yy.value_stack(yy.tos-4);

when  934 =>
--#line  11879

   declare
      New_Node : constant Record_Representation_Clause_Ptr :=
        new Record_Representation_Clause_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Mod_Clause_Expression (New_Node.all, 
yy.value_stack(yy.tos-1));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  935 =>
--#line  11894

   declare
      Wrap1 : constant Variant_Part_Ptr :=
        Variant_Part_Ptr (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_End_Pragmas (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  936 =>
--#line  11905

   declare
      Wrap1 : constant Variant_Part_Ptr :=
        Variant_Part_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
   end;


when  937 =>
--#line  11917

   declare
      Wrap1 : constant Primary_Identifier_Lists.List :=
        Primary_Identifier_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Identifier_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  938 =>
--#line  11927

   declare
      Wrap1 : constant Primary_Identifier_Lists.List :=
        new Primary_Identifier_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Identifier_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos));
   end;


when  939 =>
--#line  11940
 
yyval := 
yy.value_stack(yy.tos);

when  940 =>
--#line  11942
 
yyval := 
yy.value_stack(yy.tos);

when  941 =>
--#line  11944
 
yyval := 
yy.value_stack(yy.tos);

when  942 =>
--#line  11949
 
yyval := 
yy.value_stack(yy.tos);

when  943 =>
--#line  11951
 
yyval := 
yy.value_stack(yy.tos);

when  944 =>
--#line  11956

   declare
      New_Node : constant Subtype_Indication_Ptr :=
        new Subtype_Indication_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Subtype_Mark (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  945 =>
--#line  11972

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        Primary_Association_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-2));
   end;


when  946 =>
--#line  11982

   declare
      Wrap1 : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Primary_Association_Lists.Add (Wrap1.all, 
yy.value_stack(yy.tos-1));
   end;


when  947 =>
--#line  11995

   declare
      New_Node : constant Primary_Association_Lists.List :=
        Primary_Association_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Association_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  948 =>
--#line  12005

   declare
      New_Node : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Association_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  949 =>
--#line  12018
 
yyval := 
yy.value_stack(yy.tos-1);

when  950 =>
--#line  12023
 
yyval := 
yy.value_stack(yy.tos);

when  951 =>
--#line  12028

   declare
      New_Node : constant Primary_Defining_Name_Lists.List :=
        Primary_Defining_Name_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Defining_Name_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  952 =>
--#line  12038

   declare
      New_Node : constant Primary_Defining_Name_Lists.List :=
        new Primary_Defining_Name_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Defining_Name_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  953 =>
--#line  12051
 
yyval := 
yy.value_stack(yy.tos);

when  954 =>
--#line  12057

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  955 =>
--#line  12067

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  956 =>
--#line  12080

   declare
      New_Node : constant Primary_Identifier_Lists.List :=
        Primary_Identifier_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Identifier_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  957 =>
--#line  12090

   declare
      New_Node : constant Primary_Identifier_Lists.List :=
        new Primary_Identifier_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Identifier_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  958 =>
--#line  12103

   declare
      New_Node : constant Primary_Definition_Lists.List :=
        Primary_Definition_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Definition_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  959 =>
--#line  12113

   declare
      New_Node : constant Primary_Definition_Lists.List :=
        new Primary_Definition_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Definition_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  960 =>
--#line  12126

   declare
      New_Node : constant Primary_Definition_Lists.List :=
        Primary_Definition_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Definition_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  961 =>
--#line  12136

   declare
      New_Node : constant Primary_Definition_Lists.List :=
        new Primary_Definition_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Definition_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  962 =>
--#line  12150

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  963 =>
--#line  12160

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  964 =>
--#line  12173
 
yyval := 
yy.value_stack(yy.tos);

when  965 =>
--#line  12178
 
yyval := 
yy.value_stack(yy.tos);

when  966 =>
--#line  12180
 
yyval := 
yy.value_stack(yy.tos);

when  969 =>
--#line  12190
 
yyval := 
yy.value_stack(yy.tos);

when  970 =>
--#line  12192
 
yyval := 
yy.value_stack(yy.tos);

when  971 =>
--#line  12197

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  972 =>
--#line  12207

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  973 =>
--#line  12220

   declare
      New_Node : constant Primary_Statement_Lists.List :=
        Primary_Statement_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Statement_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  974 =>
--#line  12230

   declare
      New_Node : constant Primary_Statement_Lists.List :=
        new Primary_Statement_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Statement_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  975 =>
--#line  12243

   declare
      New_Node : constant Primary_Variant_Lists.List :=
        Primary_Variant_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Variant_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  976 =>
--#line  12253

   declare
      New_Node : constant Primary_Variant_Lists.List :=
        new Primary_Variant_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Variant_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  977 =>
--#line  12266

   declare
      New_Node : constant Primary_Choise_Lists.List :=
        Primary_Choise_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Choise_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  978 =>
--#line  12276

   declare
      New_Node : constant Primary_Choise_Lists.List :=
        new Primary_Choise_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Choise_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  979 =>
--#line  12289

   declare
      New_Node : constant Primary_Expression_Lists.List :=
        Primary_Expression_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Primary_Expression_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  980 =>
--#line  12300

   declare
      New_Node : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Primary_Expression_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  981 =>
--#line  12314

   declare
      Wrap1 : constant Tagged_Incomplete_Type_Definition_Ptr :=
        new Tagged_Incomplete_Type_Definition_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_Has_Tagged (Wrap1.all, True);
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  982 =>
--#line  12329

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  983 =>
--#line  12339

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  984 =>
--#line  12352
 
yyval := 
yy.value_stack(yy.tos);

when  985 =>
--#line  12354
 
yyval := 
yy.value_stack(yy.tos);

when  986 =>
--#line  12359
 
yyval := 
yy.value_stack(yy.tos-1);

when  987 =>
--#line  12364

   declare
      New_Node : constant Primary_Association_Lists.List :=
        Primary_Association_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Association_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  988 =>
--#line  12374

   declare
      New_Node : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Association_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  989 =>
--#line  12387
 
yyval := 
yy.value_stack(yy.tos-1);

when  990 =>
--#line  12392
 
yyval := 
yy.value_stack(yy.tos);

when  991 =>
--#line  12394
 
yyval := 
yy.value_stack(yy.tos);

when  992 =>
--#line  12399

   declare
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Call_Node : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Prefix (Call_Node.all, T (Sym_Node));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  993 =>
--#line  12432

   declare
      Call_Node  : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node   : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
      Arg_2_Node : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
   begin
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_2_Node));
      Set_Actual_Parameter (Arg_Node.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, T (Sym_Node));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  994 =>
--#line  12467

   declare
      Node1 : constant And_Then_Short_Circuit_Ptr:=
        new And_Then_Short_Circuit_Node;
   begin
      Set_Short_Circuit_Operation_Left_Expression (Node1.all, 
yy.value_stack(yy.tos-3));
      Set_Short_Circuit_Operation_Right_Expression (Node1.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Node1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Node1.all, End_Position (
yy.value_stack(yy.tos).all));
      
yyval := YYSTYPE (Node1);
   end;


when  995 =>
--#line  12480

   declare
      Node1 : constant And_Then_Short_Circuit_Ptr:=
        new And_Then_Short_Circuit_Node;
   begin
      Set_Short_Circuit_Operation_Right_Expression (Node1.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Node1.all, End_Position (
yy.value_stack(yy.tos).all));
      
yyval := YYSTYPE (Node1);
   end;


when  996 =>
--#line  12495

   declare
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Call_Node : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Prefix (Call_Node.all, T (Sym_Node));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  997 =>
--#line  12528

   declare
      Call_Node  : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node   : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
      Arg_2_Node : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
   begin
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_2_Node));
      Set_Actual_Parameter (Arg_Node.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, T (Sym_Node));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  998 =>
--#line  12563

   declare
      Node1 : constant Or_Else_Short_Circuit_Ptr:=
        new Or_Else_Short_Circuit_Node;
   begin
      Set_Short_Circuit_Operation_Left_Expression (Node1.all, 
yy.value_stack(yy.tos-3));
      Set_Short_Circuit_Operation_Right_Expression (Node1.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Node1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_End_Position (Node1.all, End_Position (
yy.value_stack(yy.tos).all));
      
yyval := YYSTYPE (Node1);
   end;


when  999 =>
--#line  12576

   declare
      Node1 : constant Or_Else_Short_Circuit_Ptr:=
        new Or_Else_Short_Circuit_Node;
   begin
      Set_Short_Circuit_Operation_Right_Expression (Node1.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Node1.all, End_Position (
yy.value_stack(yy.tos).all));
      
yyval := YYSTYPE (Node1);
   end;


when  1000 =>
--#line  12591

   declare
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Call_Node : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Prefix (Call_Node.all, T (Sym_Node));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  1001 =>
--#line  12624

   declare
      Call_Node  : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node   : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
      Arg_2_Node : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
   begin
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_2_Node));
      Set_Actual_Parameter (Arg_Node.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, T (Sym_Node));
      Set_Name_Image (Sym_Node.all, '"' & To_String (Raw_Image (
yy.value_stack(yy.tos-1).all)) & '"');
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  1002 =>
--#line  12659

   declare
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Call_Node : constant Function_Call_Ptr :=
        new Function_Call_Node;
   begin
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  1003 =>
--#line  12688

   declare
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Call_Node : constant Function_Call_Ptr :=
        new Function_Call_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  1004 =>
--#line  12716

   declare
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Call_Node : constant Function_Call_Ptr :=
        new Function_Call_Node;
   begin
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  1005 =>
--#line  12745

   declare
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Call_Node : constant Function_Call_Ptr :=
        new Function_Call_Node;
   begin
      Set_Actual_Parameter (Arg_Node1.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Arg_Node1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Arg_Node1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  1006 =>
--#line  12773

   declare
      Arg_Node1  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_Node2  : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Call_Node : constant Function_Call_Ptr :=
        new Function_Call_Node;
   begin
      Set_Actual_Parameter (Arg_Node2.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node2.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node1));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node2));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, 
yy.value_stack(yy.tos-1));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  1007 =>
--#line  12801

   declare
      Call_Node  : constant Function_Call_Ptr :=
        new Function_Call_Node;
      Arg_List   : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
      Arg_Node   : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
      Sym_Node   : constant Operator_Symbol_Ptr :=
        new Operator_Symbol_Node;
      Arg_2_Node : constant Parameter_Association_Ptr :=
        new Parameter_Association_Node;
   begin
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_2_Node));
      Set_Actual_Parameter (Arg_Node.all, 
yy.value_stack(yy.tos));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Association_Lists.Add (Arg_List.all, T (Arg_Node));
      Set_Function_Call_Parameters (Call_Node.all, T (Arg_List));
      Set_Start_Position (Arg_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Arg_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Prefix (Call_Node.all, T (Sym_Node));
      Set_Name_Image (Sym_Node.all, """**""");
      Set_Start_Position (Sym_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Sym_Node.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Call_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Call_Node.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Is_Prefix_Call (Call_Node.all, False);
      
yyval := YYSTYPE (Call_Node);
   end;


when  1008 =>
--#line  12836

   declare
      New_Node : constant Primary_Statement_Lists.List :=
        Primary_Statement_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Statement_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1009 =>
--#line  12846

   declare
      New_Node : constant Primary_Statement_Lists.List :=
        new Primary_Statement_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Statement_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1010 =>
--#line  12859

   declare
      New_Node : constant Primary_Defining_Name_Lists.List :=
        Primary_Defining_Name_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Defining_Name_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1011 =>
--#line  12869

   declare
      New_Node : constant Primary_Defining_Name_Lists.List :=
        new Primary_Defining_Name_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Defining_Name_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1012 =>
--#line  12882

   declare
      Wrap1 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-4));
      Wrap2 : constant Elsif_Path_Ptr :=
        new Elsif_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Condition_Expression (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Sequence_Of_Statements (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1013 =>
--#line  12900

   declare
      Wrap1 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap2 : constant Elsif_Path_Ptr :=
        new Elsif_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Condition_Expression (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Sequence_Of_Statements (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1014 =>
--#line  12921

   declare
      New_Node : constant Else_Path_Ptr :=
        new Else_Path_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position (New_Node.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Sequence_Of_Statements (New_Node.all, 
yy.value_stack(yy.tos));
      Set_End_Position (New_Node.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  1015 =>
--#line  12936

   declare
      New_Node : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Path_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1016 =>
--#line  12946

   declare
      New_Node : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Path_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1017 =>
--#line  12959
 
yyval := 
yy.value_stack(yy.tos-1);

when  1018 =>
--#line  12964

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
   end;


when  1019 =>
--#line  12977
 
yyval := 
yy.value_stack(yy.tos);

when  1020 =>
--#line  12982

   declare
      New_Node : constant Primary_Parameter_Lists.List :=
        Primary_Parameter_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Parameter_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1021 =>
--#line  12992

   declare
      New_Node : constant Primary_Parameter_Lists.List :=
        new Primary_Parameter_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Parameter_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1022 =>
--#line  13005
 
yyval := 
yy.value_stack(yy.tos);

when  1023 =>
--#line  13007
 
yyval := 
yy.value_stack(yy.tos);

when  1024 =>
--#line  13012
 
yyval := 
yy.value_stack(yy.tos-1);

when  1025 =>
--#line  13017
 
yyval := 
yy.value_stack(yy.tos-2);

when  1026 =>
--#line  13022

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1027 =>
--#line  13032

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1028 =>
--#line  13045

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
   end;


when  1029 =>
--#line  13055

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  1030 =>
--#line  13069
 
yyval := 
yy.value_stack(yy.tos);

when  1031 =>
--#line  13074

   declare
      New_Node : constant Primary_Expression_Lists.List :=
        Primary_Expression_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Expression_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1032 =>
--#line  13084

   declare
      New_Node : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Expression_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1033 =>
--#line  13097

   declare
      New_Node : constant Primary_Expression_Lists.List :=
        Primary_Expression_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Expression_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1034 =>
--#line  13107

   declare
      New_Node : constant Primary_Expression_Lists.List :=
        new Primary_Expression_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Expression_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1035 =>
--#line  13120

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1036 =>
--#line  13130

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1037 =>
--#line  13143

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
   end;


when  1038 =>
--#line  13153

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  1039 =>
--#line  13167
 
yyval := 
yy.value_stack(yy.tos-1);

when  1040 =>
--#line  13172

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1041 =>
--#line  13182

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1042 =>
--#line  13195

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
   end;


when  1043 =>
--#line  13205

   declare
      Wrap1 : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  1044 =>
--#line  13219

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1045 =>
--#line  13229

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1046 =>
--#line  13242

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1047 =>
--#line  13252

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1048 =>
--#line  13265
 
yyval := 
yy.value_stack(yy.tos-1);

when  1049 =>
--#line  13270
 
yyval := 
yy.value_stack(yy.tos-1);

when  1050 =>
--#line  13275

   declare
      Wrap1 : constant Handled_Statements_Ptr :=
        Handled_Statements_Ptr (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Identifier (Wrap1.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  1051 =>
--#line  13287

   declare
      Wrap1 : constant Handled_Statements_Ptr :=
        Handled_Statements_Ptr (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos).all));
   end;


when  1052 =>
--#line  13301
 
yyval := 
yy.value_stack(yy.tos-1);

when  1054 =>
--#line  13310

   declare
      Wrap1 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-4));
      Wrap2 : constant Or_Path_Ptr :=
        new Or_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Guard (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_Pragmas (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_Sequence_Of_Statements (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1055 =>
--#line  13327

   declare
      Wrap1 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-3));
      Wrap2 : constant Or_Path_Ptr :=
        new Or_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Guard (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_Sequence_Of_Statements (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1056 =>
--#line  13343

   declare
      Wrap1 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-3));
      Wrap2 : constant Or_Path_Ptr :=
        new Or_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Pragmas (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_Sequence_Of_Statements (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1057 =>
--#line  13359

   declare
      Wrap1 : constant Primary_Path_Lists.List :=
        Primary_Path_Lists.List (
yy.value_stack(yy.tos-2));
      Wrap2 : constant Or_Path_Ptr :=
        new Or_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Sequence_Of_Statements (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1058 =>
--#line  13374

   declare
      Wrap1 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap2 : constant Or_Path_Ptr :=
        new Or_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Guard (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Set_Pragmas (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_Sequence_Of_Statements (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1059 =>
--#line  13391

   declare
      Wrap1 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap2 : constant Or_Path_Ptr :=
        new Or_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Guard (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_Sequence_Of_Statements (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1060 =>
--#line  13407

   declare
      Wrap1 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap2 : constant Or_Path_Ptr :=
        new Or_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Pragmas (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Set_Sequence_Of_Statements (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1061 =>
--#line  13423

   declare
      Wrap1 : constant Primary_Path_Lists.List :=
        new Primary_Path_Lists.List_Node;
      Wrap2 : constant Or_Path_Ptr :=
        new Or_Path_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Sequence_Of_Statements (Wrap2.all, 
yy.value_stack(yy.tos));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos).all));
      Primary_Path_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1062 =>
--#line  13441

   declare
      New_Node : constant Primary_Unit_Lists.List :=
        Primary_Unit_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Unit_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1063 =>
--#line  13451

   declare
      New_Node : constant Primary_Unit_Lists.List :=
        new Primary_Unit_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Unit_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1064 =>
--#line  13464

   declare
      New_Node : constant Primary_Clause_Lists.List :=
        Primary_Clause_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Clause_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1065 =>
--#line  13474

   declare
      New_Node : constant Primary_Clause_Lists.List :=
        new Primary_Clause_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Clause_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1066 =>
--#line  13487

   declare
      Wrap1 : constant Primary_Handler_Lists.List :=
        Primary_Handler_Lists.List (
yy.value_stack(yy.tos));
      Wrap2 : constant Exception_Handler_Ptr :=
        Exception_Handler_Ptr (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_Pragmas (Wrap2.all, 
yy.value_stack(yy.tos-2));
      Primary_Handler_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1067 =>
--#line  13502

   declare
      Wrap1 : constant Primary_Handler_Lists.List :=
        new Primary_Handler_Lists.List_Node;
      Wrap2 : constant Exception_Handler_Ptr :=
        Exception_Handler_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_Pragmas (Wrap2.all, 
yy.value_stack(yy.tos-1));
      Primary_Handler_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1068 =>
--#line  13518

   declare
      Wrap1 : constant Primary_Handler_Lists.List :=
        Primary_Handler_Lists.List (
yy.value_stack(yy.tos));
      Wrap2 : constant Exception_Handler_Ptr :=
        Exception_Handler_Ptr (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Primary_Handler_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1069 =>
--#line  13531

   declare
      Wrap1 : constant Primary_Handler_Lists.List :=
        new Primary_Handler_Lists.List_Node;
      Wrap2 : constant Exception_Handler_Ptr :=
        Exception_Handler_Ptr (
yy.value_stack(yy.tos));
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-1).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Primary_Handler_Lists.Add (Wrap1.all, T (Wrap2));
   end;


when  1070 =>
--#line  13548

   declare
      New_Node : constant Primary_Handler_Lists.List :=
        Primary_Handler_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Handler_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1071 =>
--#line  13558

   declare
      New_Node : constant Primary_Handler_Lists.List :=
        new Primary_Handler_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Handler_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1072 =>
--#line  13571
 
yyval := 
yy.value_stack(yy.tos-1);

when  1073 =>
--#line  13576

   declare
      New_Node : constant Primary_Choise_Lists.List :=
        Primary_Choise_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Choise_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1074 =>
--#line  13586

   declare
      New_Node : constant Primary_Choise_Lists.List :=
        new Primary_Choise_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Choise_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1075 =>
--#line  13599
 
yyval := 
yy.value_stack(yy.tos);

when  1076 =>
--#line  13604

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        Primary_Declaration_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1077 =>
--#line  13614

   declare
      New_Node : constant Primary_Declaration_Lists.List :=
        new Primary_Declaration_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Declaration_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1078 =>
--#line  13627

   declare
      New_Node : constant Primary_Association_Lists.List :=
        Primary_Association_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Association_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1079 =>
--#line  13637

   declare
      New_Node : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Association_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1080 =>
--#line  13650
 
yyval := 
yy.value_stack(yy.tos);

when  1081 =>
--#line  13655

   declare
      New_Node : constant Primary_Association_Lists.List :=
        Primary_Association_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Association_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1082 =>
--#line  13665

   declare
      New_Node : constant Primary_Association_Lists.List :=
        new Primary_Association_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Association_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1083 =>
--#line  13678

   declare
      Wrap1 : constant Generic_Association_Ptr :=
        new Generic_Association_Node;
      Wrap2 : constant Others_Choice_Ptr :=
        new Others_Choice_Node;
      Wrap3 : constant Box_Expression_Ptr :=
        new Box_Expression_Node;
   begin
      
yyval := YYSTYPE (Wrap1);
      Set_Start_Position (Wrap1.all, Start_Position (
yy.value_stack(yy.tos-3).all));
      Set_Start_Position (Wrap2.all, Start_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap2.all, End_Position (
yy.value_stack(yy.tos-2).all));
      Set_End_Position (Wrap1.all, End_Position (
yy.value_stack(yy.tos-1).all));
      Set_Start_Position (Wrap3.all, Start_Position (
yy.value_stack(yy.tos).all));
      Set_End_Position (Wrap3.all, End_Position (
yy.value_stack(yy.tos).all));
      Set_Actual_Parameter (Wrap1.all, T (Wrap3));
      Set_End_Position (Wrap1.all, End_Position (T (Wrap3).all));
      Set_Start_Position (Wrap1.all, Start_Position (T (Wrap2).all));
      Set_Formal_Parameter (Wrap1.all, T (Wrap2));
   end;


when  1084 =>
--#line  13704

   declare
      New_Node : constant Primary_Clause_Lists.List :=
        Primary_Clause_Lists.List (
yy.value_stack(yy.tos-1));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Clause_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1085 =>
--#line  13714

   declare
      New_Node : constant Primary_Clause_Lists.List :=
        new Primary_Clause_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Clause_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1086 =>
--#line  13727

   declare
      New_Node : constant Primary_Identifier_Lists.List :=
        Primary_Identifier_Lists.List (
yy.value_stack(yy.tos-2));
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Identifier_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1087 =>
--#line  13737

   declare
      New_Node : constant Primary_Identifier_Lists.List :=
        new Primary_Identifier_Lists.List_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Primary_Identifier_Lists.Add (New_Node.all, 
yy.value_stack(yy.tos));
   end;


when  1088 =>
--#line  13750

   declare
      New_Node : constant Pragma_Ptr := new Pragma_Node;
   begin
      
yyval := YYSTYPE (New_Node);
      Set_Start_Position
        (New_Node.all, (Get_Current_Line, Get_Current_Column - 1));
      Set_Pragma_Name_Image (New_Node.all, "Syntax_Error");
      Set_Pragma_Kind (New_Node.all, Asis.An_Implementation_Defined_Pragma);
      Set_End_Position
        (New_Node.all, (Get_Current_Line, Get_Current_Column - 1));
   end;


when  1089 =>
--#line  13766
 Modes.
yy.value_stack(yy.tos) := An_In_Mode; 

when  1090 =>
--#line  13767
 Modes.
yy.value_stack(yy.tos-1) := An_In_Out_Mode; 

when  1091 =>
--#line  13768
 Modes.
yy.value_stack(yy.tos) := An_Out_Mode; 

when  1092 =>
--#line  13772
 Last_Access_Kind := An_Access_To_Variable; 

when  1093 =>
--#line  13773
 Last_Access_Kind := An_Access_To_Constant; 

when  1094 =>
--#line  13777
 Last_Overriding_Kind := An_Indicator_of_Not_Overriding; 

when  1095 =>
--#line  13778
 Last_Overriding_Kind := An_Indicator_of_Overriding; 

                    when others => null;
                end case;


            -- Pop RHS states and goto next state
            yy.tos      := yy.tos - rule_length(yy.rule_id) + 1;
            if yy.tos > yy.stack_size then
                text_io.put_line(" Stack size exceeded on state_stack");
                raise yy_Tokens.syntax_error;
            end if;
            yy.state_stack(yy.tos) := goto_state(yy.state_stack(yy.tos-1) ,
                                 get_lhs_rule(yy.rule_id));

              yy.value_stack(yy.tos) := yyval;

            if yy.debug then
                reduce_debug(yy.rule_id,
                    goto_state(yy.state_stack(yy.tos - 1),
                               get_lhs_rule(yy.rule_id)));
            end if;

        end if;


    end loop;


end yyparse;

   begin
      Scanners.Initialize (Scanner, Source_Buffers.Buffer_Start (Input));
      Line.From := Source_Buffers.Buffer_Start (Input);
      YYParse;
      declare
         From    : Source_Buffers.Cursor;
         To      : Source_Buffers.Cursor;
      begin
         Scanners.Token_Span (Scanner, From, To);
         Line.To := From;
         Lines.Vectors.Add (Line_List, Line);
      end;
      Result := Last_Compilation;
   end Run;
end Asis.Gela.Parser;
